/*######################################################################
###
###   OBJECT LOADER HEADER FILE
###
######################################################################*/

#ifndef OBJLOADER_H
#define OBJLOADER_H

#include "./fruity_math.h"
#include "./fruity_exception.h"
#include "./opengl_includes.h"

extern cVector3f global_light ;
extern plane4f   groundPlane  ;



extern bool useEnv, 
            useGloss ,
            useLighting,
            useShadows  ,
            useQuickShadows  ,
            show_rays ;



extern bool initRenderer( void );


extern float _pAmbient[],
             _pDiffuse [],
             _pSpecular[],
             _pPosition[],
             _pZero    [];


//________________________________________________________________________________ Material Class

class material

  //
  //  Ripped off the old version's material class. This is a complete hack
  //  but good enough for the moment....
  //

  {

    public :


      //  GL_TEXTURE0_ARB
      char           TexPathA  [128] ;      // Texture Path
      GLuint         TexObjA         ;      // Resulting texture object

      //  GL_TEXTURE1_ARB
      char           TexPathB  [128] ;
      GLuint         TexObjB         ;



      //  Rendering booleans
      bool       Gloss        ,         // Render Gloss Map Pass
                 EnvMap       ;         // Environment map surface

      // Constructor
      material(void);

      // Destructor - Ensures Texture Objects Are Destroyed

  };


//________________________________________________________________________________ Vertex Class

class Vertex : public cVector3f

  //
  //     Derived from cVector3f to allow us to calculate 
  //  plane equations for faces easily
  //

  {
     public:
  
       Vertex( void ) : cVector3f() { inShadow = false; };

       inline Vertex& operator=( const cVector3f &v )
         {
           x = v.x; y = v.y; z = v.z;
           return *this;
         };

      friend bool operator==( const Vertex &V1 , const Vertex &V2  )
        {
          if( V1.x == V2.x &&
              V1.y == V2.y &&
              V1.z == V2.z )
          return true;
          return false;
        };

       bool inShadow;

  };




//________________________________________________________________________________ Normal Class

class Normal : public cVector3f

  //
  //     Derived from cVector3f to allow us to do 
  //  quick lighting calculations
  //

  {
     public:

       Normal( void ) : cVector3f() { };

  };


//________________________________________________________________________________ Tex Coord Class

class TexCoord

  //
  //  Contains u & v texturing co-ordinates
  //

  {
     public:

       float u;
       float v;
  };

//________________________________________________________________________________ Triangle Class


class Triangle : public plane4f

  //
  // Derived from 'plane4f' to allow us to create a bsp easily
  // also allows for lighting calculations , culling etc.
  //

  {
     public:

       inline void draw( void )
         {
           if( useLighting )
             {
                // Vertex 1
                if( v1->inShadow )
                  {
                    glColor3f( 0 , 0 , 0 );
                    glMaterialfv( GL_FRONT , GL_SPECULAR , _pZero );
                  }
                else
                  {
                    glMaterialfv( GL_FRONT , GL_SPECULAR , _pSpecular );
                    glColor3fv( _pDiffuse );
                  }
                glTexCoord2f( t1->u , t1->v );
                glNormal3f( n1->x , n1->y , n1->z );
                glVertex3f( v1->x , v1->y , v1->z );

                // Vertex 2;
                if( v2->inShadow )
                  {
                    glColor3f( 0 , 0 , 0 );
                    glMaterialfv( GL_FRONT , GL_SPECULAR , _pZero );
                  }
                else
                  {
                    glMaterialfv( GL_FRONT , GL_SPECULAR , _pSpecular );
                    glColor3fv( _pDiffuse );
                  }
                glTexCoord2f( t2->u , t2->v );
                glNormal3f( n2->x , n2->y , n2->z );
                glVertex3f( v2->x , v2->y , v2->z );

                // Vertex 3
                if( v3->inShadow )
                  {
                    glColor3f( 0 , 0 , 0 );
                    glMaterialfv( GL_FRONT , GL_SPECULAR , _pZero );
                  }
                else
                  {
                    glMaterialfv( GL_FRONT , GL_SPECULAR , _pSpecular );
                    glColor3fv( _pDiffuse );
                  }
                glTexCoord2f( t3->u , t3->v );
                glNormal3f( n3->x , n3->y , n3->z );
                glVertex3f( v3->x , v3->y , v3->z );
              }
            else
              {
//glColor3f( 1 , 1 , 1 );
                    // Vertex 1
                    glColor3f( angle1 , angle1 , angle1 );
                    glTexCoord2f( t1->u , t1->v );
                    glVertex3f( v1->x , v1->y , v1->z );

                    // Vertex 2;
                    glColor3f( angle2 , angle2 , angle2 );
                    glTexCoord2f( t2->u , t2->v );
                    glVertex3f( v2->x , v2->y , v2->z );

                    // Vertex 3
                    glColor3f( angle3 , angle3 , angle3 );
                    glTexCoord2f( t3->u , t3->v );
                    glVertex3f( v3->x , v3->y , v3->z );
              }
         }

       void setup     ( void );
       void calc_light( void );

       Triangle( void );


       TexCoord *t1;
       Normal   *n1;
       Vertex   *v1;

       TexCoord *t2;
       Normal   *n2;
       Vertex   *v2;

       TexCoord *t3;
       Normal   *n3;
       Vertex   *v3;

       uint      mat;
       uint      vinds[3];

       float     angle1 , 
                 angle2 ,
                 angle3 ;

       bool      facingAwayFromLight;

       Triangle *sibling;
       Triangle *child;
  };


//________________________________________________________________________________ Triangle Class


class objFile

  //
  //  Loads up, holds the contents , and draws an *.obj file with textures & stuff
  //

  {

     public:

       virtual bool load  ( char * );
       bool LoadMD( char * );
       virtual bool draw  ( void );
       bool calc_dynamic_shadows( objFile &scene );
       void draw_shadows  ( void );
       void calc_shadows  ( matrix m );
       void calc_shadows  ( void );
       void drawRays( void );

       ~objFile( void );
       objFile ( void );


       unsigned int    MaterialCount;
       material       *MaterialArray;



     protected:

       Vertex         *shadowPositions;

       unsigned int    vertexCount;       //  Total Number Of Verticies
       Vertex         *vertexArray;       //  Array Of Verticies

       unsigned int    normalCount;       //  Total Number Of Normals
       Normal         *normalArray;       //  Array Of Normals

       unsigned int    texCoordCount;     //  Total Number Of TexCoords
       TexCoord       *texCoordArray;     //  Array Of TexCoords

       unsigned int    triangleCount;
       Triangle       *triangleArray;

  };



#endif