/******************************************************************************/
/*                                                                            */
/*        Project: Boona Taxi                                                 */
/*          Title: Game Engine (GLUT callbacks)                               */
/*        Version: 0.2                                                        */
/*                                                                            */
/*        Started: 19/09/2000                                                 */
/*   Last Updated: 26/02/2001                                                 */
/*                                                                            */
/*         Author: Michael Bonnington                                         */
/*                                                                            */
/******************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <iostream.h>

#include "./opengl_includes.h"
#include "./font.h"
#include "./frametime.h"
#include "./callbacks.h"
#include "./car.h"
#include "./tga.h"
#include "./objloader.h"

#include "./fruity_exception.h"
#include "./fruity_math.h"



/* Constants =================================================================*/

/* Constants for game modes (add/remove/change as required)... */
#define TITLES 5
#define SELECT 0
#define OPTIONS 1
#define HIGHSCORES 2
#define CREDITS 3
#define EXIT 4
#define GAME 6
#define PAUSED 7

/* Constants for camera views... */
#define CHASE 0
#define COPTER 1
#define IN_CAR 2

//get rid of these when car class is sorted...
#define ETYPE 0
#define ESPRIT 1
#define PUG 2
#define BUG 3



/* Macros ====================================================================*/
#define resetView(a) (a).spin_x  = 0;\
                     (a).spin_y  = 0;\
                     (a).dolly   = 0;\
                     (a).trans_x = 0;\
                     (a).trans_y = 0;



/* Function prototypes =======================================================*/
void changeGameMode(int);

/* Drawing functions... */
void drawCity(void);
void drawOverlay(void);
void drawHUD(void);

/* Game mode-specific GLUT callback functions... */
void idleTitles(void);
void displayTitles(void);

void idleSelect(void);
void displaySelect(void);

void idleOptions(void);
void displayOptions(void);

void idleHighscores(void);
void displayHighscores(void);

void idleCredits(void);
void displayCredits(void);

void idleGame(void);
void displayGame(void);

void idlePaused(void);
void displayPaused(void);

/* General GLUT callback functions... */
void reshape(int, int);
void keyDown(unsigned char, int, int);
void keyUp(unsigned char, int, int);
void specialDown(int, int, int);
void specialUp(int, int, int);
void mouse(int, int, int, int);
void motion(int, int);



/* Globals ===================================================================*/
/*##############################################################################
###                            EXTERNAL GLOBALS
##############################################################################*/



/*------------------------  Engine Requires These  ---------------------------*/

/* Structure to hold window information, including size and position... */
struct window win;

/*----------------------------------------------------------------------------*/

//not sure what this was for
//extern int WIN_MAT;





/* Globals ===================================================================*/
int current_car = ETYPE;   //remove car when I have car class
objFile Thingy;

/*------------------------  Engine Requires These  ---------------------------*/

// USED TO HOLD SET UP OF ORTHOGRAPHIC AND PERSPECTIVE VIEWS
GLuint persp, ortho;

char statusLine[256];   //line of text to display messages on-screen

/*----------------------------------------------------------------------------*/

car player(0, 0, 0);
camera cam;




//cModel city;   //temporary



/* Mikes Display Lists */
//GLuint overlay, hud;

/* Global data structures (defined in callbacks.h header file)... */
game boona;


GLuint logoTexture , shitTexture;


bool keys[128], specials[128];










/* GLUT callback registration ================================================*/
void registerCallbacks(void)
{
   GLfloat fog_colour[4] = {0.46667, 0.73333, 1.0, 1.0};

   printf("Initialising OpenGL:\n  Vendor: %s\nRenderer: %s\n Version: %s\n\n",
          (char *)glGetString(GL_VENDOR),
          (char *)glGetString(GL_RENDERER),
          (char *)glGetString(GL_VERSION));

   /* Initialise OpenGL/GLUT stuff... */
   glEnable(GL_CULL_FACE);
   glCullFace(GL_BACK);

   //temporary fog setup...
   glClearColor(0.46667, 0.73333, 1.0, 1.0);
   glFogi(GL_FOG_MODE, GL_LINEAR);
   glFogfv(GL_FOG_COLOR, fog_colour);
   glFogf(GL_FOG_DENSITY, 0.05);
   glFogf(GL_FOG_END, 200);
   //glEnable(GL_FOG);

   glLightModelf( GL_LIGHT_MODEL_LOCAL_VIEWER , GL_TRUE ); 


   glutIgnoreKeyRepeat(1);
   glutSetCursor(GLUT_CURSOR_NONE);

   /* Initialise frame counter... */
   if(!initFrameCounter())
   {
      printf("ERROR: Unable to initialise frame counter.\n\n");
      changeGameMode(EXIT);
   }

   makeRasterFont();   //initialise font

   initRenderer();
   InitErrorLog();   //look at this later

   /* Load the models... */
   //player.LoadModel("./models/etype.obj");
   //city.LoadModel("./models/city.obj");

   /* Load textures not associated with models... */
   loadTexture("./textures/background_logo.tga", &logoTexture);
   loadTexture("./textures/reflect_old.tga", &shitTexture);
   //loadTGA("./textures/etype.tga", 200); //this is TEMP until materials work

   Thingy.load( "./models/city.obj" );

   Thingy.LoadMD( "./models/city.mdf" );



   player.load( "./models/etype.obj" );
   player.LoadMD( "./models/etype.mdf" );


   boona.option = SELECT;

   /* Register GLUT callback functions... */
   glutReshapeFunc(reshape);
   glutKeyboardFunc(keyDown);
   glutKeyboardUpFunc(keyUp);
   glutSpecialFunc(specialDown);
   glutSpecialUpFunc(specialUp);
   changeGameMode(GAME);


   glClearStencil( 0 );

   glutPostRedisplay();
}



/* Function to change current game mode ======================================*/
void changeGameMode(int mode)
{
   switch(mode)
   {
      case TITLES:
         boona.game_mode = TITLES;
         glutIdleFunc(idleTitles);
         glutDisplayFunc(displayTitles);
         glutMouseFunc(NULL);
         glutMotionFunc(NULL);
         break;

      case SELECT:
         boona.game_mode = SELECT;
         resetView(boona);
         glutIdleFunc(idleSelect);
         glutDisplayFunc(displaySelect);
         glutMouseFunc(mouse);
         glutMotionFunc(motion);
         break;

      case OPTIONS:
         boona.game_mode = OPTIONS;
         glutIdleFunc(idleOptions);
         glutDisplayFunc(displayOptions);
         glutMouseFunc(NULL);
         glutMotionFunc(NULL);
         break;

      case HIGHSCORES:
         boona.game_mode = HIGHSCORES;
         glutIdleFunc(idleHighscores);
         glutDisplayFunc(displayHighscores);
         glutMouseFunc(NULL);
         glutMotionFunc(NULL);
         break;

      case CREDITS:
         boona.game_mode = CREDITS;
         glutIdleFunc(idleCredits);
         glutDisplayFunc(displayCredits);
         glutMouseFunc(NULL);
         glutMotionFunc(NULL);
         break;

      case EXIT:   //a little bit suspect at the moment with the game mode shit...
         boona.game_mode = EXIT;
         //save configuration files and highscores here
         //seems to crash on exit when using game mode under Windows 2000...
         if(glutGameModeGet(GLUT_GAME_MODE_ACTIVE)) glutLeaveGameMode();
         exit(0);
         break;

      case GAME:
         boona.game_mode = GAME;
         resetView(boona);
         boona.game_time = 300;   //allows 5 mins play - temporary assignment
         glutIdleFunc(idleGame);
         glutDisplayFunc(displayGame);
         glutMouseFunc(NULL);
         glutMotionFunc(NULL);
         break;

      case PAUSED:
         boona.game_mode = PAUSED;
         glutIdleFunc(idlePaused);
         glutDisplayFunc(displayPaused);
         glutMouseFunc(mouse);
         glutMotionFunc(motion);
         break;

      default:
         break;
   }
   glutPostRedisplay();
}



/* Function to draw presentation screen overlay graphics =====================*/
void drawOverlay(void)
{
   /* Make sure the orthographic display list has been initialised with
    * 'glCallList(persp);'
    * before trying to call this function.
    */

   glBegin(GL_QUADS);
    glColor3f(0.93333, 0.66667, 0.0);
    glVertex2f(0, win.height-44);
    glVertex2f(0, win.height);
    glVertex2f(win.width, win.height);
    glVertex2f(win.width, win.height-44);

    glColor3f(1.0, 0.8, 0.06667);
    glVertex2f(0, win.height-128);
    glVertex2f(0, win.height-44);
    glVertex2f(win.width, win.height-44);
    glVertex2f(win.width, win.height-128);

    glVertex2f(0, 0);
    glVertex2f(0, 64);
    glVertex2f(win.width, 64);
    glVertex2f(win.width, 0);
   glEnd();

   smallFont("www.boonaracer.co.uk/taxi", 16, win.height-100, 0.0, 0.0, 0.0);
   smallFont(statusLine, 16, win.height-16, 0.0, 0.0, 0.0);
}

/* Function to draw heads up display =========================================*/
void drawHUD(void)
{
   /* Make sure the orthographic display list has been initialised with
    * 'glCallList(persp);'
    * before trying to call this function.
    */

   //try putting some of this in display lists to speed it up...

   char info_text[8];

   sprintf(info_text, "$%4.2f", player.getScore());
   smallFont("Fare", 16, 28, 1.0, 1.0, 1.0);
   largeOutlineFont(info_text, 16, 60, 1.0, 0.8, 0.06667);

   sprintf(info_text, "%d", (int)boona.game_time);
   smallFont("Time", win.width-96, 28, 1.0, 1.0, 1.0);
   largeOutlineFont(info_text, win.width-96, 60, 1.0, 0.8, 0.06667);

   sprintf(info_text, "%d", (int)(player.getSpeed()*10));
   smallFont("Speed", win.width-96, win.height-48, 1.0, 1.0, 1.0);
   largeOutlineFont(info_text, win.width-96, win.height-16, 1.0, 0.8, 0.06667);

   if(player.reverse)   //Are we in reverse?
      largeOutlineFont("R", win.width-32, win.height-16, 1.0, 0.6, 0.0);
   else                 //Are we in drive?
      largeOutlineFont("D", win.width-32, win.height-16, 0.33333, 0.8, 0.33333);

   //draw rotating map here
}

/* Interpolate camera position ===============================================*/
void interpolateCam(float new_pos_x, float new_pos_y, float new_pos_z,
                    float new_aim_x, float new_aim_y, float new_aim_z,
                    float amount)
{
   cam.pos_x = cam.old_pos_x*(1-amount) + new_pos_x*amount;
   cam.pos_y = cam.old_pos_y*(1-amount) + new_pos_y*amount;
   cam.pos_z = cam.old_pos_z*(1-amount) + new_pos_z*amount;
   cam.pos_x = cam.old_aim_x*(1-amount) + new_aim_x*amount;
   cam.pos_y = cam.old_aim_y*(1-amount) + new_aim_y*amount;
   cam.pos_z = cam.old_aim_z*(1-amount) + new_aim_z*amount;
}

/* Update and interpolate camera position ====================================*/
void updateCam(void)
{
   float new_pos_x, new_pos_y, new_pos_z, new_aim_x, new_aim_y, new_aim_z;

   //not needed at present...
   /* cam.distance = sqrt((cam.aim_x - cam.pos_x)*(cam.aim_x - cam.pos_x) +
                          (cam.aim_z - cam.pos_z)*(cam.aim_z - cam.pos_z)); */

   //these values need tweaking...
   switch(boona.cam_view)
   {
      case IN_CAR:
         new_pos_x = player.pos_x;
         new_pos_y = player.pos_y + 0.85;
         new_pos_z = player.pos_z;
         new_aim_x = player.pos_x + player.dx;
         new_aim_y = player.pos_y + 0.85;
         new_aim_z = player.pos_z + player.dz;
         break;

      case CHASE:
         new_pos_x = player.pos_x - player.dx*4;
         new_pos_y = player.pos_y + 1.6;
         new_pos_z = player.pos_z - player.dz*4;
         new_aim_x = player.pos_x;
         new_aim_y = player.pos_y + 1.2;
         new_aim_z = player.pos_z;
         break;

      case COPTER:
         new_pos_x = player.pos_x - player.dx*4;
         new_pos_y = player.pos_y + 4;
         new_pos_z = player.pos_z - player.dz*4;
         new_aim_x = player.pos_x;
         new_aim_y = player.pos_y + 2;
         new_aim_z = player.pos_z;
         break;

      default:
         break;
   }

/* The next six lines take the new position and aiming coordinates for the
 * camera and smoothly interpolates from its current position to the new
 * position over a period of time. Only linear interpolation at the moment -
 * could be re-written to use quadratic or cubic interpolation for a smoother
 * effect...
 */

   cam.pos_x = cam.old_pos_x*(1-boona.counter) + new_pos_x*boona.counter;
   cam.pos_y = cam.old_pos_y*(1-boona.counter) + new_pos_y*boona.counter;
   cam.pos_z = cam.old_pos_z*(1-boona.counter) + new_pos_z*boona.counter;
   cam.aim_x = cam.old_aim_x*(1-boona.counter) + new_aim_x*boona.counter;
   cam.aim_y = cam.old_aim_y*(1-boona.counter) + new_aim_y*boona.counter;
   cam.aim_z = cam.old_aim_z*(1-boona.counter) + new_aim_z*boona.counter;
}

/* ===========================================================================*/
void positionCam(void)
{
   switch(boona.cam_view)
   {
      case IN_CAR:
         cam.pos_x = player.pos_x;
         cam.pos_y = player.pos_y + 0.85;
         cam.pos_z = player.pos_z;
         cam.aim_x = player.pos_x + player.dx;
         cam.aim_y = player.pos_y + 0.85;
         cam.aim_z = player.pos_z + player.dz;
         break;

      case CHASE:
         cam.pos_x = player.pos_x - player.dx*4;
         cam.pos_y = player.pos_y + 1.6;
         cam.pos_z = player.pos_z - player.dz*4;
         cam.aim_x = player.pos_x;
         cam.aim_y = player.pos_y + 1.2;
         cam.aim_z = player.pos_z;
         break;

      case COPTER:
         cam.pos_x = player.pos_x - player.dx*4;
         cam.pos_y = player.pos_y + 4;
         cam.pos_z = player.pos_z - player.dz*4;
         cam.aim_x = player.pos_x;
         cam.aim_y = player.pos_y + 2;
         cam.aim_z = player.pos_z;
         break;

      default:
         break;
   }
}



/*********************** ******************************************************/
/* TITLES              * * game mode-specific idle/display callback functions */
/*********************** ******************************************************/

/* Idle function (Title screen) ==============================================*/
void idleTitles(void)
{
   if(specials[GLUT_KEY_LEFT])
   {
      if(boona.option > SELECT) boona.option--;
      else boona.option = EXIT;
      specials[GLUT_KEY_LEFT] = false;
   }
   else
   if(specials[GLUT_KEY_RIGHT])
   {
      if(boona.option < EXIT) boona.option++;
      else boona.option = SELECT;
      specials[GLUT_KEY_RIGHT] = false;
   }
   else
   if(keys[13])
   {
      changeGameMode( boona.option );
      keys[13] = false;
   }
   else
   if(keys[27])
   {
      if(boona.option == EXIT) changeGameMode(EXIT);
      else boona.option = EXIT;   //don't exit immediately
      keys[27] = false;
   }
}

/* Display function (Title screen) ===========================================*/
void displayTitles(void)
{
   glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
   glLoadIdentity();

   glCallList(ortho);

   drawOverlay();
   largeFont("Boona Taxi Title Screen", 16, 48, 0.93333, 0.66667, 0.0);
   largeFont("<          >", 0, win.height-60, 0.93333, 0.66667, 0.0);

   switch(boona.option)
   {
      case SELECT:
         largeOutlineFont("Begin Game", 16, win.height-60, 1.0, 1.0, 1.0);
         sprintf(statusLine, "Press 'ENTER' to begin a new game; 'LEFT' or 'RIGHT' to select another option");
         break;

      case OPTIONS:
         largeOutlineFont("Options", 16, win.height-60, 1.0, 1.0, 1.0);
         sprintf(statusLine, "Modify the game settings");
         break;

      case HIGHSCORES:
         largeOutlineFont("Highscores", 16, win.height-60, 1.0, 1.0, 1.0);
         sprintf(statusLine, "See the ten best Boona Taxi drivers");
         break;

      case CREDITS:
         largeOutlineFont("Credits", 16, win.height-60, 1.0, 1.0, 1.0);
         sprintf(statusLine, "People who worked on Boona Taxi");
         break;

      case EXIT:
         largeOutlineFont("Exit Game", 16, win.height-60, 1.0, 1.0, 1.0);
         sprintf(statusLine, "Press 'ENTER' or 'ESC' to exit Boona Taxi");
         break;

      default:
         break;
   }

   glutSwapBuffers();
}

/******************************************************************************/
/* end of TITLES                                                              */
/******************************************************************************/



/*********************** ******************************************************/
/* SELECT              * * game mode-specific idle/display callback functions */
/*********************** ******************************************************/

/* Idle function (Car select screen) =========================================*/
void idleSelect(void)
{
   double frametime = getFrametime();

   if(specials[GLUT_KEY_LEFT])
   {
      if(current_car > ETYPE) current_car--;
      else current_car = BUG;
      specials[GLUT_KEY_LEFT] = false;
   }

   if(specials[GLUT_KEY_RIGHT])
   {
      if(current_car < BUG) current_car++;
      else current_car = ETYPE;
      specials[GLUT_KEY_RIGHT] = false;
   }

   if(keys[13])
   {
      //player can select etype only...
      if(current_car == ETYPE) changeGameMode(GAME);
      else sprintf(statusLine, "Sorry, you can only pick the E-Type Jag");
      keys[13] = false;
   }

   if(keys[27])
   {
      changeGameMode(TITLES);
      keys[27] = false;
   }

   /* Calculate scrolling background position... */
   if(boona.counter < 1-frametime/4) boona.counter+=frametime/4;
   else boona.counter = 0;

   /* Calculate car rotation... */
   boona.spin_y += frametime*16;

   glutPostRedisplay();
}

/* Display function (Car select screen) ======================================*/
void displaySelect(void)
{
   glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
   glLoadIdentity();

   /* Draw scrolling background... */
   glCallList(ortho);
   glEnable(GL_TEXTURE_2D);
   glBindTexture(GL_TEXTURE_2D, logoTexture);
   glBegin(GL_QUADS);
   glTexCoord2f(0+boona.counter, 1.5+boona.counter);
   glVertex2f(0, 64);
   glTexCoord2f(0+boona.counter, 0+boona.counter);
   glVertex2f(0, win.height-128);
   glTexCoord2f(2.5+boona.counter, 0+boona.counter);
   glVertex2f(win.width, win.height-128);
   glTexCoord2f(2.5+boona.counter, 1.5+boona.counter);
   glVertex2f(win.width, 64);
   glEnd();

   /* Draw all the 3D stuff... */
   glCallList(persp);

   glPushMatrix();
    glColor3f(1.0, 1.0, 1.0);
    gluLookAt(0, 0.8, 2.6, 0, 0, 0, 0, 1, 0);

    glPushMatrix();
     glTranslatef(boona.trans_x, boona.trans_y, boona.dolly);
     glRotatef(boona.spin_x, 1.0, 0.0, 0.0);

     glPushMatrix();
      glRotatef(boona.spin_y, 0.0, 1.0, 0.0);
      //glBindTexture(GL_TEXTURE_2D, 200); //TEMP
      player.draw();   //Draw the car model
     glPopMatrix();

    glPopMatrix();

   glPopMatrix();

   /* Draw all the 2D stuff on top of the 3D... */
   glCallList(ortho);

   drawOverlay();
   largeFont("Boona Taxi Vehicle Select Screen", 16, 48, 0.93333, 0.66667, 0.0);
   largeFont("<            >", 0, win.height-60, 0.93333, 0.66667, 0.0);

   //change this when I have a proper car class sorted out...
   switch(current_car)
   {
      case ETYPE:
         //player.LoadModel("./models/etype.obj");
         largeOutlineFont("E-type Jag", 16, win.height-60, 1.0, 1.0, 1.0);
         sprintf(statusLine, "E-type Jag");
         break;

      case ESPRIT:
         //player.LoadModel("./models/esprit.obj");
         largeOutlineFont("Lotus Esprit", 16, win.height-60, 1.0, 1.0, 1.0);
         sprintf(statusLine, "Lotus Esprit");
         break;

      case PUG:
         //player.LoadModel("./models/peugeot.obj");
         largeOutlineFont("Peugeot 406", 16, win.height-60, 1.0, 1.0, 1.0);
         sprintf(statusLine, "Peugeot 406");
         break;

      case BUG:
         //player.LoadModel("./models/bug.obj");
         largeOutlineFont("Bad Boy Bug", 16, win.height-60, 1.0, 1.0, 1.0);
         sprintf(statusLine, "Bad Boy Bug");
         break;

      default:
         break;
   }

   glutSwapBuffers();
}

/******************************************************************************/
/* end of SELECT                                                              */
/******************************************************************************/



/*********************** ******************************************************/
/* OPTIONS             * * game mode-specific idle/display callback functions */
/*********************** ******************************************************/

/* Idle function (Options screen) ============================================*/
void idleOptions(void)
{
   if(keys[13] || keys[27])
   {
      changeGameMode(TITLES);
      keys[13] = false;
      keys[27] = false;
   }

   glutPostRedisplay();
}

/* Display function (Options screen) =========================================*/
void displayOptions(void)
{
   glCallList(ortho);

   drawOverlay();
   largeFont("Boona Taxi Options Screen", 16, 48, 0.93333, 0.66667, 0.0);
   largeOutlineFont("Options", 16, win.height-60, 1.0, 1.0, 1.0);
   sprintf(statusLine, "Press 'ENTER' or 'ESC' to return");

   glutSwapBuffers();
}

/******************************************************************************/
/* end of OPTIONS                                                             */
/******************************************************************************/



/*********************** ******************************************************/
/* HIGHSCORES          * * game mode-specific idle/display callback functions */
/*********************** ******************************************************/

/* Idle function (Highscores screen) =========================================*/
void idleHighscores(void)
{
   if(keys[13] || keys[27])
   {
      changeGameMode(TITLES);
      keys[13] = false;
      keys[27] = false;
   }

   glutPostRedisplay();
}

/* Display function (Highscores screen) ======================================*/
void displayHighscores(void)
{
   glCallList(ortho);

   drawOverlay();
   largeFont("Boona Taxi Highscores Screen", 16, 48, 0.93333, 0.66667, 0.0);
   largeOutlineFont("Highscores", 16, win.height-60, 1.0, 1.0, 1.0);
   sprintf(statusLine, "Press 'ENTER' or 'ESC' to return");

   glutSwapBuffers();
}

/******************************************************************************/
/* end of HIGHSCORES                                                          */
/******************************************************************************/



/*********************** ******************************************************/
/* CREDITS             * * game mode-specific idle/display callback functions */
/*********************** ******************************************************/

/* Idle function (Credits screen) ============================================*/
void idleCredits(void)
{
   if(keys[13] || keys[27])
   {
      changeGameMode(TITLES);
      keys[13] = false;
      keys[27] = false;
   }

   glutPostRedisplay();
}

/* Display function (Credits screen) =========================================*/
void displayCredits(void)
{
   glCallList(ortho);

   drawOverlay();
   largeFont("Boona Taxi Credits Screen", 16, 48, 0.93333, 0.66667, 0.0);
   largeOutlineFont("Credits", 16, win.height-60, 1.0, 1.0, 1.0);
   sprintf(statusLine, "Press 'ENTER' or 'ESC' to return");

   smallFont("Written by Michael Bonnington",         32,  96, 0.0, 0.0, 0.0);
   smallFont("Enormous thanks to Rob Bateman",        32, 112, 0.0, 0.0, 0.0);
   smallFont("and the rest of the Boona Racer team:", 32, 128, 0.0, 0.0, 0.0);
   smallFont("Arthur Yarwood, Eike Anderson, JP Li",  32, 144, 0.0, 0.0, 0.0);

   glutSwapBuffers();
}

/******************************************************************************/
/* end of CREDITS                                                             */
/******************************************************************************/



/*********************** ******************************************************/
/* GAME                * * game mode-specific idle/display callback functions */
/*********************** ******************************************************/

/* Idle function (Game) ======================================================*/
void idleGame(void)
{
   double frametime = getFrametime();

   if(keys['p'] || keys['P'] || keys[27])
   {
      changeGameMode(PAUSED);
      keys['p'] = false;
      keys['P'] = false;
      keys[27] = false;
   }

   if(specials[GLUT_KEY_F3])
   {
      if(boona.heads_up_display) boona.heads_up_display = false;
      else boona.heads_up_display = true;
      specials[GLUT_KEY_F3] = false;
   }

   if(specials[GLUT_KEY_F4])   //Change camera view
   {
      boona.counter = 0;
      cam.old_pos_x = cam.pos_x;
      cam.old_pos_y = cam.pos_y;
      cam.old_pos_z = cam.pos_z;
      cam.old_aim_x = cam.aim_x;
      cam.old_aim_y = cam.aim_y;
      cam.old_aim_z = cam.aim_z;
      if(boona.cam_view < IN_CAR) boona.cam_view++;
      else boona.cam_view = CHASE;
      specials[GLUT_KEY_F4] = false;
   }

   if(specials[GLUT_KEY_F5])
   {
      boona.top_down_view = !boona.top_down_view;
      specials[GLUT_KEY_F5] = false;
   }

   /* Update player position... */
   player.update(frametime);

   /* Update camera position... */
   if(boona.counter < 1-frametime*2)
   {
      boona.counter+=frametime*2;
      updateCam();
   }
   else
   {
      boona.counter = 1;
      positionCam();
   }

   /* Update game time... */
   if(boona.game_time > 0) boona.game_time -= frametime;
   else
   {
      boona.game_time = 0;
      changeGameMode(TITLES);
   }

   sprintf(statusLine, "Frames/second: %6.2f", (float)1/frametime);
   //printf("dx: %f, dz: %f, heading: %f\n", player.dx, player.dz, player.getHeading());

   glutPostRedisplay();
}

/* Display function (Game) ===================================================*/
void displayGame(void)
{
   
   DebugLog( "started displaygame" );


   glClear( GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT );
   /* Draw all the 3D stuff... */
   glCallList(persp);

   glLoadIdentity();


   glPushMatrix();
    glColor3f(1.0, 1.0, 1.0);
    glDisable( GL_TEXTURE_2D );
    if(boona.top_down_view)
    {
       gluLookAt(player.pos_x, player.pos_y+40, player.pos_z,
                 player.pos_x, player.pos_y   , player.pos_z,
                 1, 0, 0);
    }
    else
    {
       gluLookAt(cam.pos_x, cam.pos_y, cam.pos_z,
                 cam.aim_x, cam.aim_y, cam.aim_z,
                 0, 1, 0);
    }

    glLightfv( GL_LIGHT0 , GL_AMBIENT  , _pAmbient  );
    glLightfv( GL_LIGHT0 , GL_AMBIENT  , _pDiffuse  );
    glLightfv( GL_LIGHT0 , GL_AMBIENT  , _pSpecular );
    glLightfv( GL_LIGHT0 , GL_POSITION , _pPosition );

    glEnable( GL_TEXTURE_2D );

    glPushMatrix();
   
      //draw big quad on the ground so we can see what's going on...
      glColor3f(0.4, 0.4, 0.4);
      glBindTexture(GL_TEXTURE_2D, logoTexture);
      glBegin(GL_QUADS);
       glTexCoord2f(0, 8);
       glVertex3f(-128, -0.01, -128);
       glTexCoord2f(0, 0);
       glVertex3f(-128, -0.01,  128);
       glTexCoord2f(8, 0);
       glVertex3f( 128, -0.01,  128);
       glTexCoord2f(8, 8);
       glVertex3f( 128, -0.01, -128);
      glEnd();

      glPushMatrix();
        glDisable( GL_TEXTURE_2D );
        glCullFace( GL_FRONT );
        Thingy.draw_shadows();
        glCullFace( GL_BACK );
        glEnable( GL_TEXTURE_2D );
      glPopMatrix();


      glEnable(GL_FOG);
       Thingy.draw();
       useLighting = true;
       player.draw();
       useLighting = false;
      glDisable(GL_FOG);

      glDisable( GL_TEXTURE_2D );

      if( show_rays ) Thingy.drawRays();

    glPopMatrix();

   glPopMatrix();

    //drawOthers();
   DebugLog( "draw player" );


   DebugLog( "draw ground" );

   /* Draw all the 2D stuff on top of the 3D... */
   glCallList(ortho);
   glLoadIdentity();
   smallFont(statusLine, 16, win.height-16, 1.0, 1.0, 1.0);
   if(boona.heads_up_display) drawHUD();
   DebugLog( "draw ortho stuff" );

   glutSwapBuffers();
   DebugLog( "finished displaygame" );
}

/******************************************************************************/
/* end of GAME                                                                */
/******************************************************************************/



/******************************************************************************/
/* PAUSED game mode-specific GLUT callback function:                          */
/*        idle, display                                                       */
/******************************************************************************/

/* Idle function (Paused screen) =============================================*/
void idlePaused(void)
{
   if(keys['p'] || keys['P'] || keys[13])
   {
      changeGameMode(GAME);
      keys['p'] = false;
      keys['P'] = false;
      keys[13] = false;
   }

   if(keys[27])
   {
      changeGameMode(TITLES);
      keys[27] = false;
   }

   sprintf(statusLine, "Press 'P' to resume or 'ESC' to quit");

   glutPostRedisplay();
}

/* Display function (Paused screen) ==========================================*/
void displayPaused(void)
  {

    DebugLog( "started displayPaused" );


    glClear( GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT );
    /* Draw all the 3D stuff... */
    glCallList(persp);


    glPushMatrix();
      glColor3f(1.0, 1.0, 1.0);
      glTranslatef(boona.trans_x, boona.trans_y, boona.dolly);
      glRotatef(boona.spin_x, 1.0, 0.0, 0.0);
  
      glRotatef(boona.spin_y, 0.0, 1.0, 0.0);
  
      gluLookAt(cam.pos_x, cam.pos_y, cam.pos_z,
                cam.aim_x, cam.aim_y, cam.aim_z,
                0, 1, 0);

      glPushMatrix();
   
        //draw big quad on the ground so we can see what's going on...
        glColor3f(0.4, 0.4, 0.4);
        glBindTexture(GL_TEXTURE_2D, logoTexture);
        glBegin(GL_QUADS);
         glTexCoord2f(0, 8);
         glVertex3f(-128, -0.01, -128);
         glTexCoord2f(0, 0);
         glVertex3f(-128, -0.01,  128);
         glTexCoord2f(8, 0);
         glVertex3f( 128, -0.01,  128);
         glTexCoord2f(8, 8);
         glVertex3f( 128, -0.01, -128);
        glEnd();

        glPushMatrix();
          glDisable( GL_TEXTURE_2D );
          glCullFace( GL_FRONT );
          Thingy.draw_shadows();
          glCullFace( GL_BACK );
          glEnable( GL_TEXTURE_2D );
        glPopMatrix(); 

      //  glEnable(GL_FOG);
          Thingy.draw();
          player.draw();
       // glDisable(GL_FOG);

       Thingy.drawRays();

      glPopMatrix();

    glPopMatrix();

     /* Draw all the 2D stuff on top of the 3D... */
    glCallList(ortho);
    drawOverlay();
    largeFont("Boona Taxi Paused Screen", 16, 48, 0.93333, 0.66667, 0.0);
    largeOutlineFont("Game Paused", 16, win.height-60, 1.0, 1.0, 1.0);

    glutSwapBuffers();
  }

/******************************************************************************/
/* end of PAUSED                                                              */
/******************************************************************************/



/* Reshape function ==========================================================*/
void reshape(int w, int h)
{
   if(h == 0) h = 1;   //Required to prevent crash when window is minimised

   win.width = w;
   win.height = h;


   glDeleteLists(persp, 1);
   glDeleteLists(ortho, 1);
   persp = glGenLists(1);
   ortho = glGenLists(1);

   glNewList(persp, GL_COMPILE);
    glViewport(0, 0, (GLsizei) w, (GLsizei) h);
    glMatrixMode(GL_PROJECTION);
    glLoadIdentity();
    gluPerspective(75.0, (GLfloat) w/(GLfloat) h, 0.1, 256);
    glMatrixMode(GL_MODELVIEW);
    glLoadIdentity();
    glEnable(GL_DEPTH_TEST);
    glEnable(GL_TEXTURE_2D);
   glEndList();

   glNewList(ortho, GL_COMPILE);
    glViewport(0, 0, (GLsizei) w, (GLsizei) h);
    glMatrixMode(GL_PROJECTION);
    glLoadIdentity();
    glOrtho(0, w, h, 0, -1, 1);
    glMatrixMode(GL_MODELVIEW);
    glLoadIdentity();
    glDisable(GL_DEPTH_TEST);
    glDisable(GL_TEXTURE_2D);
   glEndList();


   glCallList(persp);
}

/* Keyboard function - key is pressed ========================================*/
void keyDown(unsigned char key, int x, int y)
{
   keys[key] = true;

   if(boona.game_mode == GAME)
   {
      switch(key)
      {
         case 'z':
            player.brake = true;
            break;

         case 'x':
            player.accelerator = true;
            break;

         case ' ':
            player.handbrake = true;
            break;

         default:
            break;
      }
   }
}

/* Keyboard function - key is released =======================================*/
void keyUp(unsigned char key, int x, int y)
{
   keys[key] = false;

   if(boona.game_mode == GAME)
   {
      switch(key)
      {
         case 'z':
            player.brake = false;
            break;

         case 'x':
            player.accelerator = false;
            break;

         case ' ':
            player.handbrake = false;
            break;

         default:
            break;
      }
   }
}

/* Keyboard function - special key is pressed ================================*/
void specialDown(int key, int x, int y)
{
   specials[key] = true;

   if(boona.game_mode == GAME)
   {
      switch(key)
      {
         case GLUT_KEY_LEFT:
            player.left  = true;
            player.right = false;
            break;

         case GLUT_KEY_RIGHT:
            player.right = true;
            player.left  = false;
            break;

         case GLUT_KEY_UP:
            player.reverse = false;
            break;

         case GLUT_KEY_DOWN:
            player.reverse = true;
            break;

         default:
            break;
      }
   }
}

/* Keyboard function - special key is released ===============================*/
void specialUp(int key, int x, int y)
{
   specials[key] = false;

   if(boona.game_mode == GAME)
   {
      switch(key)
      {
         case GLUT_KEY_LEFT:
            player.left = false;
            break;

         case GLUT_KEY_RIGHT:
            player.right = false;
            break;

         default:
            break;
      }
   }
}

/* Mouse input function ======================================================*/
void mouse(int button, int state, int x, int y)
{
   if(state == GLUT_DOWN)
   {
      boona.mouse_press_x = x; boona.mouse_press_y = y;

      switch(button)
      {
         case GLUT_LEFT_BUTTON:
            boona.lmb = true;
            break;

         case GLUT_MIDDLE_BUTTON:
            boona.mmb = true;
            break;

         default:
            break;
      }
   }

   if(state == GLUT_UP)
   {
      glutSetCursor(GLUT_CURSOR_NONE);
      switch(button)
      {
         case GLUT_LEFT_BUTTON:
            boona.lmb = false;
            break;

         case GLUT_MIDDLE_BUTTON:
            boona.mmb = false;
            break;

         default:
            break;
      }
   }

   glutPostRedisplay();
}

/* Motion function ===========================================================*/
void motion(int x, int y)
{
   int step_x = (x - boona.mouse_press_x), step_y = (y - boona.mouse_press_y);

   boona.mouse_press_x = x; boona.mouse_press_y = y;

   if(boona.lmb && !boona.mmb)        /* Tumble */
   {
#ifdef WIN32
      glutSetCursor(GLUT_CURSOR_CROSSHAIR);
#else
      glutSetCursor(GLUT_CURSOR_CYCLE);
#endif
      boona.spin_x += 0.5*step_y;
      boona.spin_y += 0.5*step_x;
   }
   else if(boona.mmb && !boona.lmb)   /* Translate */
   {
#ifdef WIN32
      glutSetCursor(GLUT_CURSOR_SPRAY);
#else
      glutSetCursor(GLUT_CURSOR_CROSSHAIR);
#endif
      boona.trans_x += 0.03*step_x;
      boona.trans_y -= 0.03*step_y;
   }
   else if(boona.lmb && boona.mmb)    /* Dolly */
   {
      glutSetCursor(GLUT_CURSOR_LEFT_RIGHT);
      boona.dolly += 0.03*step_x;
   }
   glutPostRedisplay();
}
