/******************************************************************************/
/*                                                                            */
/*        Project: Boona Taxi                                                 */
/*          Title: Game Engine (GLUT callbacks)                               */
/*        Version: 0.2                                                        */
/*                                                                            */
/*        Started: 19/09/2000                                                 */
/*   Last Updated: 26/02/2001                                                 */
/*                                                                            */
/*         Author: Michael Bonnington                                         */
/*                                                                            */
/******************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <iostream.h>

#include "./opengl_includes.h"
#include "./font.h"
#include "./frametime.h"
#include "./callbacks.h"
#include "./car.h"
#include "./tga.h"

#include "./fruity_exception.h"
#include "./fruity_math.h"



/* Constants =================================================================*/

/* Constants for camera views... */
#define CHASE 0
#define COPTER 1
#define IN_CAR 2

//get rid of these when car class is sorted...
#define ETYPE 0
#define ESPRIT 1
#define PUG 2
#define BUG 3



/* Macros ====================================================================*/
#define resetView(a) (a).spin_x  = 0;\
                     (a).spin_y  = 0;\
                     (a).dolly   = 0;\
                     (a).trans_x = 0;\
                     (a).trans_y = 0;



/* Function prototypes =======================================================*/
void changeGameMode(int);

/* Drawing functions... */
void drawCity(void);
void drawOverlay(void);
void drawHUD(void);


/* General GLUT callback functions... */
void reshape(int, int);
void keyDown(unsigned char, int, int);
void keyUp(unsigned char, int, int);
void specialDown(int, int, int);
void specialUp(int, int, int);
void mouse(int, int, int, int);
void motion(int, int);



/* Globals ===================================================================*/
/*##############################################################################
###                            EXTERNAL GLOBALS
##############################################################################*/



/*------------------------  Engine Requires These  ---------------------------*/

/* Structure to hold window information, including size and position... */
struct window win;

/*----------------------------------------------------------------------------*/

//not sure what this was for
//extern int WIN_MAT;





/* Globals ===================================================================*/
int current_car = ETYPE;   //remove car when I have car class


/*------------------------  Engine Requires These  ---------------------------*/

// USED TO HOLD SET UP OF ORTHOGRAPHIC AND PERSPECTIVE VIEWS
GLuint persp, ortho;

char statusLine[256];   //line of text to display messages on-screen

/*----------------------------------------------------------------------------*/

car player(0, 0, 0);
camera cam;




//cModel city;   //temporary



/* Mikes Display Lists */
//GLuint overlay, hud;

/* Global data structures (defined in callbacks.h header file)... */
game boona;


GLuint logoTexture;


bool keys[128], specials[128];














/* GLUT callback registration ================================================*/
void registerCallbacks(void)
{
   //GLfloat fog_colour[3] = {0.46667, 0.73333, 1.0};

   printf("Initialising OpenGL:\n  Vendor: %s\nRenderer: %s\n Version: %s\n\n",
          (char *)glGetString(GL_VENDOR),
          (char *)glGetString(GL_RENDERER),
          (char *)glGetString(GL_VERSION));

   /* Initialise OpenGL/GLUT stuff... */
   glEnable(GL_CULL_FACE);
   glCullFace(GL_BACK);

/*temporary fog setup...
   glFogi(GL_FOG_MODE, GL_EXP);
   glFogfv(GL_FOG_COLOR, fog_colour);
   glFogf(GL_FOG_DENSITY, 0.05);
   glEnable(GL_FOG);*/

   glutIgnoreKeyRepeat(1);
   glutSetCursor(GLUT_CURSOR_NONE);

   /* Initialise frame counter... */
   if(!initFrameCounter())
   {
      printf("ERROR: Unable to initialise frame counter.\n\n");
      changeGameMode(EXIT);
   }

   makeRasterFont();   //initialise font

   //InitRenderer();
   InitErrorLog();   //look at this later

   /* Load the models... */
   //player.LoadModel("./models/etype.obj");
   //city.LoadModel("./models/city.obj");

   /* Load textures not associated with models... */
   loadTexture("./textures/background_logo.tga", &logoTexture);
   //loadTGA("./textures/etype.tga", 200); //this is TEMP until materials work

   boona.option = SELECT;

   /* Register GLUT callback functions... */
   glutReshapeFunc(reshape);
   glutKeyboardFunc(keyDown);
   glutKeyboardUpFunc(keyUp);
   glutSpecialFunc(specialDown);
   glutSpecialUpFunc(specialUp);
   changeGameMode(TITLES);

   glutPostRedisplay();
}





/* Function to draw presentation screen overlay graphics =====================*/
void drawOverlay(void)
{
   /* Make sure the orthographic display list has been initialised with
    * 'glCallList(persp);'
    * before trying to call this function.
    */

   glBegin(GL_QUADS);
    glColor3f(0.93333, 0.66667, 0.0);
    glVertex2f(0, win.height-44);
    glVertex2f(0, win.height);
    glVertex2f(win.width, win.height);
    glVertex2f(win.width, win.height-44);

    glColor3f(1.0, 0.8, 0.06667);
    glVertex2f(0, win.height-128);
    glVertex2f(0, win.height-44);
    glVertex2f(win.width, win.height-44);
    glVertex2f(win.width, win.height-128);

    glVertex2f(0, 0);
    glVertex2f(0, 64);
    glVertex2f(win.width, 64);
    glVertex2f(win.width, 0);
   glEnd();

   smallFont("www.boonaracer.co.uk/taxi", 16, win.height-100, 0.0, 0.0, 0.0);
   smallFont(statusLine, 16, win.height-16, 0.0, 0.0, 0.0);
}

/* Function to draw heads up display =========================================*/
void drawHUD(void)
{
   /* Make sure the orthographic display list has been initialised with
    * 'glCallList(persp);'
    * before trying to call this function.
    */

   //try putting some of this in display lists to speed it up...

   char info_text[8];

   sprintf(info_text, "$%4.2f", player.getScore());
   smallFont("Fare", 16, 28, 1.0, 1.0, 1.0);
   largeOutlineFont(info_text, 16, 60, 1.0, 0.8, 0.06667);

   sprintf(info_text, "%d", (int)boona.game_time);
   smallFont("Time", win.width-96, 28, 1.0, 1.0, 1.0);
   largeOutlineFont(info_text, win.width-96, 60, 1.0, 0.8, 0.06667);

   sprintf(info_text, "%d", (int)(player.getSpeed()*10));
   smallFont("Speed", win.width-96, win.height-48, 1.0, 1.0, 1.0);
   largeOutlineFont(info_text, win.width-96, win.height-16, 1.0, 0.8, 0.06667);

   if(player.reverse)   //Are we in reverse?
      largeOutlineFont("R", win.width-32, win.height-16, 1.0, 0.6, 0.0);
   else                 //Are we in drive?
      largeOutlineFont("D", win.width-32, win.height-16, 0.33333, 0.8, 0.33333);

   //draw rotating map here
}

/* Interpolate camera position ===============================================*/
void interpolateCam(float new_pos_x, float new_pos_y, float new_pos_z,
                    float new_aim_x, float new_aim_y, float new_aim_z,
                    float amount)
{
   cam.pos_x = cam.old_pos_x*(1-amount) + new_pos_x*amount;
   cam.pos_y = cam.old_pos_y*(1-amount) + new_pos_y*amount;
   cam.pos_z = cam.old_pos_z*(1-amount) + new_pos_z*amount;
   cam.pos_x = cam.old_aim_x*(1-amount) + new_aim_x*amount;
   cam.pos_y = cam.old_aim_y*(1-amount) + new_aim_y*amount;
   cam.pos_z = cam.old_aim_z*(1-amount) + new_aim_z*amount;
}

/* Update and interpolate camera position ====================================*/
void updateCam(void)
{
   float new_pos_x, new_pos_y, new_pos_z, new_aim_x, new_aim_y, new_aim_z;

   //not needed at present...
   /* cam.distance = sqrt((cam.aim_x - cam.pos_x)*(cam.aim_x - cam.pos_x) +
                          (cam.aim_z - cam.pos_z)*(cam.aim_z - cam.pos_z)); */

   //these values need tweaking...
   switch(boona.cam_view)
   {
      case IN_CAR:
         new_pos_x = player.pos_x;
         new_pos_y = player.pos_y + 0.85;
         new_pos_z = player.pos_z;
         new_aim_x = player.pos_x + player.dx;
         new_aim_y = player.pos_y + 0.85;
         new_aim_z = player.pos_z + player.dz;
         break;

      case CHASE:
         new_pos_x = player.pos_x - player.dx*4;
         new_pos_y = player.pos_y + 1.6;
         new_pos_z = player.pos_z - player.dz*4;
         new_aim_x = player.pos_x;
         new_aim_y = player.pos_y + 1.2;
         new_aim_z = player.pos_z;
         break;

      case COPTER:
         new_pos_x = player.pos_x - player.dx*4;
         new_pos_y = player.pos_y + 4;
         new_pos_z = player.pos_z - player.dz*4;
         new_aim_x = player.pos_x;
         new_aim_y = player.pos_y + 2;
         new_aim_z = player.pos_z;
         break;

      default:
         break;
   }

/* The next six lines take the new position and aiming coordinates for the
 * camera and smoothly interpolates from its current position to the new
 * position over a period of time. Only linear interpolation at the moment -
 * could be re-written to use quadratic or cubic interpolation for a smoother
 * effect...
 */

   cam.pos_x = cam.old_pos_x*(1-boona.counter) + new_pos_x*boona.counter;
   cam.pos_y = cam.old_pos_y*(1-boona.counter) + new_pos_y*boona.counter;
   cam.pos_z = cam.old_pos_z*(1-boona.counter) + new_pos_z*boona.counter;
   cam.aim_x = cam.old_aim_x*(1-boona.counter) + new_aim_x*boona.counter;
   cam.aim_y = cam.old_aim_y*(1-boona.counter) + new_aim_y*boona.counter;
   cam.aim_z = cam.old_aim_z*(1-boona.counter) + new_aim_z*boona.counter;
}

/* ===========================================================================*/
void positionCam(void)
{
   switch(boona.cam_view)
   {
      case IN_CAR:
         cam.pos_x = player.pos_x;
         cam.pos_y = player.pos_y + 0.85;
         cam.pos_z = player.pos_z;
         cam.aim_x = player.pos_x + player.dx;
         cam.aim_y = player.pos_y + 0.85;
         cam.aim_z = player.pos_z + player.dz;
         break;

      case CHASE:
         cam.pos_x = player.pos_x - player.dx*4;
         cam.pos_y = player.pos_y + 1.6;
         cam.pos_z = player.pos_z - player.dz*4;
         cam.aim_x = player.pos_x;
         cam.aim_y = player.pos_y + 1.2;
         cam.aim_z = player.pos_z;
         break;

      case COPTER:
         cam.pos_x = player.pos_x - player.dx*4;
         cam.pos_y = player.pos_y + 4;
         cam.pos_z = player.pos_z - player.dz*4;
         cam.aim_x = player.pos_x;
         cam.aim_y = player.pos_y + 2;
         cam.aim_z = player.pos_z;
         break;

      default:
         break;
   }
}










/* Reshape function ==========================================================*/
void reshape(int w, int h)
{
   if(h == 0) h = 1;   //Required to prevent crash when window is minimised

   win.width = w;
   win.height = h;

   glDeleteLists(persp, 1);
   glDeleteLists(ortho, 1);
   persp = glGenLists(1);
   ortho = glGenLists(1);

   glNewList(persp, GL_COMPILE);
    glViewport(0, 0, (GLsizei) w, (GLsizei) h);
    glMatrixMode(GL_PROJECTION);
    glLoadIdentity();
    gluPerspective(75.0, (GLfloat) w/(GLfloat) h, 0.1, 256);
    glMatrixMode(GL_MODELVIEW);
    glLoadIdentity();
    glEnable(GL_DEPTH_TEST);
    glEnable(GL_TEXTURE_2D);
    //glEnable(GL_FOG);
   glEndList();

   glNewList(ortho, GL_COMPILE);
    glViewport(0, 0, (GLsizei) w, (GLsizei) h);
    glMatrixMode(GL_PROJECTION);
    glLoadIdentity();
    glOrtho(0, w, h, 0, -1, 1);
    glMatrixMode(GL_MODELVIEW);
    glLoadIdentity();
    glDisable(GL_DEPTH_TEST);
    glDisable(GL_TEXTURE_2D);
    //glDisable(GL_FOG);
   glEndList();

   glCallList(persp);
}

/* Keyboard function - key is pressed ========================================*/
void keyDown(unsigned char key, int x, int y)
{
   keys[key] = true;

   if(boona.game_mode == GAME)
   {
      switch(key)
      {
         case 'z':
            player.brake = true;
            break;

         case 'x':
            player.accelerator = true;
            break;

         case ' ':
            player.handbrake = true;
            break;

         default:
            break;
      }
   }
}

/* Keyboard function - key is released =======================================*/
void keyUp(unsigned char key, int x, int y)
{
   keys[key] = false;

   if(boona.game_mode == GAME)
   {
      switch(key)
      {
         case 'z':
            player.brake = false;
            break;

         case 'x':
            player.accelerator = false;
            break;

         case ' ':
            player.handbrake = false;
            break;

         default:
            break;
      }
   }
}

/* Keyboard function - special key is pressed ================================*/
void specialDown(int key, int x, int y)
{
   specials[key] = true;

   if(boona.game_mode == GAME)
   {
      switch(key)
      {
         case GLUT_KEY_LEFT:
            player.left  = true;
            player.right = false;
            break;

         case GLUT_KEY_RIGHT:
            player.right = true;
            player.left  = false;
            break;

         case GLUT_KEY_UP:
            player.reverse = false;
            break;

         case GLUT_KEY_DOWN:
            player.reverse = true;
            break;

         default:
            break;
      }
   }
}

/* Keyboard function - special key is released ===============================*/
void specialUp(int key, int x, int y)
{
   specials[key] = false;

   if(boona.game_mode == GAME)
   {
      switch(key)
      {
         case GLUT_KEY_LEFT:
            player.left = false;
            break;

         case GLUT_KEY_RIGHT:
            player.right = false;
            break;

         default:
            break;
      }
   }
}

/* Mouse input function ======================================================*/
void mouse(int button, int state, int x, int y)
{


   glutPostRedisplay();
}

/* Motion function ===========================================================*/
void motion(int x, int y)
{

   glutPostRedisplay();
}
