/******************************************************************************/
/*                                                                            */
/*        Project: Boona Taxi                                                 */
/*          Title: Car functions                                              */
/*        Version: 0.2                                                        */
/*                                                                            */
/*        Started: 15/12/2000                                                 */
/*   Last Updated: 09/03/2001                                                 */
/*                                                                            */
/*         Author: Michael Bonnington                                         */
/*                                                                            */
/******************************************************************************/

#include "./car.h"


#define HIGH_SHADOW

extern objFile Thingy;


/* Update position of player-controlled car ==================================*/
void car::update(double frametime)
{
   if(accelerator)
   {
      if(reverse)   //accelerating backwards - two levels
      {
         if(speed > frametime-(max_speed/2))
         {
            speed -= frametime;
            if(speed > -(max_speed/2)*0.5)
            {
               speed -= frametime;
            }
         }
         else speed = -(max_speed/2);
      }

      else   //accelerating forwards - four levels
      {
         if(speed < max_speed-frametime)
         {
            speed += frametime;
            if(speed < max_speed*0.75)
            {
               speed += frametime;
               if(speed < max_speed*0.5)
               {
                  speed += frametime;
                  if(speed < max_speed*0.25)
                  {
                     speed += frametime;
                  }
               }
            }
         }
         else speed = max_speed;
      }

      if(left)
      {
         angle += frametime*100;
         heading += frametime*80;
         dx = sin(heading*PI_OVER_180);
         dz = cos(heading*PI_OVER_180);
      }
      else if(right)
      {
         angle -= frametime*100;
         heading -= frametime*80;
         dx = sin(heading*PI_OVER_180);
         dz = cos(heading*PI_OVER_180);
      }
      else
      {
         heading = angle;
         dx = sin(heading*PI_OVER_180);
         dz = cos(heading*PI_OVER_180);
      }
   }

   else if(brake)
   {
      //brake - make non-linear
      if(speed > frametime*4) speed -= frametime*4; //*braking coefficient
      else if(speed < -frametime*4) speed += frametime*4; //*braking coefficient
      else
      {
         speed = 0;
         left  = false;
         right = false;
      }

      if(left)
      {
         angle += frametime*80;
         heading += frametime*100;
         dx = sin(heading*PI_OVER_180);
         dz = cos(heading*PI_OVER_180);
      }
      if(right)
      {
         angle -= frametime*80;
         heading -= frametime*100;
         dx = sin(heading*PI_OVER_180);
         dz = cos(heading*PI_OVER_180);
      }
   }

   else if(handbrake)
   {
      //handbrake - make non-linear
      if(speed > frametime*2) speed -= frametime*2; //*braking coefficient
      else if(speed < -frametime*2) speed += frametime*2; //*braking coefficient
      else
      {
         speed = 0;
         left  = false;
         right = false;
         handbrake = false;
      }

      if(left)
      {
         angle += frametime*120;
      }
      else if(right)
      {
         angle -= frametime*120;
      }
      else
      {
         //heading = angle;
         dx = sin(heading*PI_OVER_180);
         dz = cos(heading*PI_OVER_180);
      }
   }

   else
   {
      if(left)
      {
         angle += frametime*60;
         heading += frametime*50;
      }
      if(right)
      {
         angle -= frametime*60;
         heading -= frametime*50;
      }

      /* Calculate drag... */
      if(speed > frametime) speed -= frametime; //*drag or weight???
      else if(speed < -frametime) speed += frametime;
      else speed = 0;

      /* Calculate heading to approach angle over time... */
      /*if(heading < angle-frametime*90) heading += frametime*90;
      else if(heading > angle+frametime*90) heading -= frametime*90;
      else heading = angle;*/
      dx = sin(heading*PI_OVER_180);
      dz = cos(heading*PI_OVER_180);

      if(left)
      {
         angle += frametime*60;
      }
      if(right)
      {
         angle -= frametime*60;
      }
   }

   /* Update car position... */
      //not sure if there are too many uses of frametime?
   pos_x += dx*speed*frametime*5;
   pos_y = 0; //1-weight
   pos_z += dz*speed*frametime*5;
}

/* Draw car ==================================================================*/
bool car::draw(void)
  {

    matrix xForm;
    xForm.translate(pos_x, pos_y, pos_z).rotateY( angle );


    glPushMatrix();

      glDisable( GL_FOG );
      calc_shadows( xForm );
      calc_dynamic_shadows( Thingy );
      glDisable( GL_DEPTH_TEST );
      draw_shadows();
      glEnable( GL_FOG );
      glEnable( GL_DEPTH_TEST );


      glTranslatef(pos_x, pos_y, pos_z);   //place car at correct position
      glRotatef(angle, 0.0, 1.0, 0.0);     //point car in correct direction  

      glPushMatrix();

        useLighting = true;
        objFile::draw();
        useLighting = false;

      /*  glDisable(GL_TEXTURE_2D);

        if(accelerator)   //draw boosters
          {
            glBegin(GL_TRIANGLES);
             glColor3f(1.0, 0.6, 0.3);
             glVertex3f( 0.4, 0.3, -2.2);
             glVertex3f( 0.0, 0.3, -3.2);
             glVertex3f(-0.4, 0.3, -2.2);
             glColor3f(1.0, 0.9, 0.7);
             glVertex3f( 0.3, 0.31, -2.2);
             glVertex3f( 0.0, 0.31, -2.8);
             glVertex3f(-0.3, 0.31, -2.2);
            glEnd();
          }

        if(brake)   //draw brake lights
          {
            glColor3f(1.0, 0.0, 0.0);
             glBegin(GL_QUADS);
             glVertex3f( 0.8, 0.6, -2.21);
             glVertex3f( 0.8, 0.5, -2.21);
             glVertex3f(-0.8, 0.5, -2.21);
             glVertex3f(-0.8, 0.6, -2.21);
             glEnd();
          } */

      glPopMatrix();

      glEnable(GL_TEXTURE_2D);
    glPopMatrix();

    //draw heading vector...
    /*glColor3f(0.0, 1.0, 0.0);
    glBegin(GL_LINES);
    glVertex3f(pos_x, 0.3, pos_z);
    glVertex3f(pos_x+(dx*8), 0.3, pos_z+(dz*8));
    glEnd();*/

    return true;
  }

