/******************************************************************************/
/*                                                                            */
/*        Project: Boona Taxi                                                 */
/*          Title: Car class header file                                      */
/*        Version: 0.2                                                        */
/*                                                                            */
/*        Started: 15/12/2000                                                 */
/*   Last Updated: 26/02/2001                                                 */
/*                                                                            */
/*         Author: Michael Bonnington                                         */
/*                                                                            */
/******************************************************************************/

/* car.h: The car class is derived from the model data class, enabling the
 *        drawing functions to be used within this class. Apart from this, the
 *        functionality is mainly in providing a acceptable simulation of car
 *        physics ("simulation" is not really correct, as the cars and their
 *        handling attributes are purely fantasy).
 */

#ifndef CAR_H
#define CAR_H

#include "./opengl_includes.h"
#include "./fruity_math.h"
#include "./objloader.h"


/* Class definition ==========================================================*/
class car : public objFile
{
   public:

      /* Data to hold the car's position and direction... */
      float pos_x, pos_y, pos_z, dx, dz;

      /* Player inputs affecting the car... */
      bool left, right, reverse, brake, accelerator, handbrake;

      /* Member function prototypes... */
      void update(double);   //function to calculate new position
      bool draw(void);       //function to draw car

      /* Constructor... */
      car(float x=0, float y=0, float z=0, float initial_speed=0) : objFile()
      {
         pos_x = x; pos_y = y; pos_z = z;
         score = 0;
         speed = initial_speed;
         angle = 0;
         heading = angle;
         dx = sin(heading*PI_OVER_180);
         dz = cos(heading*PI_OVER_180);
         left = false; right = false; reverse = false;
         brake = false; accelerator = false; handbrake = false;

         weight = 1; //max weight
         max_speed = 8;

         //vect.set(x, y, z);
      };

      /* Destructor... */
      ~car(void)
      {
      };

      /* Query functions... */
      inline float getScore(void)
      {
         return score;
      };

      inline float getSpeed(void)
      {
         return speed;
      };

      inline float getAngle(void)
      {
         return angle;
      };

      inline float getHeading(void)
      {
         return heading;
      };


   private:

      float score, speed, angle, heading,
      weight, max_speed, acc_factor;   //modifiable car attributes

      //cVector3f vect;
};



#endif
