/******************************************************************************/
/*                                                                            */
/*        Project: Boona Taxi                                                 */
/*          Title: Font Definitions                                           */
/*        Version: 1.2                                                        */
/*                                                                            */
/*        Started: 06/05/2000                                                 */
/*   Last Updated: 30/11/2000                                                 */
/*                                                                            */
/*         Author: Michael Bonnington                                         */
/*                                                                            */
/******************************************************************************/

/* font.h : This header file contains declarations and functions for three
 *          raster fonts...
 *
 *          - SmallFont, which is based on the Windows font Tahoma, 8 point.
 *            All characters are identical to the orignals, except for the
 *            uppercase 'W' which I made smaller in order to avoid having a
 *            larger array. This font contains all ASCII characters from 0 to
 *            95 (96 characters), which is all the useful ones.
 *
 *          - LargeOutlineFont, a bold two-colour font designed by myself for
 *            'Boona Racer 2000'. Only contains uppercase characters, numbers,
 *            and some other selected characters. There are 48 charactes in
 *            total.
 *
 *          - LargeFont, the same as the above, but without the black outline.
 *
 *          These fonts have been modified so that the dollar sign ($) is
 *          replaced by a pound sign ().
 */

#ifndef FONT_H
#define FONT_H

#include "./opengl_includes.h"
#include <string.h>



/* Function prototypes =======================================================*/
void makeRasterFont(void);
void smallFont(char *, int, int, float, float, float);
void largeFont(char *, int, int, float, float, float);
void largeOutlineFont(char *, int, int, float, float, float);



/******************************************************************************/
/* Small Font                                                                 */
/* 8x12 pixels, variable width, 96 chars                                      */
/* see "SmallFont.gif"                                                        */
/******************************************************************************/

/* Hex definition for each character... */
GLubyte s_characters[96][12] = {
   {0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00},   /* 00 space */
   {0x00, 0x00, 0x40, 0x00, 0x40, 0x40, 0x40, 0x40, 0x40, 0x40, 0x00, 0x00},   /* 01 ! */
   {0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0xA0, 0xA0, 0xA0, 0x00},   /* 02 " */
   {0x00, 0x00, 0x50, 0x50, 0xFC, 0x28, 0x28, 0x7E, 0x14, 0x14, 0x00, 0x00},   /* 03 # */
   {0x00, 0x00, 0xF8, 0x40, 0x40, 0xF0, 0x40, 0x40, 0x40, 0x38, 0x00, 0x00},   /* 04 $ () */
   {0x00, 0x00, 0x22, 0x25, 0x15, 0x12, 0x48, 0xA8, 0xA4, 0x44, 0x00, 0x00},   /* 05 % */
   {0x00, 0x00, 0x72, 0x8C, 0x88, 0x94, 0x64, 0x90, 0x90, 0x60, 0x00, 0x00},   /* 06 & */
   {0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x80, 0x80, 0x80, 0x00},   /* 07 ' */
   {0x20, 0x40, 0x40, 0x80, 0x80, 0x80, 0x80, 0x80, 0x40, 0x40, 0x20, 0x00},   /* 08 ( */
   {0x80, 0x40, 0x40, 0x20, 0x20, 0x20, 0x20, 0x20, 0x40, 0x40, 0x80, 0x00},   /* 09 ) */
   {0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x20, 0xA8, 0x70, 0xA8, 0x20, 0x00},   /* 10 * */
   {0x00, 0x00, 0x10, 0x10, 0x10, 0xFE, 0x10, 0x10, 0x10, 0x00, 0x00, 0x00},   /* 11 + */
   {0x80, 0x40, 0x40, 0x40, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00},   /* 12 , */
   {0x00, 0x00, 0x00, 0x00, 0x00, 0xE0, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00},   /* 13 - */
   {0x00, 0x00, 0x40, 0x40, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00},   /* 14 . */
   {0x80, 0x80, 0x80, 0x40, 0x40, 0x40, 0x40, 0x40, 0x20, 0x20, 0x20, 0x00},   /* 15 / */
   {0x00, 0x00, 0x70, 0x88, 0x88, 0x88, 0x88, 0x88, 0x88, 0x70, 0x00, 0x00},   /* 16 0 */
   {0x00, 0x00, 0x70, 0x20, 0x20, 0x20, 0x20, 0x20, 0x60, 0x20, 0x00, 0x00},   /* 17 1 */
   {0x00, 0x00, 0xF8, 0x80, 0x40, 0x20, 0x10, 0x08, 0x88, 0x70, 0x00, 0x00},   /* 18 2 */
   {0x00, 0x00, 0x70, 0x88, 0x08, 0x08, 0x30, 0x08, 0x88, 0x70, 0x00, 0x00},   /* 19 3 */
   {0x00, 0x00, 0x10, 0x10, 0x10, 0xF8, 0x90, 0x50, 0x30, 0x10, 0x00, 0x00},   /* 20 4 */
   {0x00, 0x00, 0x70, 0x88, 0x08, 0x08, 0xF0, 0x80, 0x80, 0xF8, 0x00, 0x00},   /* 21 5 */
   {0x00, 0x00, 0x70, 0x88, 0x88, 0x88, 0xF0, 0x80, 0x40, 0x30, 0x00, 0x00},   /* 22 6 */
   {0x00, 0x00, 0x40, 0x40, 0x20, 0x20, 0x10, 0x10, 0x08, 0xF8, 0x00, 0x00},   /* 23 7 */
   {0x00, 0x00, 0x70, 0x88, 0x88, 0x88, 0x70, 0x88, 0x88, 0x70, 0x00, 0x00},   /* 24 8 */
   {0x00, 0x00, 0x60, 0x10, 0x08, 0x78, 0x88, 0x88, 0x88, 0x70, 0x00, 0x00},   /* 25 9 */
   {0x00, 0x00, 0x40, 0x40, 0x00, 0x00, 0x40, 0x40, 0x00, 0x00, 0x00, 0x00},   /* 26 : */
   {0x80, 0x40, 0x40, 0x40, 0x00, 0x00, 0x40, 0x40, 0x00, 0x00, 0x00, 0x00},   /* 27 ; */
   {0x00, 0x00, 0x02, 0x0C, 0x30, 0x40, 0x30, 0x0C, 0x02, 0x00, 0x00, 0x00},   /* 28 < */
   {0x00, 0x00, 0x00, 0x00, 0xFE, 0x00, 0xFE, 0x00, 0x00, 0x00, 0x00, 0x00},   /* 29 = */
   {0x00, 0x00, 0x40, 0x30, 0x0C, 0x02, 0x0C, 0x30, 0x40, 0x00, 0x00, 0x00},   /* 30 > */
   {0x00, 0x00, 0x40, 0x00, 0x40, 0x40, 0x20, 0x10, 0x10, 0xE0, 0x00, 0x00},   /* 31 ? */
   {0x00, 0x38, 0x40, 0x9E, 0xA9, 0xA9, 0xA9, 0x99, 0x42, 0x3C, 0x00, 0x00},   /* 32 @ */
   {0x00, 0x00, 0x84, 0x84, 0xFC, 0x48, 0x48, 0x48, 0x30, 0x30, 0x00, 0x00},   /* 33 A */
   {0x00, 0x00, 0xF0, 0x88, 0x88, 0x88, 0xF0, 0x88, 0x88, 0xF0, 0x00, 0x00},   /* 34 B */
   {0x00, 0x00, 0x3C, 0x40, 0x80, 0x80, 0x80, 0x80, 0x40, 0x3C, 0x00, 0x00},   /* 35 C */
   {0x00, 0x00, 0xF0, 0x88, 0x84, 0x84, 0x84, 0x84, 0x88, 0xF0, 0x00, 0x00},   /* 36 D */
   {0x00, 0x00, 0xF8, 0x80, 0x80, 0x80, 0xF0, 0x80, 0x80, 0xF8, 0x00, 0x00},   /* 37 E */
   {0x00, 0x00, 0x80, 0x80, 0x80, 0x80, 0xF0, 0x80, 0x80, 0xF8, 0x00, 0x00},   /* 38 F */
   {0x00, 0x00, 0x3C, 0x44, 0x84, 0x9C, 0x80, 0x80, 0x40, 0x3C, 0x00, 0x00},   /* 39 G */
   {0x00, 0x00, 0x84, 0x84, 0x84, 0x84, 0xFC, 0x84, 0x84, 0x84, 0x00, 0x00},   /* 40 H */
   {0x00, 0x00, 0xE0, 0x40, 0x40, 0x40, 0x40, 0x40, 0x40, 0xE0, 0x00, 0x00},   /* 41 I */
   {0x00, 0x00, 0xE0, 0x10, 0x10, 0x10, 0x10, 0x10, 0x10, 0x70, 0x00, 0x00},   /* 42 J */
   {0x00, 0x00, 0x88, 0x90, 0xA0, 0xC0, 0xC0, 0xA0, 0x90, 0x88, 0x00, 0x00},   /* 43 K */
   {0x00, 0x00, 0xF0, 0x80, 0x80, 0x80, 0x80, 0x80, 0x80, 0x80, 0x00, 0x00},   /* 44 L */
   {0x00, 0x00, 0x82, 0x82, 0x92, 0x92, 0xAA, 0xAA, 0xC6, 0xC6, 0x00, 0x00},   /* 45 M */
   {0x00, 0x00, 0x8C, 0x8C, 0x94, 0x94, 0xA4, 0xA4, 0xC4, 0xC4, 0x00, 0x00},   /* 46 N */
   {0x00, 0x00, 0x38, 0x44, 0x82, 0x82, 0x82, 0x82, 0x44, 0x38, 0x00, 0x00},   /* 47 O */
   {0x00, 0x00, 0x80, 0x80, 0x80, 0xF0, 0x88, 0x88, 0x88, 0xF0, 0x00, 0x00},   /* 48 P */
   {0x06, 0x08, 0x38, 0x44, 0x82, 0x82, 0x82, 0x82, 0x44, 0x38, 0x00, 0x00},   /* 49 Q */
   {0x00, 0x00, 0x84, 0x88, 0x90, 0xF0, 0x88, 0x88, 0x88, 0xF0, 0x00, 0x00},   /* 50 R */
   {0x00, 0x00, 0xF0, 0x08, 0x08, 0x08, 0x70, 0x80, 0x80, 0x78, 0x00, 0x00},   /* 51 S */
   {0x00, 0x00, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0xF8, 0x00, 0x00},   /* 52 T */
   {0x00, 0x00, 0x78, 0x84, 0x84, 0x84, 0x84, 0x84, 0x84, 0x84, 0x00, 0x00},   /* 53 U */
   {0x00, 0x00, 0x20, 0x20, 0x50, 0x50, 0x50, 0x88, 0x88, 0x88, 0x00, 0x00},   /* 54 V */
   {0x00, 0x00, 0x44, 0x44, 0x44, 0xAA, 0xAA, 0x92, 0x92, 0x92, 0x00, 0x00},   /* 55 W */
   {0x00, 0x00, 0x88, 0x88, 0x50, 0x20, 0x20, 0x50, 0x88, 0x88, 0x00, 0x00},   /* 56 X */
   {0x00, 0x00, 0x20, 0x20, 0x20, 0x20, 0x50, 0x50, 0x88, 0x88, 0x00, 0x00},   /* 57 Y */
   {0x00, 0x00, 0xF8, 0x80, 0x40, 0x20, 0x20, 0x10, 0x08, 0xF8, 0x00, 0x00},   /* 58 Z */
   {0xE0, 0x80, 0x80, 0x80, 0x80, 0x80, 0x80, 0x80, 0x80, 0x80, 0xE0, 0x00},   /* 59 [ */
   {0x20, 0x20, 0x20, 0x40, 0x40, 0x40, 0x40, 0x40, 0x80, 0x80, 0x80, 0x00},   /* 60 \ */
   {0xE0, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0xE0, 0x00},   /* 61 ] */
   {0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x82, 0x44, 0x28, 0x10, 0x00, 0x00},   /* 62 ^ */
   {0xFC, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00},   /* 63 _ */
   {0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x20, 0x40, 0x00},   /* 64 ` */
   {0x00, 0x00, 0x78, 0x88, 0x88, 0x78, 0x08, 0x70, 0x00, 0x00, 0x00, 0x00},   /* 65 a */
   {0x00, 0x00, 0xF0, 0x88, 0x88, 0x88, 0x88, 0xF0, 0x80, 0x80, 0x80, 0x00},   /* 66 b */
   {0x00, 0x00, 0x70, 0x80, 0x80, 0x80, 0x80, 0x70, 0x00, 0x00, 0x00, 0x00},   /* 67 c */
   {0x00, 0x00, 0x78, 0x88, 0x88, 0x88, 0x88, 0x78, 0x08, 0x08, 0x08, 0x00},   /* 68 d */
   {0x00, 0x00, 0x70, 0x88, 0x80, 0xF8, 0x88, 0x70, 0x00, 0x00, 0x00, 0x00},   /* 69 e */
   {0x00, 0x00, 0x80, 0x80, 0x80, 0x80, 0x80, 0xE0, 0x80, 0x80, 0x60, 0x00},   /* 70 f */
   {0x70, 0x08, 0x78, 0x88, 0x88, 0x88, 0x88, 0x78, 0x00, 0x00, 0x00, 0x00},   /* 71 g */
   {0x00, 0x00, 0x88, 0x88, 0x88, 0x88, 0x88, 0xF0, 0x80, 0x80, 0x80, 0x00},   /* 72 h */
   {0x00, 0x00, 0x80, 0x80, 0x80, 0x80, 0x80, 0x80, 0x00, 0x80, 0x00, 0x00},   /* 73 i */
   {0x80, 0x40, 0x40, 0x40, 0x40, 0x40, 0x40, 0xC0, 0x00, 0x40, 0x00, 0x00},   /* 74 j */
   {0x00, 0x00, 0x88, 0x90, 0xA0, 0xC0, 0xA0, 0x90, 0x80, 0x80, 0x80, 0x00},   /* 75 k */
   {0x00, 0x00, 0x80, 0x80, 0x80, 0x80, 0x80, 0x80, 0x80, 0x80, 0x80, 0x00},   /* 76 l */
   {0x00, 0x00, 0x92, 0x92, 0x92, 0x92, 0x92, 0xEC, 0x00, 0x00, 0x00, 0x00},   /* 77 m */
   {0x00, 0x00, 0x88, 0x88, 0x88, 0x88, 0x88, 0xF0, 0x00, 0x00, 0x00, 0x00},   /* 78 n */
   {0x00, 0x00, 0x70, 0x88, 0x88, 0x88, 0x88, 0x70, 0x00, 0x00, 0x00, 0x00},   /* 79 o */
   {0x80, 0x80, 0xF0, 0x88, 0x88, 0x88, 0x88, 0xF0, 0x00, 0x00, 0x00, 0x00},   /* 80 p */
   {0x08, 0x08, 0x78, 0x88, 0x88, 0x88, 0x88, 0x78, 0x00, 0x00, 0x00, 0x00},   /* 81 q */
   {0x00, 0x00, 0x80, 0x80, 0x80, 0x80, 0xC0, 0xA0, 0x00, 0x00, 0x00, 0x00},   /* 82 r */
   {0x00, 0x00, 0xE0, 0x10, 0x30, 0xC0, 0x80, 0x70, 0x00, 0x00, 0x00, 0x00},   /* 83 s */
   {0x00, 0x00, 0x60, 0x80, 0x80, 0x80, 0x80, 0xE0, 0x80, 0x80, 0x00, 0x00},   /* 84 t */
   {0x00, 0x00, 0x78, 0x88, 0x88, 0x88, 0x88, 0x88, 0x00, 0x00, 0x00, 0x00},   /* 85 u */
   {0x00, 0x00, 0x20, 0x20, 0x50, 0x50, 0x88, 0x88, 0x00, 0x00, 0x00, 0x00},   /* 86 v */
   {0x00, 0x00, 0x44, 0x44, 0xAA, 0xAA, 0x92, 0x92, 0x00, 0x00, 0x00, 0x00},   /* 87 w */
   {0x00, 0x00, 0x88, 0x50, 0x20, 0x20, 0x50, 0x88, 0x00, 0x00, 0x00, 0x00},   /* 88 x */
   {0x40, 0x40, 0x20, 0x20, 0x50, 0x50, 0x88, 0x88, 0x00, 0x00, 0x00, 0x00},   /* 89 y */
   {0x00, 0x00, 0xF0, 0x80, 0x40, 0x20, 0x10, 0xF0, 0x00, 0x00, 0x00, 0x00},   /* 90 z */
   {0x10, 0x20, 0x20, 0x20, 0x20, 0xC0, 0x20, 0x20, 0x20, 0x20, 0x10, 0x00},   /* 91 { */
   {0x40, 0x40, 0x40, 0x40, 0x40, 0x40, 0x40, 0x40, 0x40, 0x40, 0x40, 0x00},   /* 92 | */
   {0x80, 0x40, 0x40, 0x40, 0x40, 0x30, 0x40, 0x40, 0x40, 0x40, 0x80, 0x00},   /* 93 } */
   {0x00, 0x00, 0x00, 0x00, 0x8C, 0x92, 0x62, 0x00, 0x00, 0x00, 0x00, 0x00},   /* 94 ~ */
   {0x00, 0x00, 0xFF, 0x81, 0x81, 0x81, 0x81, 0x81, 0x81, 0xFF, 0x00, 0x00}    /* 95  */
};

/* Specify kerning values for each character... */
unsigned int s_kern[96] = {
   5, 4, 4, 0, 2, -1, 1, 6, 4, 4, 2, 0, 4, 4, 4, 4, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 4, 4, 0, 0, 0, 3,
   -1, 1, 2, 1, 1, 2, 2, 1, 1, 4, 3, 2, 3, 0, 1, 0, 2, 0, 1, 2, 2, 1, 2, 0, 2, 2, 2, 4, 4, 4, 0, 2,
   4, 2, 2, 3, 2, 2, 4, 2, 2, 6, 5, 3, 6, 0, 2, 2, 2, 2, 4, 3, 4, 2, 2, 0, 2, 2, 3, 3, 4, 3, 0, 0};

/* End of small font declaration =============================================*/



/******************************************************************************/
/* Large Font                                                                 */
/* 16x32 pixels, fixed width, 48 chars, uppercase only                        */
/* see "LargeFont.gif"                                                        */
/******************************************************************************/

/* Hex definition for each character (outline colour)... */
GLubyte l_o_space[] =
{0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00};   /* 00 space */

GLubyte l_o_exclamation[] =
{0x0F, 0xF0, 0x0F, 0xF0, 0x0C, 0x30, 0x0C, 0x30,
 0x0C, 0x30, 0x0C, 0x30, 0x0F, 0xF0, 0x0F, 0xF0,
 0x0C, 0x30, 0x0C, 0x30, 0x0C, 0x30, 0x0C, 0x30,
 0x0C, 0x30, 0x0C, 0x30, 0x0C, 0x30, 0x0C, 0x30,
 0x0C, 0x30, 0x0C, 0x30, 0x0C, 0x30, 0x0C, 0x30,
 0x0C, 0x30, 0x0C, 0x30, 0x0C, 0x30, 0x0C, 0x30,
 0x0C, 0x30, 0x0C, 0x30, 0x0C, 0x30, 0x0C, 0x30,
 0x0C, 0x30, 0x0C, 0x30, 0x0F, 0xF0, 0x0F, 0xF0};   /* 01 ! */

GLubyte l_o_double_quote[] =
{0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0xC0, 0xC0,
 0xE0, 0xE0, 0xF0, 0xF0, 0xF8, 0xF8, 0xDC, 0xDC,
 0xCE, 0xCE, 0xC7, 0xC7, 0xC3, 0xC3, 0xC3, 0xC3,
 0xC3, 0xC3, 0xC3, 0xC3, 0xFF, 0xFF, 0xFF, 0xFF};   /* 02 " */

GLubyte l_o_pound_sign[] =
{0xFF, 0xFF, 0xFF, 0xFF, 0xC0, 0x03, 0xC0, 0x03,
 0xC0, 0x03, 0xC0, 0x03, 0xF0, 0xC3, 0xF0, 0xC3,
 0x30, 0xFF, 0x30, 0xFF, 0x30, 0xC0, 0x30, 0xC0,
 0xF0, 0xFC, 0xF0, 0xFC, 0xC0, 0x0C, 0xC0, 0x0C,
 0xC0, 0x0C, 0xC0, 0x0C, 0xF0, 0xFF, 0xF0, 0xFF,
 0x30, 0xC3, 0x30, 0xC3, 0x30, 0xC3, 0x30, 0xC3,
 0x30, 0xC3, 0x30, 0xC3, 0x30, 0xC3, 0x38, 0x07,
 0x18, 0x06, 0x1E, 0x1E, 0x0F, 0xFC, 0x03, 0xF0};   /* 04 $ () */

GLubyte l_o_single_quote[] =
{0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x0C, 0x00,
 0x0E, 0x00, 0x0F, 0x00, 0x0F, 0x80, 0x0D, 0xC0,
 0x0C, 0xE0, 0x0C, 0x70, 0x0C, 0x30, 0x0C, 0x30,
 0x0C, 0x30, 0x0C, 0x30, 0x0F, 0xF0, 0x0F, 0xF0};   /* 07 ' */

GLubyte l_o_plus_sign[] =
{0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
 0x0F, 0xF0, 0x0F, 0xF0, 0x0C, 0x30, 0x0C, 0x30,
 0xFC, 0x3F, 0xFC, 0x3F, 0xC0, 0x03, 0xC0, 0x03,
 0xC0, 0x03, 0xC0, 0x03, 0xFC, 0x3F, 0xFC, 0x3F,
 0x0C, 0x30, 0x0C, 0x30, 0x0F, 0xF0, 0x0F, 0xF0,
 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00};   /* 11 + */

GLubyte l_o_hyphen[] =
{0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
 0xFF, 0xFF, 0xFF, 0xFF, 0xC0, 0x03, 0xC0, 0x03,
 0xC0, 0x03, 0xC0, 0x03, 0xFF, 0xFF, 0xFF, 0xFF,
 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00};   /* 13 - */

GLubyte l_o_fullstop[] =
{0x0F, 0xF0, 0x0F, 0xF0, 0x0C, 0x30, 0x0C, 0x30,
 0x0C, 0x30, 0x0C, 0x30, 0x0F, 0xF0, 0x0F, 0xF0,
 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00};   /* 14 . */

GLubyte l_o_numbers[][64] = {
{0x0F, 0xF0, 0x3F, 0xFC, 0x78, 0x1E, 0x60, 0x06,
 0xE0, 0x07, 0xC1, 0x83, 0xC3, 0xC3, 0xC3, 0xC3,
 0xC3, 0xC3, 0xC3, 0xC3, 0xC3, 0xC3, 0xC3, 0xC3,
 0xC3, 0xC3, 0xC3, 0xC3, 0xC3, 0xC3, 0xC3, 0xC3,
 0xC3, 0xC3, 0xC3, 0xC3, 0xC3, 0xC3, 0xC3, 0xC3,
 0xC3, 0xC3, 0xC3, 0xC3, 0xC3, 0xC3, 0xC3, 0xC3,
 0xC3, 0xC3, 0xC3, 0xC3, 0xC1, 0x83, 0xE0, 0x07,
 0x60, 0x06, 0x78, 0x1E, 0x3F, 0xFC, 0x0F, 0xF0},   /* 16 0 */

{0x3F, 0xFC, 0x3F, 0xFC, 0x30, 0x0C, 0x30, 0x0C,
 0x30, 0x0C, 0x30, 0x0C, 0x3C, 0x3C, 0x3C, 0x3C,
 0x0C, 0x30, 0x0C, 0x30, 0x0C, 0x30, 0x0C, 0x30,
 0x0C, 0x30, 0x0C, 0x30, 0x0C, 0x30, 0x0C, 0x30,
 0x0C, 0x30, 0x0C, 0x30, 0x0C, 0x30, 0x0C, 0x30,
 0x0C, 0x30, 0x0C, 0x30, 0x0C, 0x30, 0x0C, 0x30,
 0x3C, 0x30, 0x3C, 0x30, 0x30, 0x30, 0x30, 0x30,
 0x30, 0x30, 0x30, 0x30, 0x3F, 0xF0, 0x3F, 0xF0},   /* 17 1 */

{0xFF, 0xFF, 0xFF, 0xFF, 0xC0, 0x03, 0xC0, 0x03,
 0xC0, 0x03, 0xC0, 0x03, 0xC3, 0xC3, 0xC3, 0xC3,
 0xC3, 0xC3, 0xC3, 0xC3, 0xC3, 0xFF, 0xC3, 0xFF,
 0xE1, 0xC0, 0x60, 0xE0, 0x70, 0x70, 0x38, 0x38,
 0x1C, 0x1C, 0x0E, 0x0E, 0x07, 0x06, 0x03, 0x87,
 0xFF, 0xC3, 0xFF, 0xC3, 0xC3, 0xC3, 0xC3, 0xC3,
 0xC3, 0xC3, 0xC3, 0xC3, 0xC1, 0x83, 0xE0, 0x07,
 0x60, 0x06, 0x78, 0x1E, 0x3F, 0xFC, 0x0F, 0xF0},   /* 18 2 */

{0x0F, 0xF0, 0x3F, 0xFC, 0x78, 0x1E, 0x60, 0x06,
 0xE0, 0x07, 0xC1, 0x83, 0xC3, 0xC3, 0xC3, 0xC3,
 0xC3, 0xC3, 0xC3, 0xC3, 0xFF, 0xC3, 0xFF, 0xC3,
 0x0F, 0xC3, 0x0F, 0x87, 0x0C, 0x06, 0x0C, 0x0E,
 0x0C, 0x0E, 0x0C, 0x06, 0x0F, 0x87, 0x0F, 0xC3,
 0xFF, 0xC3, 0xFF, 0xC3, 0xC3, 0xC3, 0xC3, 0xC3,
 0xC3, 0xC3, 0xC3, 0xC3, 0xC1, 0x83, 0xE0, 0x07,
 0x60, 0x06, 0x78, 0x1E, 0x3F, 0xFC, 0x0F, 0xF0},   /* 19 3 */

{0x03, 0xFC, 0x03, 0xFC, 0x03, 0x0C, 0x03, 0x0C,
 0x03, 0x0C, 0x03, 0x0C, 0x03, 0x0C, 0x03, 0x0C,
 0xFF, 0x0F, 0xFF, 0x0F, 0xC0, 0x03, 0xC0, 0x03,
 0xC0, 0x03, 0xC0, 0x03, 0xC3, 0x0F, 0xC3, 0x0F,
 0xC3, 0x0C, 0xC3, 0x0C, 0xC3, 0x0C, 0xC3, 0x0C,
 0xC3, 0x0C, 0xC3, 0x0C, 0xC3, 0x0C, 0xC3, 0x0C,
 0xC3, 0x0C, 0xC3, 0x0C, 0xC3, 0x0C, 0xC3, 0x0C,
 0xC3, 0x0C, 0xC3, 0x0C, 0xFF, 0xFC, 0xFF, 0xFC},   /* 20 4 */

{0x0F, 0xF0, 0x3F, 0xFC, 0x78, 0x1E, 0x60, 0x06,
 0xE0, 0x07, 0xC1, 0x83, 0xC3, 0xC3, 0xC3, 0xC3,
 0xC3, 0xC3, 0xC3, 0xC3, 0xFF, 0xC3, 0xFF, 0xC3,
 0xFF, 0xC3, 0xFF, 0xC3, 0xC3, 0x83, 0xC0, 0x07,
 0xC0, 0x06, 0xC0, 0x1E, 0xC3, 0xFC, 0xC3, 0xF0,
 0xC3, 0x00, 0xC3, 0x00, 0xC3, 0x00, 0xC3, 0x00,
 0xC3, 0xFF, 0xC3, 0xFF, 0xC0, 0x03, 0xC0, 0x03,
 0xC0, 0x03, 0xC0, 0x03, 0xFF, 0xFF, 0xFF, 0xFF},   /* 21 5 */

{0x0F, 0xF0, 0x3F, 0xFC, 0x78, 0x1E, 0x60, 0x06,
 0xE0, 0x07, 0xC1, 0x83, 0xC3, 0xC3, 0xC3, 0xC3,
 0xC3, 0xC3, 0xC3, 0xC3, 0xC3, 0xC3, 0xC3, 0xC3,
 0xC3, 0xC3, 0xC3, 0xC3, 0xC1, 0x83, 0xC0, 0x07,
 0xC0, 0x06, 0xC0, 0x1E, 0xC3, 0xFC, 0xC3, 0xF0,
 0xC3, 0xFF, 0xC3, 0xFF, 0xC3, 0xC3, 0xC3, 0xC3,
 0xC3, 0xC3, 0xC3, 0xC3, 0xC1, 0x83, 0xE0, 0x07,
 0x60, 0x06, 0x78, 0x1E, 0x3F, 0xFC, 0x0F, 0xF0},   /* 22 6 */

{0x0F, 0xF0, 0x0F, 0xF0, 0x0C, 0x30, 0x0C, 0x30,
 0x0C, 0x30, 0x0C, 0x30, 0x0C, 0x30, 0x0C, 0x30,
 0x0C, 0x30, 0x0C, 0x30, 0x0C, 0x30, 0x0C, 0x30,
 0x0C, 0x30, 0x0C, 0x30, 0x0C, 0x30, 0x06, 0x18,
 0x06, 0x18, 0x06, 0x18, 0x06, 0x1C, 0x07, 0x0C,
 0xFF, 0x0E, 0xFF, 0x86, 0xC3, 0x87, 0xC3, 0xC3,
 0xC3, 0xC3, 0xC3, 0xC3, 0xC0, 0x03, 0xC0, 0x03,
 0xC0, 0x03, 0xC0, 0x03, 0xFF, 0xFF, 0xFF, 0xFF},   /* 23 7 */

{0x0F, 0xF0, 0x3F, 0xFC, 0x78, 0x1E, 0x60, 0x06,
 0xE0, 0x07, 0xC1, 0x83, 0xC3, 0xC3, 0xC3, 0xC3,
 0xC3, 0xC3, 0xC3, 0xC3, 0xC3, 0xC3, 0xC3, 0xC3,
 0xC3, 0xC3, 0xE1, 0x87, 0x60, 0x06, 0x70, 0x0E,
 0x70, 0x0E, 0x60, 0x06, 0xE1, 0x87, 0xC3, 0xC3,
 0xC3, 0xC3, 0xC3, 0xC3, 0xC3, 0xC3, 0xC3, 0xC3,
 0xC3, 0xC3, 0xC3, 0xC3, 0xC1, 0x83, 0xE0, 0x07,
 0x60, 0x06, 0x78, 0x1E, 0x3F, 0xFC, 0x0F, 0xF0},   /* 24 8 */

{0x0F, 0xF0, 0x3F, 0xFC, 0x78, 0x1E, 0x60, 0x06,
 0xE0, 0x07, 0xC1, 0x83, 0xC3, 0xC3, 0xC3, 0xC3,
 0xC3, 0xC3, 0xC3, 0xC3, 0xFF, 0xC3, 0xFF, 0xC3,
 0x0F, 0xC3, 0x3F, 0xC3, 0x78, 0x03, 0x60, 0x03,
 0xE0, 0x03, 0xC1, 0x83, 0xC3, 0xC3, 0xC3, 0xC3,
 0xC3, 0xC3, 0xC3, 0xC3, 0xC3, 0xC3, 0xC3, 0xC3,
 0xC3, 0xC3, 0xC3, 0xC3, 0xC1, 0x83, 0xE0, 0x07,
 0x60, 0x06, 0x78, 0x1E, 0x3F, 0xFC, 0x0F, 0xF0}    /* 25 9 */
};

GLubyte l_o_colon[] =
{0x0F, 0xF0, 0x0F, 0xF0, 0x0C, 0x30, 0x0C, 0x30,
 0x0C, 0x30, 0x0C, 0x30, 0x0F, 0xF0, 0x0F, 0xF0,
 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
 0x0F, 0xF0, 0x0F, 0xF0, 0x0C, 0x30, 0x0C, 0x30,
 0x0C, 0x30, 0x0C, 0x30, 0x0F, 0xF0, 0x0F, 0xF0,
 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00};   /* 26 : */

GLubyte l_o_less_than[] =
{0x00, 0x00, 0x00, 0x03, 0x00, 0x07, 0x00, 0x0F,
 0x00, 0x1F, 0x00, 0x3B, 0x00, 0x73, 0x00, 0xE3,
 0x01, 0xC3, 0x03, 0x83, 0x07, 0x03, 0x0E, 0x03,
 0x1C, 0x03, 0x38, 0x03, 0x70, 0x03, 0xE0, 0x03,
 0xE0, 0x03, 0x70, 0x03, 0x38, 0x03, 0x1C, 0x03,
 0x0E, 0x03, 0x07, 0x03, 0x03, 0x83, 0x01, 0xC3,
 0x00, 0xE3, 0x00, 0x73, 0x00, 0x3B, 0x00, 0x1F,
 0x00, 0x0F, 0x00, 0x07, 0x00, 0x03, 0x00, 0x00};   /* 28 < */

GLubyte l_o_greater_than[] =
{0x00, 0x00, 0xC0, 0x00, 0xE0, 0x00, 0xF0, 0x00,
 0xF8, 0x00, 0xDC, 0x00, 0xCE, 0x00, 0xC7, 0x00,
 0xC3, 0x80, 0xC1, 0xC0, 0xC0, 0xE0, 0xC0, 0x70,
 0xC0, 0x38, 0xC0, 0x1C, 0xC0, 0x0E, 0xC0, 0x07,
 0xC0, 0x07, 0xC0, 0x0E, 0xC0, 0x1C, 0xC0, 0x38,
 0xC0, 0x70, 0xC0, 0xE0, 0xC1, 0xC0, 0xC3, 0x80,
 0xC7, 0x00, 0xCE, 0x00, 0xDC, 0x00, 0xF8, 0x00,
 0xF0, 0x00, 0xE0, 0x00, 0xC0, 0x00, 0x00, 0x00};   /* 30 > */

GLubyte l_o_question_mark[] =
{0x0F, 0xF0, 0x0F, 0xF0, 0x0C, 0x30, 0x0C, 0x30,
 0x0C, 0x30, 0x0C, 0x30, 0x0F, 0xF0, 0x0F, 0xF0,
 0x0C, 0x30, 0x0C, 0x30, 0x0C, 0x30, 0x0C, 0x30,
 0x0C, 0x30, 0x0C, 0x30, 0x0C, 0x30, 0x0C, 0x38,
 0x0E, 0x18, 0x06, 0x1C, 0xFF, 0x0E, 0xFF, 0x87,
 0xC3, 0xC3, 0xC3, 0xC3, 0xC3, 0xC3, 0xC3, 0xC3,
 0xC3, 0xC3, 0xC3, 0xC3, 0xC1, 0x83, 0xE0, 0x07,
 0x60, 0x06, 0x78, 0x1E, 0x3F, 0xFC, 0x0F, 0xF0};   /* 31 ? */

GLubyte l_o_letters[][64] = {
{0xFF, 0xFF, 0xFF, 0xFF, 0xC3, 0xC3, 0xC3, 0xC3,
 0xC3, 0xC3, 0xC3, 0xC3, 0xC3, 0xC3, 0xC3, 0xC3,
 0x60, 0x06, 0x60, 0x06, 0x60, 0x06, 0x60, 0x06,
 0x61, 0x86, 0x61, 0x86, 0x61, 0x86, 0x61, 0x86,
 0x30, 0x0C, 0x30, 0x0C, 0x30, 0x0C, 0x30, 0x0C,
 0x30, 0x0C, 0x30, 0x0C, 0x30, 0x0C, 0x30, 0x0C,
 0x18, 0x18, 0x18, 0x18, 0x18, 0x18, 0x18, 0x18,
 0x18, 0x18, 0x18, 0x18, 0x1F, 0xF8, 0x1F, 0xF8},   /* 33 A */

{0xFF, 0xF0, 0xFF, 0xFC, 0xC0, 0x1E, 0xC0, 0x06,
 0xC0, 0x07, 0xC3, 0x83, 0xC3, 0xC3, 0xC3, 0xC3,
 0xC3, 0xC3, 0xC3, 0xC3, 0xC3, 0xC3, 0xC3, 0xC3,
 0xC3, 0xC3, 0xC3, 0x87, 0xC0, 0x06, 0xC0, 0x0E,
 0xC0, 0x0E, 0xC0, 0x06, 0xC3, 0x87, 0xC3, 0xC3,
 0xC3, 0xC3, 0xC3, 0xC3, 0xC3, 0xC3, 0xC3, 0xC3,
 0xC3, 0xC3, 0xC3, 0xC3, 0xC3, 0x83, 0xC0, 0x07,
 0xC0, 0x06, 0xC0, 0x1E, 0xFF, 0xFC, 0xFF, 0xF0},   /* 34 B */

{0x0F, 0xF0, 0x3F, 0xFC, 0x78, 0x1E, 0x60, 0x06,
 0xE0, 0x07, 0xC1, 0x83, 0xC3, 0xC3, 0xC3, 0xC3,
 0xC3, 0xC3, 0xC3, 0xC3, 0xC3, 0xC3, 0xC3, 0xC3,
 0xC3, 0xFF, 0xC3, 0xFF, 0xC3, 0x00, 0xC3, 0x00,
 0xC3, 0x00, 0xC3, 0x00, 0xC3, 0xFF, 0xC3, 0xFF,
 0xC3, 0xC3, 0xC3, 0xC3, 0xC3, 0xC3, 0xC3, 0xC3,
 0xC3, 0xC3, 0xC3, 0xC3, 0xC1, 0x83, 0xE0, 0x07,
 0x60, 0x06, 0x78, 0x1E, 0x3F, 0xFC, 0x0F, 0xF0},   /* 35 C */

{0xFF, 0xF0, 0xFF, 0xFC, 0xC0, 0x1E, 0xC0, 0x06,
 0xC0, 0x07, 0xC3, 0x83, 0xC3, 0xC3, 0xC3, 0xC3,
 0xC3, 0xC3, 0xC3, 0xC3, 0xC3, 0xC3, 0xC3, 0xC3,
 0xC3, 0xC3, 0xC3, 0xC3, 0xC3, 0xC3, 0xC3, 0xC3,
 0xC3, 0xC3, 0xC3, 0xC3, 0xC3, 0xC3, 0xC3, 0xC3,
 0xC3, 0xC3, 0xC3, 0xC3, 0xC3, 0xC3, 0xC3, 0xC3,
 0xC3, 0xC3, 0xC3, 0xC3, 0xC3, 0x83, 0xC0, 0x07,
 0xC0, 0x06, 0xC0, 0x1E, 0xFF, 0xFC, 0xFF, 0xF0},   /* 36 D */

{0xFF, 0xFF, 0xFF, 0xFF, 0xC0, 0x03, 0xC0, 0x03,
 0xC0, 0x03, 0xC0, 0x03, 0xC3, 0xFF, 0xC3, 0xFF,
 0xC3, 0x00, 0xC3, 0x00, 0xC3, 0x00, 0xC3, 0x00,
 0xC3, 0xFC, 0xC3, 0xFC, 0xC0, 0x0C, 0xC0, 0x0C,
 0xC0, 0x0C, 0xC0, 0x0C, 0xC3, 0xFC, 0xC3, 0xFC,
 0xC3, 0x00, 0xC3, 0x00, 0xC3, 0x00, 0xC3, 0x00,
 0xC3, 0xFF, 0xC3, 0xFF, 0xC0, 0x03, 0xC0, 0x03,
 0xC0, 0x03, 0xC0, 0x03, 0xFF, 0xFF, 0xFF, 0xFF},   /* 37 E */

{0xFF, 0x00, 0xFF, 0x00, 0xC3, 0x00, 0xC3, 0x00,
 0xC3, 0x00, 0xC3, 0x00, 0xC3, 0x00, 0xC3, 0x00,
 0xC3, 0x00, 0xC3, 0x00, 0xC3, 0x00, 0xC3, 0x00,
 0xC3, 0xFC, 0xC3, 0xFC, 0xC0, 0x0C, 0xC0, 0x0C,
 0xC0, 0x0C, 0xC0, 0x0C, 0xC3, 0xFC, 0xC3, 0xFC,
 0xC3, 0x00, 0xC3, 0x00, 0xC3, 0x00, 0xC3, 0x00,
 0xC3, 0xFF, 0xC3, 0xFF, 0xC0, 0x03, 0xC0, 0x03,
 0xC0, 0x03, 0xC0, 0x03, 0xFF, 0xFF, 0xFF, 0xFF},   /* 38 F */

{0x0F, 0xF0, 0x3F, 0xFC, 0x78, 0x1E, 0x60, 0x06,
 0xE0, 0x07, 0xC1, 0x83, 0xC3, 0xC3, 0xC3, 0xC3,
 0xC3, 0xC3, 0xC3, 0xC3, 0xC3, 0xC3, 0xC3, 0xC3,
 0xC3, 0xC3, 0xC3, 0xC3, 0xC3, 0x03, 0xC3, 0x03,
 0xC3, 0x03, 0xC3, 0x03, 0xC3, 0xFF, 0xC3, 0xFF,
 0xC3, 0xC3, 0xC3, 0xC3, 0xC3, 0xC3, 0xC3, 0xC3,
 0xC3, 0xC3, 0xC3, 0xC3, 0xC1, 0x83, 0xE0, 0x07,
 0x60, 0x06, 0x78, 0x1E, 0x3F, 0xFC, 0x0F, 0xF0},   /* 39 G */

{0xFF, 0xFF, 0xFF, 0xFF, 0xC3, 0xC3, 0xC3, 0xC3,
 0xC3, 0xC3, 0xC3, 0xC3, 0xC3, 0xC3, 0xC3, 0xC3,
 0xC3, 0xC3, 0xC3, 0xC3, 0xC3, 0xC3, 0xC3, 0xC3,
 0xC3, 0xC3, 0xC3, 0xC3, 0xC0, 0x03, 0xC0, 0x03,
 0xC0, 0x03, 0xC0, 0x03, 0xC3, 0xC3, 0xC3, 0xC3,
 0xC3, 0xC3, 0xC3, 0xC3, 0xC3, 0xC3, 0xC3, 0xC3,
 0xC3, 0xC3, 0xC3, 0xC3, 0xC3, 0xC3, 0xC3, 0xC3,
 0xC3, 0xC3, 0xC3, 0xC3, 0xFF, 0xFF, 0xFF, 0xFF},   /* 40 H */

{0x3F, 0xFC, 0x3F, 0xFC, 0x30, 0x0C, 0x30, 0x0C,
 0x30, 0x0C, 0x30, 0x0C, 0x3C, 0x3C, 0x3C, 0x3C,
 0x0C, 0x30, 0x0C, 0x30, 0x0C, 0x30, 0x0C, 0x30,
 0x0C, 0x30, 0x0C, 0x30, 0x0C, 0x30, 0x0C, 0x30,
 0x0C, 0x30, 0x0C, 0x30, 0x0C, 0x30, 0x0C, 0x30,
 0x0C, 0x30, 0x0C, 0x30, 0x0C, 0x30, 0x0C, 0x30,
 0x3C, 0x3C, 0x3C, 0x3C, 0x30, 0x0C, 0x30, 0x0C,
 0x30, 0x0C, 0x30, 0x0C, 0x3F, 0xFC, 0x3F, 0xFC},   /* 41 I */

{0x0F, 0xF0, 0x3F, 0xFC, 0x78, 0x1E, 0x60, 0x06,
 0xE0, 0x07, 0xC1, 0x83, 0xC3, 0xC3, 0xC3, 0xC3,
 0xC3, 0xC3, 0xC3, 0xC3, 0xC3, 0xC3, 0xC3, 0xC3,
 0xFF, 0xC3, 0xFF, 0xC3, 0x00, 0xC3, 0x00, 0xC3,
 0x00, 0xC3, 0x00, 0xC3, 0x00, 0xC3, 0x00, 0xC3,
 0x00, 0xC3, 0x00, 0xC3, 0x00, 0xC3, 0x00, 0xC3,
 0x00, 0xC3, 0x00, 0xC3, 0x00, 0xC3, 0x00, 0xC3,
 0x00, 0xC3, 0x00, 0xC3, 0x00, 0xFF, 0x00, 0xFF},   /* 42 J */

{0xFF, 0xFF, 0xFF, 0xFF, 0xC3, 0xC3, 0xC3, 0xC3,
 0xC3, 0xC3, 0xC3, 0x87, 0xC3, 0x86, 0xC3, 0x86,
 0xC3, 0x0E, 0xC3, 0x0C, 0xC3, 0x0C, 0xC2, 0x1C,
 0xC2, 0x18, 0xC2, 0x18, 0xC0, 0x38, 0xC0, 0x30,
 0xC0, 0x30, 0xC0, 0x38, 0xC2, 0x18, 0xC2, 0x18,
 0xC2, 0x1C, 0xC3, 0x0C, 0xC3, 0x0C, 0xC3, 0x0E,
 0xC3, 0x86, 0xC3, 0x86, 0xC3, 0x87, 0xC3, 0xC3,
 0xC3, 0xC3, 0xC3, 0xC3, 0xFF, 0xFF, 0xFF, 0xFF},   /* 43 K */

{0xFF, 0xFF, 0xFF, 0xFF, 0xC0, 0x03, 0xC0, 0x03,
 0xC0, 0x03, 0xC0, 0x03, 0xC3, 0xFF, 0xC3, 0xFF,
 0xC3, 0x00, 0xC3, 0x00, 0xC3, 0x00, 0xC3, 0x00,
 0xC3, 0x00, 0xC3, 0x00, 0xC3, 0x00, 0xC3, 0x00,
 0xC3, 0x00, 0xC3, 0x00, 0xC3, 0x00, 0xC3, 0x00,
 0xC3, 0x00, 0xC3, 0x00, 0xC3, 0x00, 0xC3, 0x00,
 0xC3, 0x00, 0xC3, 0x00, 0xC3, 0x00, 0xC3, 0x00,
 0xC3, 0x00, 0xC3, 0x00, 0xFF, 0x00, 0xFF, 0x00},   /* 44 L */

{0xFF, 0xFF, 0xFF, 0xFF, 0xC3, 0xC3, 0xC3, 0xC3,
 0xC3, 0xC3, 0xC3, 0xC3, 0xC3, 0xC3, 0xC3, 0xC3,
 0xC3, 0xC3, 0xC3, 0xC3, 0xC2, 0x43, 0xC2, 0x43,
 0xC2, 0x43, 0xC2, 0x43, 0xC0, 0x03, 0xC0, 0x03,
 0xC0, 0x03, 0xC0, 0x03, 0xC0, 0x03, 0xC0, 0x03,
 0xC0, 0x03, 0xC0, 0x03, 0xC1, 0x83, 0xC1, 0x83,
 0xC1, 0x83, 0xC1, 0x83, 0xC3, 0xC3, 0xC3, 0xC3,
 0xC3, 0xC3, 0xC3, 0xC3, 0xFF, 0xFF, 0xFF, 0xFF},   /* 45 M */

{0xFF, 0xFF, 0xFF, 0xFF, 0xC3, 0x83, 0xC3, 0x83,
 0xC3, 0x83, 0xC3, 0x83, 0xC3, 0x03, 0xC3, 0x03,
 0xC3, 0x03, 0xC3, 0x03, 0xC2, 0x03, 0xC2, 0x03,
 0xC2, 0x03, 0xC2, 0x03, 0xC0, 0x03, 0xC0, 0x03,
 0xC0, 0x03, 0xC0, 0x03, 0xC0, 0x43, 0xC0, 0x43,
 0xC0, 0x43, 0xC0, 0x43, 0xC0, 0xC3, 0xC0, 0xC3,
 0xC0, 0xC3, 0xC0, 0xC3, 0xC1, 0xC3, 0xC1, 0xC3,
 0xC1, 0xC3, 0xC1, 0xC3, 0xFF, 0xFF, 0xFF, 0xFF},   /* 46 N */

{0x0F, 0xF0, 0x3F, 0xFC, 0x78, 0x1E, 0x60, 0x06,
 0xE0, 0x07, 0xC1, 0x83, 0xC3, 0xC3, 0xC3, 0xC3,
 0xC3, 0xC3, 0xC3, 0xC3, 0xC3, 0xC3, 0xC3, 0xC3,
 0xC3, 0xC3, 0xC3, 0xC3, 0xC3, 0xC3, 0xC3, 0xC3,
 0xC3, 0xC3, 0xC3, 0xC3, 0xC3, 0xC3, 0xC3, 0xC3,
 0xC3, 0xC3, 0xC3, 0xC3, 0xC3, 0xC3, 0xC3, 0xC3,
 0xC3, 0xC3, 0xC3, 0xC3, 0xC1, 0x83, 0xE0, 0x07,
 0x60, 0x06, 0x78, 0x1E, 0x3F, 0xFC, 0x0F, 0xF0},   /* 47 O */

{0xFF, 0x00, 0xFF, 0x00, 0xC3, 0x00, 0xC3, 0x00,
 0xC3, 0x00, 0xC3, 0x00, 0xC3, 0x00, 0xC3, 0x00,
 0xC3, 0x00, 0xC3, 0x00, 0xC3, 0x00, 0xC3, 0x00,
 0xC3, 0xF0, 0xC3, 0xFC, 0xC0, 0x1E, 0xC0, 0x06,
 0xC0, 0x07, 0xC3, 0x83, 0xC3, 0xC3, 0xC3, 0xC3,
 0xC3, 0xC3, 0xC3, 0xC3, 0xC3, 0xC3, 0xC3, 0xC3,
 0xC3, 0xC3, 0xC3, 0xC3, 0xC3, 0x83, 0xC0, 0x07,
 0xC0, 0x06, 0xC0, 0x1E, 0xFF, 0xFC, 0xFF, 0xF0},   /* 48 P */

{0x0F, 0xFF, 0x3F, 0xFF, 0x78, 0x13, 0x60, 0x03,
 0xE0, 0x03, 0xC1, 0x03, 0xC3, 0x07, 0xC3, 0x03,
 0xC3, 0xC3, 0xC3, 0xC3, 0xC3, 0xC3, 0xC3, 0xC3,
 0xC3, 0xC3, 0xC3, 0xC3, 0xC3, 0xC3, 0xC3, 0xC3,
 0xC3, 0xC3, 0xC3, 0xC3, 0xC3, 0xC3, 0xC3, 0xC3,
 0xC3, 0xC3, 0xC3, 0xC3, 0xC3, 0xC3, 0xC3, 0xC3,
 0xC3, 0xC3, 0xC3, 0xC3, 0xC1, 0x83, 0xE0, 0x07,
 0x60, 0x06, 0x78, 0x1E, 0x3F, 0xFC, 0x0F, 0xF0},   /* 49 Q */

{0xFF, 0xFF, 0xFF, 0xFF, 0xC3, 0xC3, 0xC3, 0xC3,
 0xC3, 0xC3, 0xC3, 0x87, 0xC3, 0x86, 0xC3, 0x86,
 0xC3, 0x0E, 0xC3, 0x0C, 0xC3, 0x0C, 0xC2, 0x1C,
 0xC2, 0x18, 0xC2, 0x18, 0xC0, 0x3C, 0xC0, 0x0E,
 0xC0, 0x06, 0xC3, 0x87, 0xC3, 0xC3, 0xC3, 0xC3,
 0xC3, 0xC3, 0xC3, 0xC3, 0xC3, 0xC3, 0xC3, 0xC3,
 0xC3, 0xC3, 0xC3, 0xC3, 0xC3, 0x83, 0xC0, 0x07,
 0xC0, 0x06, 0xC0, 0x1E, 0xFF, 0xFC, 0xFF, 0xF0},   /* 50 R */

{0x0F, 0xF0, 0x3F, 0xFC, 0x78, 0x1E, 0x60, 0x06,
 0xE0, 0x07, 0xC1, 0x83, 0xC3, 0xC3, 0xC3, 0xC3,
 0xC3, 0xC3, 0xC3, 0xC3, 0xC3, 0xC3, 0xC3, 0xC3,
 0xFF, 0x87, 0xFF, 0x06, 0x0E, 0x0E, 0x1C, 0x1C,
 0x38, 0x38, 0x70, 0x70, 0x60, 0xFF, 0xE1, 0xFF,
 0xC3, 0xC3, 0xC3, 0xC3, 0xC3, 0xC3, 0xC3, 0xC3,
 0xC3, 0xC3, 0xC3, 0xC3, 0xC1, 0x83, 0xE0, 0x07,
 0x60, 0x06, 0x78, 0x1E, 0x3F, 0xFC, 0x0F, 0xF0},   /* 51 S */

{0x0F, 0xF0, 0x0F, 0xF0, 0x0C, 0x30, 0x0C, 0x30,
 0x0C, 0x30, 0x0C, 0x30, 0x0C, 0x30, 0x0C, 0x30,
 0x0C, 0x30, 0x0C, 0x30, 0x0C, 0x30, 0x0C, 0x30,
 0x0C, 0x30, 0x0C, 0x30, 0x0C, 0x30, 0x0C, 0x30,
 0x0C, 0x30, 0x0C, 0x30, 0x0C, 0x30, 0x0C, 0x30,
 0x0C, 0x30, 0x0C, 0x30, 0x0C, 0x30, 0x0C, 0x30,
 0xFC, 0x3F, 0xFC, 0x3F, 0xC0, 0x03, 0xC0, 0x03,
 0xC0, 0x03, 0xC0, 0x03, 0xFF, 0xFF, 0xFF, 0xFF},   /* 52 T */

{0x0F, 0xF0, 0x3F, 0xFC, 0x78, 0x1E, 0x60, 0x06,
 0xE0, 0x07, 0xC1, 0x83, 0xC3, 0xC3, 0xC3, 0xC3,
 0xC3, 0xC3, 0xC3, 0xC3, 0xC3, 0xC3, 0xC3, 0xC3,
 0xC3, 0xC3, 0xC3, 0xC3, 0xC3, 0xC3, 0xC3, 0xC3,
 0xC3, 0xC3, 0xC3, 0xC3, 0xC3, 0xC3, 0xC3, 0xC3,
 0xC3, 0xC3, 0xC3, 0xC3, 0xC3, 0xC3, 0xC3, 0xC3,
 0xC3, 0xC3, 0xC3, 0xC3, 0xC3, 0xC3, 0xC3, 0xC3,
 0xC3, 0xC3, 0xC3, 0xC3, 0xFF, 0xFF, 0xFF, 0xFF},   /* 53 U */

{0x1F, 0xF8, 0x1F, 0xF8, 0x18, 0x18, 0x18, 0x18,
 0x18, 0x18, 0x18, 0x18, 0x18, 0x18, 0x18, 0x18,
 0x30, 0x0C, 0x30, 0x0C, 0x30, 0x0C, 0x30, 0x0C,
 0x30, 0x0C, 0x30, 0x0C, 0x30, 0x0C, 0x30, 0x0C,
 0x61, 0x86, 0x61, 0x86, 0x61, 0x86, 0x61, 0x86,
 0x61, 0x86, 0x61, 0x86, 0x61, 0x86, 0x61, 0x86,
 0xC3, 0xC3, 0xC3, 0xC3, 0xC3, 0xC3, 0xC3, 0xC3,
 0xC3, 0xC3, 0xC3, 0xC3, 0xFF, 0xFF, 0xFF, 0xFF},   /* 54 V */

{0xFF, 0xFF, 0xFF, 0xFF, 0xC3, 0xC3, 0xC3, 0xC3,
 0xC3, 0xC3, 0xC3, 0xC3, 0xC1, 0x83, 0xC1, 0x83,
 0xC1, 0x83, 0xC1, 0x83, 0xC0, 0x03, 0xC0, 0x03,
 0xC0, 0x03, 0xC0, 0x03, 0xC0, 0x03, 0xC0, 0x03,
 0xC0, 0x03, 0xC0, 0x03, 0xC2, 0x43, 0xC2, 0x43,
 0xC2, 0x43, 0xC2, 0x43, 0xC3, 0xC3, 0xC3, 0xC3,
 0xC3, 0xC3, 0xC3, 0xC3, 0xC3, 0xC3, 0xC3, 0xC3,
 0xC3, 0xC3, 0xC3, 0xC3, 0xFF, 0xFF, 0xFF, 0xFF},   /* 55 W */

{0xFF, 0xFF, 0xFF, 0xFF, 0xC3, 0xC3, 0xC3, 0xC3,
 0xC3, 0xC3, 0xE1, 0x87, 0x61, 0x86, 0x61, 0x86,
 0x70, 0x0E, 0x30, 0x0C, 0x30, 0x0C, 0x38, 0x1C,
 0x18, 0x18, 0x18, 0x18, 0x1C, 0x38, 0x0C, 0x30,
 0x0C, 0x30, 0x1C, 0x38, 0x18, 0x18, 0x18, 0x18,
 0x38, 0x1C, 0x30, 0x0C, 0x30, 0x0C, 0x70, 0x0E,
 0x61, 0x86, 0x61, 0x86, 0xE1, 0x87, 0xC3, 0xC3,
 0xC3, 0xC3, 0xC3, 0xC3, 0xFF, 0xFF, 0xFF, 0xFF},   /* 56 X */

{0x0F, 0xF0, 0x0F, 0xF0, 0x0C, 0x30, 0x0C, 0x30,
 0x0C, 0x30, 0x0C, 0x30, 0x0C, 0x30, 0x0C, 0x30,
 0x0C, 0x30, 0x0C, 0x30, 0x0C, 0x30, 0x0C, 0x30,
 0x0C, 0x30, 0x0C, 0x30, 0x0C, 0x30, 0x0C, 0x30,
 0x0C, 0x30, 0x1C, 0x38, 0x18, 0x18, 0x18, 0x18,
 0x38, 0x1C, 0x30, 0x0C, 0x30, 0x0C, 0x70, 0x0E,
 0x61, 0x86, 0x61, 0x86, 0xE1, 0x87, 0xC3, 0xC3,
 0xC3, 0xC3, 0xC3, 0xC3, 0xFF, 0xFF, 0xFF, 0xFF},   /* 57 Y */

{0xFF, 0xFF, 0xFF, 0xFF, 0xC0, 0x03, 0xC0, 0x03,
 0xC0, 0x03, 0xC0, 0x03, 0xC3, 0xFF, 0xC3, 0xFF,
 0xC3, 0x80, 0xE1, 0x80, 0x61, 0xC0, 0x70, 0xC0,
 0x30, 0xE0, 0x38, 0x60, 0x18, 0x70, 0x1C, 0x30,
 0x0C, 0x38, 0x0E, 0x18, 0x06, 0x1C, 0x07, 0x0C,
 0x03, 0x0E, 0x03, 0x86, 0x01, 0x87, 0x01, 0xC3,
 0xFF, 0xC3, 0xFF, 0xC3, 0xC0, 0x03, 0xC0, 0x03,
 0xC0, 0x03, 0xC0, 0x03, 0xFF, 0xFF, 0xFF, 0xFF}    /* 58 Z */
};

/* Hex definition for each character (fill colour)... */
GLubyte l_f_space[] =
{0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00};   /* 00 space */

GLubyte l_f_exclamation[] =
{0x00, 0x00, 0x00, 0x00, 0x03, 0xC0, 0x03, 0xC0,
 0x03, 0xC0, 0x03, 0xC0, 0x00, 0x00, 0x00, 0x00,
 0x03, 0xC0, 0x03, 0xC0, 0x03, 0xC0, 0x03, 0xC0,
 0x03, 0xC0, 0x03, 0xC0, 0x03, 0xC0, 0x03, 0xC0,
 0x03, 0xC0, 0x03, 0xC0, 0x03, 0xC0, 0x03, 0xC0,
 0x03, 0xC0, 0x03, 0xC0, 0x03, 0xC0, 0x03, 0xC0,
 0x03, 0xC0, 0x03, 0xC0, 0x03, 0xC0, 0x03, 0xC0,
 0x03, 0xC0, 0x03, 0xC0, 0x00, 0x00, 0x00, 0x00};   /* 01 ! */

GLubyte l_f_double_quote[] =
{0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x20, 0x20,
 0x30, 0x30, 0x38, 0x38, 0x3C, 0x3C, 0x3C, 0x3C,
 0x3C, 0x3C, 0x3C, 0x3C, 0x00, 0x00, 0x00, 0x00};   /* 02 " */

GLubyte l_f_pound_sign[] =
{0x00, 0x00, 0x00, 0x00, 0x3F, 0xFC, 0x3F, 0xFC,
 0x3F, 0xFC, 0x3F, 0xFC, 0x0F, 0x3C, 0x0F, 0x3C,
 0x0F, 0x00, 0x0F, 0x00, 0x0F, 0x00, 0x0F, 0x00,
 0x0F, 0x00, 0x0F, 0x00, 0x3F, 0xF0, 0x3F, 0xF0,
 0x3F, 0xF0, 0x3F, 0xF0, 0x0F, 0x00, 0x0F, 0x00,
 0x0F, 0x3C, 0x0F, 0x3C, 0x0F, 0x3C, 0x0F, 0x3C,
 0x0F, 0x3C, 0x0F, 0x3C, 0x0F, 0x3C, 0x07, 0xF8,
 0x07, 0xF8, 0x01, 0xE0, 0x00, 0x00, 0x00, 0x00};   /* 04 $ () */

GLubyte l_f_single_quote[] =
{0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x02, 0x00,
 0x03, 0x00, 0x03, 0x80, 0x03, 0xC0, 0x03, 0xC0,
 0x03, 0xC0, 0x03, 0xC0, 0x00, 0x00, 0x00, 0x00};   /* 07 ' */

GLubyte l_f_plus_sign[] =
{0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
 0x00, 0x00, 0x00, 0x00, 0x03, 0xC0, 0x03, 0xC0,
 0x03, 0xC0, 0x03, 0xC0, 0x3F, 0xFC, 0x3F, 0xFC,
 0x3F, 0xFC, 0x3F, 0xFC, 0x03, 0xC0, 0x03, 0xC0,
 0x03, 0xC0, 0x03, 0xC0, 0x00, 0x00, 0x00, 0x00,
 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00};   /* 11 + */

GLubyte l_f_hyphen[] =
{0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
 0x00, 0x00, 0x00, 0x00, 0x3F, 0xFC, 0x3F, 0xFC,
 0x3F, 0xFC, 0x3F, 0xFC, 0x00, 0x00, 0x00, 0x00,
 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00};   /* 13 - */

GLubyte l_f_fullstop[] =
{0x00, 0x00, 0x00, 0x00, 0x03, 0xC0, 0x03, 0xC0,
 0x03, 0xC0, 0x03, 0xC0, 0x00, 0x00, 0x00, 0x00,
 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00};   /* 14 . */

GLubyte l_f_numbers[][64] = {
{0x00, 0x00, 0x00, 0x00, 0x07, 0xE0, 0x1F, 0xF8,
 0x1F, 0xF8, 0x3E, 0x7C, 0x3C, 0x3C, 0x3C, 0x3C,
 0x3C, 0x3C, 0x3C, 0x3C, 0x3C, 0x3C, 0x3C, 0x3C,
 0x3C, 0x3C, 0x3C, 0x3C, 0x3C, 0x3C, 0x3C, 0x3C,
 0x3C, 0x3C, 0x3C, 0x3C, 0x3C, 0x3C, 0x3C, 0x3C,
 0x3C, 0x3C, 0x3C, 0x3C, 0x3C, 0x3C, 0x3C, 0x3C,
 0x3C, 0x3C, 0x3C, 0x3C, 0x3E, 0x7C, 0x1F, 0xF8,
 0x1F, 0xF8, 0x07, 0xE0, 0x00, 0x00, 0x00, 0x00},   /* 16 0 */

{0x00, 0x00, 0x00, 0x00, 0x0F, 0xF0, 0x0F, 0xF0,
 0x0F, 0xF0, 0x0F, 0xF0, 0x03, 0xC0, 0x03, 0xC0,
 0x03, 0xC0, 0x03, 0xC0, 0x03, 0xC0, 0x03, 0xC0,
 0x03, 0xC0, 0x03, 0xC0, 0x03, 0xC0, 0x03, 0xC0,
 0x03, 0xC0, 0x03, 0xC0, 0x03, 0xC0, 0x03, 0xC0,
 0x03, 0xC0, 0x03, 0xC0, 0x03, 0xC0, 0x03, 0xC0,
 0x03, 0xC0, 0x03, 0xC0, 0x0F, 0xC0, 0x0F, 0xC0,
 0x0F, 0xC0, 0x0F, 0xC0, 0x00, 0x00, 0x00, 0x00},   /* 17 1 */

{0x00, 0x00, 0x00, 0x00, 0x3F, 0xFC, 0x3F, 0xFC,
 0x3F, 0xFC, 0x3F, 0xFC, 0x3C, 0x3C, 0x3C, 0x3C,
 0x3C, 0x3C, 0x3C, 0x3C, 0x3C, 0x00, 0x3C, 0x00,
 0x1E, 0x00, 0x1F, 0x00, 0x0F, 0x80, 0x07, 0xC0,
 0x03, 0xE0, 0x01, 0xF0, 0x00, 0xF8, 0x00, 0x78,
 0x00, 0x3C, 0x00, 0x3C, 0x3C, 0x3C, 0x3C, 0x3C,
 0x3C, 0x3C, 0x3C, 0x3C, 0x3E, 0x7C, 0x1F, 0xF8,
 0x1F, 0xF8, 0x07, 0xE0, 0x00, 0x00, 0x00, 0x00},   /* 18 2 */

{0x00, 0x00, 0x00, 0x00, 0x07, 0xE0, 0x1F, 0xF8,
 0x1F, 0xF8, 0x3E, 0x7C, 0x3C, 0x3C, 0x3C, 0x3C,
 0x3C, 0x3C, 0x3C, 0x3C, 0x00, 0x3C, 0x00, 0x3C,
 0x00, 0x3C, 0x00, 0x78, 0x03, 0xF8, 0x03, 0xF0,
 0x03, 0xF0, 0x03, 0xF8, 0x00, 0x78, 0x00, 0x3C,
 0x00, 0x3C, 0x00, 0x3C, 0x3C, 0x3C, 0x3C, 0x3C,
 0x3C, 0x3C, 0x3C, 0x3C, 0x3E, 0x7C, 0x1F, 0xF8,
 0x1F, 0xF8, 0x07, 0xE0, 0x00, 0x00, 0x00, 0x00},   /* 19 3 */

{0x00, 0x00, 0x00, 0x00, 0x00, 0xF0, 0x00, 0xF0,
 0x00, 0xF0, 0x00, 0xF0, 0x00, 0xF0, 0x00, 0xF0,
 0x00, 0xF0, 0x00, 0xF0, 0x3F, 0xFC, 0x3F, 0xFC,
 0x3F, 0xFC, 0x3F, 0xFC, 0x3C, 0xF0, 0x3C, 0xF0,
 0x3C, 0xF0, 0x3C, 0xF0, 0x3C, 0xF0, 0x3C, 0xF0,
 0x3C, 0xF0, 0x3C, 0xF0, 0x3C, 0xF0, 0x3C, 0xF0,
 0x3C, 0xF0, 0x3C, 0xF0, 0x3C, 0xF0, 0x3C, 0xF0,
 0x3C, 0xF0, 0x3C, 0xF0, 0x00, 0x00, 0x00, 0x00},   /* 20 4 */

{0x00, 0x00, 0x00, 0x00, 0x07, 0xE0, 0x1F, 0xF8,
 0x1F, 0xF8, 0x3E, 0x7C, 0x3C, 0x3C, 0x3C, 0x3C,
 0x3C, 0x3C, 0x3C, 0x3C, 0x00, 0x3C, 0x00, 0x3C,
 0x00, 0x3C, 0x00, 0x3C, 0x3C, 0x7C, 0x3F, 0xF8,
 0x3F, 0xF8, 0x3F, 0xE0, 0x3C, 0x00, 0x3C, 0x00,
 0x3C, 0x00, 0x3C, 0x00, 0x3C, 0x00, 0x3C, 0x00,
 0x3C, 0x00, 0x3C, 0x00, 0x3F, 0xFC, 0x3F, 0xFC,
 0x3F, 0xFC, 0x3F, 0xFC, 0x00, 0x00, 0x00, 0x00},   /* 21 5 */

{0x00, 0x00, 0x00, 0x00, 0x07, 0xE0, 0x1F, 0xF8,
 0x1F, 0xF8, 0x3E, 0x7C, 0x3C, 0x3C, 0x3C, 0x3C,
 0x3C, 0x3C, 0x3C, 0x3C, 0x3C, 0x3C, 0x3C, 0x3C,
 0x3C, 0x3C, 0x3C, 0x3C, 0x3E, 0x7C, 0x3F, 0xF8,
 0x3F, 0xF8, 0x3F, 0xE0, 0x3C, 0x00, 0x3C, 0x00,
 0x3C, 0x00, 0x3C, 0x00, 0x3C, 0x3C, 0x3C, 0x3C,
 0x3C, 0x3C, 0x3C, 0x3C, 0x3E, 0x7C, 0x1F, 0xF8,
 0x1F, 0xF8, 0x07, 0xE0, 0x00, 0x00, 0x00, 0x00},   /* 22 6 */

{0x00, 0x00, 0x00, 0x00, 0x03, 0xC0, 0x03, 0xC0,
 0x03, 0xC0, 0x03, 0xC0, 0x03, 0xC0, 0x03, 0xC0,
 0x03, 0xC0, 0x03, 0xC0, 0x03, 0xC0, 0x03, 0xC0,
 0x03, 0xC0, 0x03, 0xC0, 0x03, 0xC0, 0x01, 0xE0,
 0x01, 0xE0, 0x01, 0xE0, 0x01, 0xE0, 0x00, 0xF0,
 0x00, 0xF0, 0x00, 0x78, 0x3C, 0x78, 0x3C, 0x3C,
 0x3C, 0x3C, 0x3C, 0x3C, 0x3F, 0xFC, 0x3F, 0xFC,
 0x3F, 0xFC, 0x3F, 0xFC, 0x00, 0x00, 0x00, 0x00},   /* 23 7 */

{0x00, 0x00, 0x00, 0x00, 0x07, 0xE0, 0x1F, 0xF8,
 0x1F, 0xF8, 0x3E, 0x7C, 0x3C, 0x3C, 0x3C, 0x3C,
 0x3C, 0x3C, 0x3C, 0x3C, 0x3C, 0x3C, 0x3C, 0x3C,
 0x3C, 0x3C, 0x1E, 0x78, 0x1F, 0xF8, 0x0F, 0xF0,
 0x0F, 0xF0, 0x1F, 0xF8, 0x1E, 0x78, 0x3C, 0x3C,
 0x3C, 0x3C, 0x3C, 0x3C, 0x3C, 0x3C, 0x3C, 0x3C,
 0x3C, 0x3C, 0x3C, 0x3C, 0x3E, 0x7C, 0x1F, 0xF8,
 0x1F, 0xF8, 0x07, 0xE0, 0x00, 0x00, 0x00, 0x00},   /* 24 8 */

{0x00, 0x00, 0x00, 0x00, 0x07, 0xE0, 0x1F, 0xF8,
 0x1F, 0xF8, 0x3E, 0x7C, 0x3C, 0x3C, 0x3C, 0x3C,
 0x3C, 0x3C, 0x3C, 0x3C, 0x00, 0x3C, 0x00, 0x3C,
 0x00, 0x3C, 0x00, 0x3C, 0x07, 0xFC, 0x1F, 0xFC,
 0x1F, 0xFC, 0x3E, 0x7C, 0x3C, 0x3C, 0x3C, 0x3C,
 0x3C, 0x3C, 0x3C, 0x3C, 0x3C, 0x3C, 0x3C, 0x3C,
 0x3C, 0x3C, 0x3C, 0x3C, 0x3E, 0x7C, 0x1F, 0xF8,
 0x1F, 0xF8, 0x07, 0xE0, 0x00, 0x00, 0x00, 0x00}    /* 25 9 */
};

GLubyte l_f_colon[] =
{0x00, 0x00, 0x00, 0x00, 0x03, 0xC0, 0x03, 0xC0,
 0x03, 0xC0, 0x03, 0xC0, 0x00, 0x00, 0x00, 0x00,
 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
 0x00, 0x00, 0x00, 0x00, 0x03, 0xC0, 0x03, 0xC0,
 0x03, 0xC0, 0x03, 0xC0, 0x00, 0x00, 0x00, 0x00,
 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00};   /* 26 : */

GLubyte l_f_less_than[] =
{0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
 0x00, 0x00, 0x00, 0x04, 0x00, 0x0C, 0x00, 0x1C,
 0x00, 0x3C, 0x00, 0x7C, 0x00, 0xFC, 0x01, 0xFC,
 0x03, 0xFC, 0x07, 0xFC, 0x0F, 0xFC, 0x1F, 0xFC,
 0x1F, 0xFC, 0x0F, 0xFC, 0x07, 0xFC, 0x03, 0xFC,
 0x01, 0xFC, 0x00, 0xFC, 0x00, 0x7C, 0x00, 0x3C,
 0x00, 0x1C, 0x00, 0x0C, 0x00, 0x04, 0x00, 0x00,
 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00};   /* 28 < */

GLubyte l_f_greater_than[] =
{0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
 0x00, 0x00, 0x20, 0x00, 0x30, 0x00, 0x38, 0x00,
 0x3C, 0x00, 0x3E, 0x00, 0x3F, 0x00, 0x3F, 0x80,
 0x3F, 0xC0, 0x3F, 0xE0, 0x3F, 0xF0, 0x3F, 0xF8,
 0x3F, 0xF8, 0x3F, 0xF0, 0x3F, 0xE0, 0x3F, 0xC0,
 0x3F, 0x80, 0x3F, 0x00, 0x3E, 0x00, 0x3C, 0x00,
 0x38, 0x00, 0x30, 0x00, 0x20, 0x00, 0x00, 0x00,
 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00};   /* 30 > */

GLubyte l_f_question_mark[] =
{0x00, 0x00, 0x00, 0x00, 0x03, 0xC0, 0x03, 0xC0,
 0x03, 0xC0, 0x03, 0xC0, 0x00, 0x00, 0x00, 0x00,
 0x03, 0xC0, 0x03, 0xC0, 0x03, 0xC0, 0x03, 0xC0,
 0x03, 0xC0, 0x03, 0xC0, 0x03, 0xC0, 0x03, 0xC0,
 0x01, 0xE0, 0x01, 0xE0, 0x00, 0xF0, 0x00, 0x78,
 0x3C, 0x3C, 0x3C, 0x3C, 0x3C, 0x3C, 0x3C, 0x3C,
 0x3C, 0x3C, 0x3C, 0x3C, 0x3E, 0x7C, 0x1F, 0xF8,
 0x1F, 0xF8, 0x07, 0xE0, 0x00, 0x00, 0x00, 0x00};   /* 31 ? */

GLubyte l_f_letters[][64] = {
{0x00, 0x00, 0x00, 0x00, 0x3C, 0x3C, 0x3C, 0x3C,
 0x3C, 0x3C, 0x3C, 0x3C, 0x3C, 0x3C, 0x3C, 0x3C,
 0x1F, 0xF8, 0x1F, 0xF8, 0x1F, 0xF8, 0x1F, 0xF8,
 0x1E, 0x78, 0x1E, 0x78, 0x1E, 0x78, 0x1E, 0x78,
 0x0F, 0xF0, 0x0F, 0xF0, 0x0F, 0xF0, 0x0F, 0xF0,
 0x0F, 0xF0, 0x0F, 0xF0, 0x0F, 0xF0, 0x0F, 0xF0,
 0x07, 0xE0, 0x07, 0xE0, 0x07, 0xE0, 0x07, 0xE0,
 0x07, 0xE0, 0x07, 0xE0, 0x00, 0x00, 0x00, 0x00},   /* 33 A */

{0x00, 0x00, 0x00, 0x00, 0x3F, 0xE0, 0x3F, 0xF8,
 0x3F, 0xF8, 0x3C, 0x7C, 0x3C, 0x3C, 0x3C, 0x3C,
 0x3C, 0x3C, 0x3C, 0x3C, 0x3C, 0x3C, 0x3C, 0x3C,
 0x3C, 0x3C, 0x3C, 0x78, 0x3F, 0xF8, 0x3F, 0xF0,
 0x3F, 0xF0, 0x3F, 0xF8, 0x3C, 0x78, 0x3C, 0x3C,
 0x3C, 0x3C, 0x3C, 0x3C, 0x3C, 0x3C, 0x3C, 0x3C,
 0x3C, 0x3C, 0x3C, 0x3C, 0x3C, 0x7C, 0x3F, 0xF8,
 0x3F, 0xF8, 0x3F, 0xE0, 0x00, 0x00, 0x00, 0x00},   /* 34 B */

{0x00, 0x00, 0x00, 0x00, 0x07, 0xE0, 0x1F, 0xF8,
 0x1F, 0xF8, 0x3E, 0x7C, 0x3C, 0x3C, 0x3C, 0x3C,
 0x3C, 0x3C, 0x3C, 0x3C, 0x3C, 0x3C, 0x3C, 0x3C,
 0x3C, 0x00, 0x3C, 0x00, 0x3C, 0x00, 0x3C, 0x00,
 0x3C, 0x00, 0x3C, 0x00, 0x3C, 0x00, 0x3C, 0x00,
 0x3C, 0x3C, 0x3C, 0x3C, 0x3C, 0x3C, 0x3C, 0x3C,
 0x3C, 0x3C, 0x3C, 0x3C, 0x3E, 0x7C, 0x1F, 0xF8,
 0x1F, 0xF8, 0x07, 0xE0, 0x00, 0x00, 0x00, 0x00},   /* 35 C */

{0x00, 0x00, 0x00, 0x00, 0x3F, 0xE0, 0x3F, 0xF8,
 0x3F, 0xF8, 0x3C, 0x7C, 0x3C, 0x3C, 0x3C, 0x3C,
 0x3C, 0x3C, 0x3C, 0x3C, 0x3C, 0x3C, 0x3C, 0x3C,
 0x3C, 0x3C, 0x3C, 0x3C, 0x3C, 0x3C, 0x3C, 0x3C,
 0x3C, 0x3C, 0x3C, 0x3C, 0x3C, 0x3C, 0x3C, 0x3C,
 0x3C, 0x3C, 0x3C, 0x3C, 0x3C, 0x3C, 0x3C, 0x3C,
 0x3C, 0x3C, 0x3C, 0x3C, 0x3C, 0x7C, 0x3F, 0xF8,
 0x3F, 0xF8, 0x3F, 0xE0, 0x00, 0x00, 0x00, 0x00},   /* 36 D */

{0x00, 0x00, 0x00, 0x00, 0x3F, 0xFC, 0x3F, 0xFC,
 0x3F, 0xFC, 0x3F, 0xFC, 0x3C, 0x00, 0x3C, 0x00,
 0x3C, 0x00, 0x3C, 0x00, 0x3C, 0x00, 0x3C, 0x00,
 0x3C, 0x00, 0x3C, 0x00, 0x3F, 0xF0, 0x3F, 0xF0,
 0x3F, 0xF0, 0x3F, 0xF0, 0x3C, 0x00, 0x3C, 0x00,
 0x3C, 0x00, 0x3C, 0x00, 0x3C, 0x00, 0x3C, 0x00,
 0x3C, 0x00, 0x3C, 0x00, 0x3F, 0xFC, 0x3F, 0xFC,
 0x3F, 0xFC, 0x3F, 0xFC, 0x00, 0x00, 0x00, 0x00},   /* 37 E */

{0x00, 0x00, 0x00, 0x00, 0x3C, 0x00, 0x3C, 0x00,
 0x3C, 0x00, 0x3C, 0x00, 0x3C, 0x00, 0x3C, 0x00,
 0x3C, 0x00, 0x3C, 0x00, 0x3C, 0x00, 0x3C, 0x00,
 0x3C, 0x00, 0x3C, 0x00, 0x3F, 0xF0, 0x3F, 0xF0,
 0x3F, 0xF0, 0x3F, 0xF0, 0x3C, 0x00, 0x3C, 0x00,
 0x3C, 0x00, 0x3C, 0x00, 0x3C, 0x00, 0x3C, 0x00,
 0x3C, 0x00, 0x3C, 0x00, 0x3F, 0xFC, 0x3F, 0xFC,
 0x3F, 0xFC, 0x3F, 0xFC, 0x00, 0x00, 0x00, 0x00},   /* 38 F */

{0x00, 0x00, 0x00, 0x00, 0x07, 0xE0, 0x1F, 0xF8,
 0x1F, 0xF8, 0x3E, 0x7C, 0x3C, 0x3C, 0x3C, 0x3C,
 0x3C, 0x3C, 0x3C, 0x3C, 0x3C, 0x3C, 0x3C, 0x3C,
 0x3C, 0x3C, 0x3C, 0x3C, 0x3C, 0xFC, 0x3C, 0xFC,
 0x3C, 0xFC, 0x3C, 0xFC, 0x3C, 0x00, 0x3C, 0x00,
 0x3C, 0x3C, 0x3C, 0x3C, 0x3C, 0x3C, 0x3C, 0x3C,
 0x3C, 0x3C, 0x3C, 0x3C, 0x3E, 0x7C, 0x1F, 0xF8,
 0x1F, 0xF8, 0x07, 0xE0, 0x00, 0x00, 0x00, 0x00},   /* 39 G */

{0x00, 0x00, 0x00, 0x00, 0x3C, 0x3C, 0x3C, 0x3C,
 0x3C, 0x3C, 0x3C, 0x3C, 0x3C, 0x3C, 0x3C, 0x3C,
 0x3C, 0x3C, 0x3C, 0x3C, 0x3C, 0x3C, 0x3C, 0x3C,
 0x3C, 0x3C, 0x3C, 0x3C, 0x3F, 0xFC, 0x3F, 0xFC,
 0x3F, 0xFC, 0x3F, 0xFC, 0x3C, 0x3C, 0x3C, 0x3C,
 0x3C, 0x3C, 0x3C, 0x3C, 0x3C, 0x3C, 0x3C, 0x3C,
 0x3C, 0x3C, 0x3C, 0x3C, 0x3C, 0x3C, 0x3C, 0x3C,
 0x3C, 0x3C, 0x3C, 0x3C, 0x00, 0x00, 0x00, 0x00},   /* 40 H */

{0x00, 0x00, 0x00, 0x00, 0x0F, 0xF0, 0x0F, 0xF0,
 0x0F, 0xF0, 0x0F, 0xF0, 0x03, 0xC0, 0x03, 0xC0,
 0x03, 0xC0, 0x03, 0xC0, 0x03, 0xC0, 0x03, 0xC0,
 0x03, 0xC0, 0x03, 0xC0, 0x03, 0xC0, 0x03, 0xC0,
 0x03, 0xC0, 0x03, 0xC0, 0x03, 0xC0, 0x03, 0xC0,
 0x03, 0xC0, 0x03, 0xC0, 0x03, 0xC0, 0x03, 0xC0,
 0x03, 0xC0, 0x03, 0xC0, 0x0F, 0xF0, 0x0F, 0xF0,
 0x0F, 0xF0, 0x0F, 0xF0, 0x00, 0x00, 0x00, 0x00},   /* 41 I */

{0x00, 0x00, 0x00, 0x00, 0x07, 0xE0, 0x1F, 0xF8,
 0x1F, 0xF8, 0x3E, 0x7C, 0x3C, 0x3C, 0x3C, 0x3C,
 0x3C, 0x3C, 0x3C, 0x3C, 0x3C, 0x3C, 0x3C, 0x3C,
 0x00, 0x3C, 0x00, 0x3C, 0x00, 0x3C, 0x00, 0x3C,
 0x00, 0x3C, 0x00, 0x3C, 0x00, 0x3C, 0x00, 0x3C,
 0x00, 0x3C, 0x00, 0x3C, 0x00, 0x3C, 0x00, 0x3C,
 0x00, 0x3C, 0x00, 0x3C, 0x00, 0x3C, 0x00, 0x3C,
 0x00, 0x3C, 0x00, 0x3C, 0x00, 0x00, 0x00, 0x00},   /* 42 J */

{0x00, 0x00, 0x00, 0x00, 0x3C, 0x3C, 0x3C, 0x3C,
 0x3C, 0x3C, 0x3C, 0x78, 0x3C, 0x78, 0x3C, 0x78,
 0x3C, 0xF0, 0x3C, 0xF0, 0x3C, 0xF0, 0x3D, 0xE0,
 0x3D, 0xE0, 0x3D, 0xE0, 0x3F, 0xC0, 0x3F, 0xC0,
 0x3F, 0xC0, 0x3F, 0xC0, 0x3D, 0xE0, 0x3D, 0xE0,
 0x3D, 0xE0, 0x3C, 0xF0, 0x3C, 0xF0, 0x3C, 0xF0,
 0x3C, 0x78, 0x3C, 0x78, 0x3C, 0x78, 0x3C, 0x3C,
 0x3C, 0x3C, 0x3C, 0x3C, 0x00, 0x00, 0x00, 0x00},   /* 43 K */

{0x00, 0x00, 0x00, 0x00, 0x3F, 0xFC, 0x3F, 0xFC,
 0x3F, 0xFC, 0x3F, 0xFC, 0x3C, 0x00, 0x3C, 0x00,
 0x3C, 0x00, 0x3C, 0x00, 0x3C, 0x00, 0x3C, 0x00,
 0x3C, 0x00, 0x3C, 0x00, 0x3C, 0x00, 0x3C, 0x00,
 0x3C, 0x00, 0x3C, 0x00, 0x3C, 0x00, 0x3C, 0x00,
 0x3C, 0x00, 0x3C, 0x00, 0x3C, 0x00, 0x3C, 0x00,
 0x3C, 0x00, 0x3C, 0x00, 0x3C, 0x00, 0x3C, 0x00,
 0x3C, 0x00, 0x3C, 0x00, 0x00, 0x00, 0x00, 0x00},   /* 44 L */

{0x00, 0x00, 0x00, 0x00, 0x3C, 0x3C, 0x3C, 0x3C,
 0x3C, 0x3C, 0x3C, 0x3C, 0x3C, 0x3C, 0x3C, 0x3C,
 0x3C, 0x3C, 0x3C, 0x3C, 0x3D, 0xBC, 0x3D, 0xBC,
 0x3D, 0xBC, 0x3D, 0xBC, 0x3F, 0xFC, 0x3F, 0xFC,
 0x3F, 0xFC, 0x3F, 0xFC, 0x3F, 0xFC, 0x3F, 0xFC,
 0x3F, 0xFC, 0x3F, 0xFC, 0x3E, 0x7C, 0x3E, 0x7C,
 0x3E, 0x7C, 0x3E, 0x7C, 0x3C, 0x3C, 0x3C, 0x3C,
 0x3C, 0x3C, 0x3C, 0x3C, 0x00, 0x00, 0x00, 0x00},   /* 45 M */

{0x00, 0x00, 0x00, 0x00, 0x3C, 0x7C, 0x3C, 0x7C,
 0x3C, 0x7C, 0x3C, 0x7C, 0x3C, 0xFC, 0x3C, 0xFC,
 0x3C, 0xFC, 0x3C, 0xFC, 0x3D, 0xFC, 0x3D, 0xFC,
 0x3D, 0xFC, 0x3D, 0xFC, 0x3F, 0xFC, 0x3F, 0xFC,
 0x3F, 0xFC, 0x3F, 0xFC, 0x3F, 0xBC, 0x3F, 0xBC,
 0x3F, 0xBC, 0x3F, 0xBC, 0x3F, 0x3C, 0x3F, 0x3C,
 0x3F, 0x3C, 0x3F, 0x3C, 0x3E, 0x3C, 0x3E, 0x3C,
 0x3E, 0x3C, 0x3E, 0x3C, 0x00, 0x00, 0x00, 0x00},   /* 46 N */

{0x00, 0x00, 0x00, 0x00, 0x07, 0xE0, 0x1F, 0xF8,
 0x1F, 0xF8, 0x3E, 0x7C, 0x3C, 0x3C, 0x3C, 0x3C,
 0x3C, 0x3C, 0x3C, 0x3C, 0x3C, 0x3C, 0x3C, 0x3C,
 0x3C, 0x3C, 0x3C, 0x3C, 0x3C, 0x3C, 0x3C, 0x3C,
 0x3C, 0x3C, 0x3C, 0x3C, 0x3C, 0x3C, 0x3C, 0x3C,
 0x3C, 0x3C, 0x3C, 0x3C, 0x3C, 0x3C, 0x3C, 0x3C,
 0x3C, 0x3C, 0x3C, 0x3C, 0x3E, 0x7C, 0x1F, 0xF8,
 0x1F, 0xF8, 0x07, 0xE0, 0x00, 0x00, 0x00, 0x00},   /* 47 O */

{0x00, 0x00, 0x00, 0x00, 0x3C, 0x00, 0x3C, 0x00,
 0x3C, 0x00, 0x3C, 0x00, 0x3C, 0x00, 0x3C, 0x00,
 0x3C, 0x00, 0x3C, 0x00, 0x3C, 0x00, 0x3C, 0x00,
 0x3C, 0x00, 0x3C, 0x00, 0x3F, 0xE0, 0x3F, 0xF8,
 0x3F, 0xF8, 0x3C, 0x7C, 0x3C, 0x3C, 0x3C, 0x3C,
 0x3C, 0x3C, 0x3C, 0x3C, 0x3C, 0x3C, 0x3C, 0x3C,
 0x3C, 0x3C, 0x3C, 0x3C, 0x3C, 0x7C, 0x3F, 0xF8,
 0x3F, 0xF8, 0x3F, 0xE0, 0x00, 0x00, 0x00, 0x00},   /* 48 P */

{0x00, 0x00, 0x00, 0x00, 0x07, 0xEC, 0x1F, 0xFC,
 0x1F, 0xFC, 0x3E, 0xFC, 0x3C, 0xF8, 0x3C, 0xFC,
 0x3C, 0x3C, 0x3C, 0x3C, 0x3C, 0x3C, 0x3C, 0x3C,
 0x3C, 0x3C, 0x3C, 0x3C, 0x3C, 0x3C, 0x3C, 0x3C,
 0x3C, 0x3C, 0x3C, 0x3C, 0x3C, 0x3C, 0x3C, 0x3C,
 0x3C, 0x3C, 0x3C, 0x3C, 0x3C, 0x3C, 0x3C, 0x3C,
 0x3C, 0x3C, 0x3C, 0x3C, 0x3E, 0x7C, 0x1F, 0xF8,
 0x1F, 0xF8, 0x07, 0xE0, 0x00, 0x00, 0x00, 0x00},   /* 49 Q */

{0x00, 0x00, 0x00, 0x00, 0x3C, 0x3C, 0x3C, 0x3C,
 0x3C, 0x3C, 0x3C, 0x78, 0x3C, 0x78, 0x3C, 0x78,
 0x3C, 0xF0, 0x3C, 0xF0, 0x3C, 0xF0, 0x3D, 0xE0,
 0x3D, 0xE0, 0x3D, 0xE0, 0x3F, 0xC0, 0x3F, 0xF0,
 0x3F, 0xF8, 0x3C, 0x78, 0x3C, 0x3C, 0x3C, 0x3C,
 0x3C, 0x3C, 0x3C, 0x3C, 0x3C, 0x3C, 0x3C, 0x3C,
 0x3C, 0x3C, 0x3C, 0x3C, 0x3C, 0x7C, 0x3F, 0xF8,
 0x3F, 0xF8, 0x3F, 0xE0, 0x00, 0x00, 0x00, 0x00},   /* 50 R */

{0x00, 0x00, 0x00, 0x00, 0x07, 0xE0, 0x1F, 0xF8,
 0x1F, 0xF8, 0x3E, 0x7C, 0x3C, 0x3C, 0x3C, 0x3C,
 0x3C, 0x3C, 0x3C, 0x3C, 0x3C, 0x3C, 0x3C, 0x3C,
 0x00, 0x78, 0x00, 0xF8, 0x01, 0xF0, 0x03, 0xE0,
 0x07, 0xC0, 0x0F, 0x80, 0x1F, 0x00, 0x1E, 0x00,
 0x3C, 0x3C, 0x3C, 0x3C, 0x3C, 0x3C, 0x3C, 0x3C,
 0x3C, 0x3C, 0x3C, 0x3C, 0x3E, 0x7C, 0x1F, 0xF8,
 0x1F, 0xF8, 0x07, 0xE0, 0x00, 0x00, 0x00, 0x00},   /* 51 S */

{0x00, 0x00, 0x00, 0x00, 0x03, 0xC0, 0x03, 0xC0,
 0x03, 0xC0, 0x03, 0xC0, 0x03, 0xC0, 0x03, 0xC0,
 0x03, 0xC0, 0x03, 0xC0, 0x03, 0xC0, 0x03, 0xC0,
 0x03, 0xC0, 0x03, 0xC0, 0x03, 0xC0, 0x03, 0xC0,
 0x03, 0xC0, 0x03, 0xC0, 0x03, 0xC0, 0x03, 0xC0,
 0x03, 0xC0, 0x03, 0xC0, 0x03, 0xC0, 0x03, 0xC0,
 0x03, 0xC0, 0x03, 0xC0, 0x3F, 0xFC, 0x3F, 0xFC,
 0x3F, 0xFC, 0x3F, 0xFC, 0x00, 0x00, 0x00, 0x00},   /* 52 T */

{0x00, 0x00, 0x00, 0x00, 0x07, 0xE0, 0x1F, 0xF8,
 0x1F, 0xF8, 0x3E, 0x7C, 0x3C, 0x3C, 0x3C, 0x3C,
 0x3C, 0x3C, 0x3C, 0x3C, 0x3C, 0x3C, 0x3C, 0x3C,
 0x3C, 0x3C, 0x3C, 0x3C, 0x3C, 0x3C, 0x3C, 0x3C,
 0x3C, 0x3C, 0x3C, 0x3C, 0x3C, 0x3C, 0x3C, 0x3C,
 0x3C, 0x3C, 0x3C, 0x3C, 0x3C, 0x3C, 0x3C, 0x3C,
 0x3C, 0x3C, 0x3C, 0x3C, 0x3C, 0x3C, 0x3C, 0x3C,
 0x3C, 0x3C, 0x3C, 0x3C, 0x00, 0x00, 0x00, 0x00},   /* 53 U */

{0x00, 0x00, 0x00, 0x00, 0x07, 0xE0, 0x07, 0xE0,
 0x07, 0xE0, 0x07, 0xE0, 0x07, 0xE0, 0x07, 0xE0,
 0x0F, 0xF0, 0x0F, 0xF0, 0x0F, 0xF0, 0x0F, 0xF0,
 0x0F, 0xF0, 0x0F, 0xF0, 0x0F, 0xF0, 0x0F, 0xF0,
 0x1E, 0x78, 0x1E, 0x78, 0x1E, 0x78, 0x1E, 0x78,
 0x1E, 0x78, 0x1E, 0x78, 0x1E, 0x78, 0x1E, 0x78,
 0x3C, 0x3C, 0x3C, 0x3C, 0x3C, 0x3C, 0x3C, 0x3C,
 0x3C, 0x3C, 0x3C, 0x3C, 0x00, 0x00, 0x00, 0x00},   /* 54 V */

{0x00, 0x00, 0x00, 0x00, 0x3C, 0x3C, 0x3C, 0x3C,
 0x3C, 0x3C, 0x3C, 0x3C, 0x3E, 0x7C, 0x3E, 0x7C,
 0x3E, 0x7C, 0x3E, 0x7C, 0x3F, 0xFC, 0x3F, 0xFC,
 0x3F, 0xFC, 0x3F, 0xFC, 0x3F, 0xFC, 0x3F, 0xFC,
 0x3F, 0xFC, 0x3F, 0xFC, 0x3D, 0xBC, 0x3D, 0xBC,
 0x3D, 0xBC, 0x3D, 0xBC, 0x3C, 0x3C, 0x3C, 0x3C,
 0x3C, 0x3C, 0x3C, 0x3C, 0x3C, 0x3C, 0x3C, 0x3C,
 0x3C, 0x3C, 0x3C, 0x3C, 0x00, 0x00, 0x00, 0x00},   /* 55 W */

{0x00, 0x00, 0x00, 0x00, 0x3C, 0x3C, 0x3C, 0x3C,
 0x3C, 0x3C, 0x1E, 0x78, 0x1E, 0x78, 0x1E, 0x78,
 0x0F, 0xF0, 0x0F, 0xF0, 0x0F, 0xF0, 0x07, 0xE0,
 0x07, 0xE0, 0x07, 0xE0, 0x03, 0xC0, 0x03, 0xC0,
 0x03, 0xC0, 0x03, 0xC0, 0x07, 0xE0, 0x07, 0xE0,
 0x07, 0xE0, 0x0F, 0xF0, 0x0F, 0xF0, 0x0F, 0xF0,
 0x1E, 0x78, 0x1E, 0x78, 0x1E, 0x78, 0x3C, 0x3C,
 0x3C, 0x3C, 0x3C, 0x3C, 0x00, 0x00, 0x00, 0x00},   /* 56 X */

{0x00, 0x00, 0x00, 0x00, 0x03, 0xC0, 0x03, 0xC0,
 0x03, 0xC0, 0x03, 0xC0, 0x03, 0xC0, 0x03, 0xC0,
 0x03, 0xC0, 0x03, 0xC0, 0x03, 0xC0, 0x03, 0xC0,
 0x03, 0xC0, 0x03, 0xC0, 0x03, 0xC0, 0x03, 0xC0,
 0x03, 0xC0, 0x03, 0xC0, 0x07, 0xE0, 0x07, 0xE0,
 0x07, 0xE0, 0x0F, 0xF0, 0x0F, 0xF0, 0x0F, 0xF0,
 0x1E, 0x78, 0x1E, 0x78, 0x1E, 0x78, 0x3C, 0x3C,
 0x3C, 0x3C, 0x3C, 0x3C, 0x00, 0x00, 0x00, 0x00},   /* 57 Y */

{0x00, 0x00, 0x00, 0x00, 0x3F, 0xFC, 0x3F, 0xFC,
 0x3F, 0xFC, 0x3F, 0xFC, 0x3C, 0x00, 0x3C, 0x00,
 0x3C, 0x00, 0x1E, 0x00, 0x1E, 0x00, 0x0F, 0x00,
 0x0F, 0x00, 0x07, 0x80, 0x07, 0x80, 0x03, 0xC0,
 0x03, 0xC0, 0x01, 0xE0, 0x01, 0xE0, 0x00, 0xF0,
 0x00, 0xF0, 0x00, 0x78, 0x00, 0x78, 0x00, 0x3C,
 0x00, 0x3C, 0x00, 0x3C, 0x3F, 0xFC, 0x3F, 0xFC,
 0x3F, 0xFC, 0x3F, 0xFC, 0x00, 0x00, 0x00, 0x00}   /* 58 Z */
};

/* End of large font declaration =============================================*/



GLuint s_FontOffset, l_f_FontOffset, l_o_FontOffset;

/* Compile font(s) into display list =========================================*/
void makeRasterFont(void)
{
   GLuint i, j;
   glPixelStorei(GL_UNPACK_ALIGNMENT, 1);

   s_FontOffset = glGenLists(128);
   l_o_FontOffset = glGenLists(128);
   l_f_FontOffset = glGenLists(128);

   /* Small font... */
   for(i=0, j=' '; i<=96; i++, j++)
   {
      glNewList(s_FontOffset + j, GL_COMPILE);
      glBitmap(8, 12, 0, 0, 8-s_kern[i], 0, s_characters[i]);
      glEndList();
   }

   /* Large font space... */
   glNewList(l_o_FontOffset + 32, GL_COMPILE);
   glBitmap(0, 32, 0, 0, 16, 0, l_o_space);   /* Outline colour */
   glEndList();
   glNewList(l_f_FontOffset + 32, GL_COMPILE);
   glBitmap(0, 32, 0, 0, 16, 0, l_f_space);   /* Fill colour */
   glEndList();

   /* Large font exclamation... */
   glNewList(l_o_FontOffset + '!', GL_COMPILE);
   glBitmap(16, 32, 0, 0, 16, 0, l_o_exclamation);   /* Outline colour */
   glEndList();
   glNewList(l_f_FontOffset + '!', GL_COMPILE);
   glBitmap(16, 32, 0, 0, 16, 0, l_f_exclamation);   /* Fill colour */
   glEndList();

   /* Large font double quote... */
   glNewList(l_o_FontOffset + '"', GL_COMPILE);
   glBitmap(16, 32, 0, 0, 16, 0, l_o_double_quote);   /* Outline colour */
   glEndList();
   glNewList(l_f_FontOffset + '"', GL_COMPILE);
   glBitmap(16, 32, 0, 0, 16, 0, l_f_double_quote);   /* Fill colour */
   glEndList();

   /* Large font pound sign... */
   glNewList(l_o_FontOffset + '$', GL_COMPILE);
   glBitmap(16, 32, 0, 0, 16, 0, l_o_pound_sign);   /* Outline colour */
   glEndList();
   glNewList(l_f_FontOffset + '$', GL_COMPILE);
   glBitmap(16, 32, 0, 0, 16, 0, l_f_pound_sign);   /* Fill colour */
   glEndList();

   /* Large font single quote... */
   glNewList(l_o_FontOffset + 39, GL_COMPILE);
   glBitmap(16, 32, 0, 0, 16, 0, l_o_single_quote);   /* Outline colour */
   glEndList();
   glNewList(l_f_FontOffset + 39, GL_COMPILE);
   glBitmap(16, 32, 0, 0, 16, 0, l_f_single_quote);   /* Fill colour */
   glEndList();

   /* Large font plus sign... */
   glNewList(l_o_FontOffset + '+', GL_COMPILE);
   glBitmap(16, 32, 0, 0, 16, 0, l_o_plus_sign);   /* Outline colour */
   glEndList();
   glNewList(l_f_FontOffset + '+', GL_COMPILE);
   glBitmap(16, 32, 0, 0, 16, 0, l_f_plus_sign);   /* Fill colour */
   glEndList();

   /* Large font hyphen... */
   glNewList(l_o_FontOffset + '-', GL_COMPILE);
   glBitmap(16, 32, 0, 0, 16, 0, l_o_hyphen);   /* Outline colour */
   glEndList();
   glNewList(l_f_FontOffset + '-', GL_COMPILE);
   glBitmap(16, 32, 0, 0, 16, 0, l_f_hyphen);   /* Fill colour */
   glEndList();

   /* Large font fullstop... */
   glNewList(l_o_FontOffset + '.', GL_COMPILE);
   glBitmap(16, 32, 0, 0, 16, 0, l_o_fullstop);   /* Outline colour */
   glEndList();
   glNewList(l_f_FontOffset + '.', GL_COMPILE);
   glBitmap(16, 32, 0, 0, 16, 0, l_f_fullstop);   /* Fill colour */
   glEndList();

   /* Large font numbers... */
   for(i=0, j='0'; i<=10; i++, j++)
   {
      glNewList(l_o_FontOffset + j, GL_COMPILE);
      glBitmap(16, 32, 0, 0, 16, 0, l_o_numbers[i]);   /* Outline colour */
      glEndList();
   }
   for(i=0, j='0'; i<=10; i++, j++)
   {
      glNewList(l_f_FontOffset + j, GL_COMPILE);
      glBitmap(16, 32, 0, 0, 16, 0, l_f_numbers[i]);   /* Fill colour */
      glEndList();
   }

   /* Large font colon... */
   glNewList(l_o_FontOffset + ':', GL_COMPILE);
   glBitmap(16, 32, 0, 0, 16, 0, l_o_colon);   /* Outline colour */
   glEndList();
   glNewList(l_f_FontOffset + ':', GL_COMPILE);
   glBitmap(16, 32, 0, 0, 16, 0, l_f_colon);   /* Fill colour */
   glEndList();

   /* Large font less than... */
   glNewList(l_o_FontOffset + '<', GL_COMPILE);
   glBitmap(16, 32, 0, 0, 16, 0, l_o_less_than);   /* Outline colour */
   glEndList();
   glNewList(l_f_FontOffset + '<', GL_COMPILE);
   glBitmap(16, 32, 0, 0, 16, 0, l_f_less_than);   /* Fill colour */
   glEndList();

   /* Large font greater than... */
   glNewList(l_o_FontOffset + '>', GL_COMPILE);
   glBitmap(16, 32, 0, 0, 16, 0, l_o_greater_than);   /* Outline colour */
   glEndList();
   glNewList(l_f_FontOffset + '>', GL_COMPILE);
   glBitmap(16, 32, 0, 0, 16, 0, l_f_greater_than);   /* Fill colour */
   glEndList();

   /* Large font question mark... */
   glNewList(l_o_FontOffset + '?', GL_COMPILE);
   glBitmap(16, 32, 0, 0, 16, 0, l_o_question_mark);   /* Outline colour */
   glEndList();
   glNewList(l_f_FontOffset + '?', GL_COMPILE);
   glBitmap(16, 32, 0, 0, 16, 0, l_f_question_mark);   /* Fill colour */
   glEndList();

   /* Large font letters... */
   for(i=0, j='a'; i<=26; i++, j++)
   {
      glNewList(l_o_FontOffset + j, GL_COMPILE);
      glBitmap(16, 32, 0, 0, 16, 0, l_o_letters[i]);   /* Outline colour */
      glEndList();
   }
   for(i=0, j='a'; i<=26; i++, j++)
   {
      glNewList(l_f_FontOffset + j, GL_COMPILE);
      glBitmap(16, 32, 0, 0, 16, 0, l_f_letters[i]);   /* Fill colour */
      glEndList();
   }
   for(i=0, j='A'; i<=26; i++, j++)
   {
      glNewList(l_o_FontOffset + j, GL_COMPILE);
      glBitmap(16, 32, 0, 0, 16, 0, l_o_letters[i]);   /* Outline colour */
      glEndList();
   }
   for(i=0, j='A'; i<=26; i++, j++)
   {
      glNewList(l_f_FontOffset + j, GL_COMPILE);
      glBitmap(16, 32, 0, 0, 16, 0, l_f_letters[i]);   /* Fill colour */
      glEndList();
   }
}

/* Print string to window using small font ===================================*/
void smallFont(char *s, int x, int y, float r, float g, float b)
{
   glPushAttrib(GL_LIST_BIT);
   glColor3f(r, g, b);
   glRasterPos2i(x, y);
   glListBase(s_FontOffset);
   glCallLists(strlen(s), GL_UNSIGNED_BYTE, (GLubyte *) s);
   glPopAttrib();
}

/* Print string to window using large font ===================================*/
void largeFont(char *s, int x, int y, float r, float g, float b)
{
   glPushAttrib(GL_LIST_BIT);
   glColor3f(r, g, b);
   glRasterPos2i(x, y);
   glListBase(l_f_FontOffset);
   glCallLists(strlen(s), GL_UNSIGNED_BYTE, (GLubyte *) s);
   glPopAttrib();
}

/* Print string to window using large font (with outline) ====================*/
void largeOutlineFont(char *s, int x, int y, float r, float g, float b)
{
   /* Draw outline... */
   glPushAttrib(GL_LIST_BIT);
   glColor3f(0, 0, 0);
   glRasterPos2i(x, y);
   glListBase(l_o_FontOffset);
   glCallLists(strlen(s), GL_UNSIGNED_BYTE, (GLubyte *) s);
   glPopAttrib();

   /* Draw fill... */
   glPushAttrib(GL_LIST_BIT);
   glColor3f(r, g, b);
   glRasterPos2i(x, y);
   glListBase(l_f_FontOffset);
   glCallLists(strlen(s), GL_UNSIGNED_BYTE, (GLubyte *) s);
   glPopAttrib();
}

#endif
