/*##############################################################################
###
###  FRUITY GAME ENGINE VERSION 0.1     :     COPYRIGHT  ROB BATEMAN 15/NOV/00
###
###  TIMER.CPP : A HIGH RES FRAME COUNTER FOR SILICON GRAPHICS WORKSTATIONS
###
##############################################################################*/

/*##############################################################################
###                             UPDATE HISTORY
##############################################################################*/
/*
 * 30/11/00 : Combined 'fruity_timerwin.cpp' & 'fruity_timer.cpp' into one file.
 *            #ifdef WIN32's ahoy. Will run on both platforms now. Removed GetFps
 *            function cos its pointless - replaced with macro.
 * 10/11/00 : Ari suggests looking on SGI website for high res frame counter 
 *            on SGI's. After much fiddling the holy grail was found. Had to 
 *            move the stuff to a new file only to be included when compiling
 *            on Irix due to the functions being used, not being in the windows
 *            version of time.h 
*/


/*##############################################################################
###                                INCLUDES
##############################################################################*/

// Engine Includes
#include "./opengl_includes.h"
#include "./frametime.h"

#include <time.h>

#ifdef WIN32
  typedef __int64 int64;
#endif


/*##############################################################################
###                                 GLOBALS
##############################################################################*/

double StartOfFrame, 
       EndOfFrame, 
       FrameT;


#ifdef WIN32
  double RateInv;
  int64  StartClk;
#else
  timespec_t tm;
#endif



/*##############################################################################
###                          GET CURRENT TIME
##############################################################################*/

double CurrentTime(void)

  {
    #ifdef WIN32

      int64 EndClk;
      QueryPerformanceCounter(( LARGE_INTEGER* )&EndClk);
      return (double)( EndClk - StartClk )*RateInv;

    #else

      clock_gettime(CLOCK_REALTIME,&tm);
      return tm.tv_sec + (tm.tv_nsec / 1e9);

    #endif
  }



/*##############################################################################
###                    INIT FRAME COUNTER FUNCTION
##############################################################################*/

bool initFrameCounter(void)
  {

     #ifdef WIN32

 
        RateInv = 1.0/(double)CLOCKS_PER_SEC;

        int64 Rate;
        if( !QueryPerformanceFrequency( ( LARGE_INTEGER* )&Rate) )
           return false;

        if(!Rate)
           return false;

        RateInv=1.0/(double)Rate;
        if(!QueryPerformanceCounter( ( LARGE_INTEGER* )&StartClk) )
           return false;


     #else

        StartOfFrame = CurrentTime();

     #endif

        return true;
  }

/*##############################################################################
###                     WORKS OUT TIME TO DRAW FRAME
##############################################################################*/

double getFrametime(void)

  {
     do
       {
          EndOfFrame=CurrentTime();
       }
     while(EndOfFrame==StartOfFrame);

     FrameT        =  EndOfFrame - StartOfFrame ;
     StartOfFrame  =  EndOfFrame ;

     return FrameT;
  }
