#ifndef _EXCEPTION_H
#define _EXCEPTION_H


#include <stdio.h>
#include <stdlib.h>
#include <fstream.h>

#include "./tga.h"                    /* Holds TGA error codes */
#include "./opengl_includes.h"

#ifndef __DEBUG__
#define __DEBUG__
#endif

#ifdef __DEBUG__

#define _filename_ "./logs/debug.txt";

extern void DebugLog( char *string );

#endif

#ifndef __DEBUG__

#define DebugLog( X ) 

#endif

extern float divide_f( float _x , float _y );
extern double divide_d( double _x , double _y );
extern float divide_i( int _x , int _y );

/*##############################################################################
###                            ERROR CODE FLAGS
##############################################################################*/

#define MEM_CHECK_MALLOC 1
#define MEM_CHECK_NULL_ACCESS 2

#define ERROR_MEM_NULL_DATA_ACCESS  98      /* Tried to access an array element that doesn't exist     */
#define ERROR_MEM_ALLOC_FAILED      99      /* Failed to allocate required memory                      */


// if the exception handler is disabled, the functions all become inline......

//___________________________________________________________________________
#ifndef HANDLE_MY_EXCEPTIONS
  inline void InitErrorLog(void){};
  inline void ErrorLog( char *string , char *stringy = NULL ){ return; }; 
  inline void loadTexture( char *filename , GLuint *uiTexObj)
    {
       glGenTextures( 1 , uiTexObj );
       loadTGA( filename , *uiTexObj );
    }
#else

//___________________________________________________________________________
  extern void loadTexture( char *filename , GLuint *uiTexObj);

  #ifdef GENERATE_EXCEPTION_LOG
    extern void InitErrorLog( void );
    extern void ErrorLog( char *string , char *stringy = NULL  );
  #else
    inline void InitErrorLog( void ){};
    inline void ErrorLog( char *string , char *stringy = NULL  ){ return; };
  #endif
#endif

//___________________________________________________________________________
#ifdef HANDLE_MY_EXCEPTIONS
  extern bool CheckMemory( void *Ptr, int type = MEM_CHECK_MALLOC );
#else 
  inline bool CheckMemory( void *Ptr, int type = MEM_CHECK_MALLOC )
    {
      if( Ptr==NULL && type == MEM_CHECK_MALLOC ) exit(0);
      return true;
    }
#endif
//___________________________________________________________________________


#endif
