#include "./scene_lighting.h"

bool cScene::loadScene( char *filename )
  {
    ifstream in( filename , ios::in );
    char buf[256];

    if( in )
      {
        in.getline( buf , 255 , '\n' );
        in >> buf
           >> uLightCount;

        pLightArray  = new cLight[ uLightCount ];
        CheckMemory( pLightArray );

        glEnable( GL_LIGHTING );

        for( uint i=0; i<uLightCount; i++ )
          {
             in >> buf
                >> buf
                >> buf //AMBIENT
                >> pLightArray[i].pAmbient[0]
                >> pLightArray[i].pAmbient[1]
                >> pLightArray[i].pAmbient[2]
                >> pLightArray[i].pAmbient[3]
                >> buf //DIFFUSE
                >> pLightArray[i].pDiffuse[0]
                >> pLightArray[i].pDiffuse[1]
                >> pLightArray[i].pDiffuse[2]
                >> pLightArray[i].pDiffuse[3]
                >> buf //SPECULAR
                >> pLightArray[i].pSpecular[0]
                >> pLightArray[i].pSpecular[1]
                >> pLightArray[i].pSpecular[2]
                >> pLightArray[i].pSpecular[3]
                >> buf //POSITION
                >> pLightArray[i].pPosition[0]
                >> pLightArray[i].pPosition[1]
                >> pLightArray[i].pPosition[2]
                >> pLightArray[i].pPosition[3]
                >> buf //DIRECTION
                >> pLightArray[i].pDirection[0]
                >> pLightArray[i].pDirection[1]
                >> pLightArray[i].pDirection[2]
                >> pLightArray[i].pDirection[3]
                >> buf //CONSTANT ATTENUATION 
                >> pLightArray[i].fConstantAttenuation
                >> buf //LINEAR ATTENUATION 
                >> pLightArray[i].fLinearAttenuation
                >> buf //QUADRATIC ATTENUATION 
                >> pLightArray[i].fQuadraticAttenuation
                >> buf //EXPONENT 
                >> pLightArray[i].fExponent
                >> buf //CUTOFF
                >> pLightArray[i].fCutoff ;

             glPushMatrix();
             glEnable( GL_LIGHT0 + i );
             glLightfv( GL_LIGHT0 + i , GL_AMBIENT  , &pLightArray[i].pAmbient[0]  );
             glLightfv( GL_LIGHT0 + i , GL_DIFFUSE  , &pLightArray[i].pDiffuse[0]  );
             glLightfv( GL_LIGHT0 + i , GL_SPECULAR , &pLightArray[i].pSpecular[0] );
             glLightfv( GL_LIGHT0 + i , GL_POSITION  , &pLightArray[i].pPosition[0]  );
             glLightfv( GL_LIGHT0 + i , GL_SPOT_DIRECTION , &pLightArray[i].pDirection[0] );
             glLightf( GL_LIGHT0 + i , GL_CONSTANT_ATTENUATION , pLightArray[i].fConstantAttenuation );
             glLightf( GL_LIGHT0 + i , GL_LINEAR_ATTENUATION , pLightArray[i].fLinearAttenuation );
             glLightf( GL_LIGHT0 + i , GL_QUADRATIC_ATTENUATION , pLightArray[i].fQuadraticAttenuation );
             glLightf( GL_LIGHT0 + i , GL_SPOT_EXPONENT , pLightArray[i].fExponent );
             glLightf( GL_LIGHT0 + i , GL_SPOT_CUTOFF , pLightArray[i].fCutoff );
             glPopMatrix();
          }
        return true;
      }

    return false;
  }

void cScene::setLightColour( uint lnum , GLenum property, float *col )
  {
    if( lnum < uLightCount ) 
      {
        switch( property )
          {
             case GL_AMBIENT:
               pLightArray[ lnum ].pAmbient[0] = col[0];
               pLightArray[ lnum ].pAmbient[1] = col[1];
               pLightArray[ lnum ].pAmbient[2] = col[2];
               pLightArray[ lnum ].pAmbient[3] = col[3];
               glLightfv( GL_LIGHT0 + lnum , GL_AMBIENT , pLightArray[ lnum ].pAmbient ); 
             break;

             case GL_DIFFUSE:
               pLightArray[ lnum ].pDiffuse[0] = col[0];
               pLightArray[ lnum ].pDiffuse[1] = col[1];
               pLightArray[ lnum ].pDiffuse[2] = col[2];
               pLightArray[ lnum ].pDiffuse[3] = col[3];
               glLightfv( GL_LIGHT0 + lnum , GL_DIFFUSE , pLightArray[ lnum ].pDiffuse ); 
             break;

             case GL_SPECULAR:
               pLightArray[ lnum ].pSpecular[0] = col[0];
               pLightArray[ lnum ].pSpecular[1] = col[1];
               pLightArray[ lnum ].pSpecular[2] = col[2];
               pLightArray[ lnum ].pSpecular[3] = col[3];
               glLightfv( GL_LIGHT0 + lnum , GL_SPECULAR , pLightArray[ lnum ].pSpecular ); 
             break;

             default:
             break;
          }
      }
  }


float *cScene::getLightColour( uint lnum , GLenum property )
  {
    if( lnum < uLightCount ) 
      {
        if( property == GL_AMBIENT )
               return pLightArray[ lnum ].pAmbient;

        if( property == GL_DIFFUSE )
               return pLightArray[ lnum ].pDiffuse;

        if( property == GL_SPECULAR )
               return pLightArray[ lnum ].pSpecular;

      }
    return NULL;
  }


void cScene::setLightAttenuation( uint lnum , char type , float val )
  {
    if( lnum < uLightCount ) 
      {
        switch( type )
          {
             case CONSTANT:
               pLightArray[ lnum ].fConstantAttenuation = val;
             break;

             case LINEAR:
               pLightArray[ lnum ].fLinearAttenuation = val;
             break;

             case QUADRATIC:
               pLightArray[ lnum ].fQuadraticAttenuation = val;
             break;

             default:
             break;
          }
      }
  }


float cScene::getLightAttenuation( uint lnum , char type )
  {
    if( lnum < uLightCount ) 
      {
        if( type == CONSTANT )
               return pLightArray[ lnum ].fConstantAttenuation;

        if( type == LINEAR   )
               return pLightArray[ lnum ].fLinearAttenuation;

        if( type == QUADRATIC )
               return pLightArray[ lnum ].fQuadraticAttenuation;

      }
    return 0;
  }


void cScene::setLightExponent( uint lnum , float fExp )
  {
    if( lnum < uLightCount ) 
      {
        pLightArray[ lnum ].fExponent =  fExp ;
      }
  }


float cScene::getLightExponent( uint lnum )
  {
    if( lnum < uLightCount ) 
      {
        return pLightArray[ lnum ].fExponent;
      }
    return 0;
  }


void cScene::setLightCutoff( uint lnum, float fCut)
  {
    if( lnum < uLightCount ) 
      {
        pLightArray[ lnum ].fCutoff = fCut;
      }
  }


float cScene::getLightCutoff( uint lnum )
  {
    if( lnum < uLightCount ) 
      {
        return pLightArray[ lnum ].fCutoff;
      }
    return 0;
  }




void cScene::setLightPos( uint lnum, cVector3f vec )
  {
    if( lnum < uLightCount )
      {
        pLightArray[ lnum ].pPosition[0] = vec.x;
        pLightArray[ lnum ].pPosition[1] = vec.y;
        pLightArray[ lnum ].pPosition[2] = vec.z;

        glLightfv( GL_LIGHT0 + lnum , GL_POSITION , pLightArray[ lnum ].pPosition );
      }
  }

void cScene::setLightAimPos( uint lnum, cVector3f vec )
  {
    if( lnum < uLightCount )
      {
        pLightArray[ lnum ].pDirection[0] = pLightArray[ lnum ].pPosition[0] - vec.x;
        pLightArray[ lnum ].pDirection[1] = pLightArray[ lnum ].pPosition[1] - vec.y;
        pLightArray[ lnum ].pDirection[2] = pLightArray[ lnum ].pPosition[2] - vec.z;

        glLightfv( GL_LIGHT0 + lnum , GL_SPOT_DIRECTION , pLightArray[ lnum ].pDirection );
      }
  }
void cScene::setLightDirection( uint lnum, cVector3f vec )
  {
    if( lnum < uLightCount )
      {
        pLightArray[ lnum ].pDirection[0] = vec.x;
        pLightArray[ lnum ].pDirection[1] = vec.y;
        pLightArray[ lnum ].pDirection[2] = vec.z;

        glLightfv( GL_LIGHT0 + lnum , GL_SPOT_DIRECTION , pLightArray[ lnum ].pDirection );
      }
  }


cVector3f cScene::getLightPos( uint lnum )
  {
    cVector3f vec( 0 , 0 , 0 );
    if( lnum < uLightCount )
      {
        vec.x = pLightArray[ lnum ].pPosition[0];
        vec.y = pLightArray[ lnum ].pPosition[1];
        vec.z = pLightArray[ lnum ].pPosition[2];
      }
    return vec;
  }

cVector3f cScene::getLightAimPos( uint lnum )
  {
    cVector3f vec( 0 , 0 , 0 );
    if( lnum < uLightCount )
      {
        vec.x = pLightArray[ lnum ].pPosition[0] + pLightArray[ lnum ].pDirection[0];
        vec.y = pLightArray[ lnum ].pPosition[1] + pLightArray[ lnum ].pDirection[1];
        vec.z = pLightArray[ lnum ].pPosition[2] + pLightArray[ lnum ].pDirection[2];
      }
    return vec;
  }

cVector3f cScene::getLightDirection( uint lnum )
  {
    cVector3f vec( 0 , 0 , 0 );
    if( lnum < uLightCount )
      {
        vec.x = pLightArray[ lnum ].pDirection[0];
        vec.y = pLightArray[ lnum ].pDirection[1];
        vec.z = pLightArray[ lnum ].pDirection[2];
      }
    return vec;
  }


//________________________________________________________________________ Ambient Scene Lighting Functions

void cScene::setLightModelAmbient( float r, float g, float b, float a )
  {
    pLightModelAmbient[0] = r;
    pLightModelAmbient[1] = g;
    pLightModelAmbient[2] = b;
    pLightModelAmbient[3] = a;

    glLightModelfv( GL_LIGHT_MODEL_AMBIENT , pLightModelAmbient );
  }

float *cScene::getLightModelAmbient( void )
  {
    return pLightModelAmbient;
  }


//________________________________________________________________________ Fog Functions


void cScene::setFogColour( float r, float g, float b, float a )
  {
    pFogColor[0] = r;
    pFogColor[1] = g;
    pFogColor[2] = b;
    pFogColor[3] = a;

    glFogfv( GL_FOG_COLOR , pFogColor );

  }


void cScene::setFogDensity( float fD )
  {
    fFogDensity = fD;
    
    glFogf( GL_FOG_DENSITY , fFogDensity );
  }


void cScene::setFogFunc( GLenum Func )
  {
    if( Func == GL_EXP || Func == GL_EXP2 || Func == GL_LINEAR )
      {
        eFogFunc = Func;
        glFogi( GL_FOG_MODE , eFogFunc );
      }
  }


void cScene::setFogStart( float fStart )
  {
    fFogStart = fStart;
    glFogf( GL_FOG_START , fFogStart );
  }


void cScene::setFogEnd( float fEnd )
  {
    fFogEnd = fEnd;
    glFogf( GL_FOG_END , fFogEnd );
  }



float *cScene::getFogColour( void )
  {
    return pFogColor;
  }


float cScene::getFogDensity( void )
  {
    return fFogDensity;
  }


GLenum cScene::getFogFunc( void )
  {
    return eFogFunc;
  }


float cScene::getFogStart( void )
  {
    return fFogStart;
  }


float cScene::getFogEnd( void )
  {
    return fFogEnd;
  }

