/*######################################################################
###
###   OBJECT LOADER SOURCE FILE
###
######################################################################*/

#include "./objloader.h"
#include "./scene_lighting.h"

#include <math.h>

#define FLOOR_LEVEL -5	

/*##############################################################################
###                         FUNCTION TO LOAD VERTICIES.
##############################################################################*/

bool objFile::load( char *filename )
  {
    uint i;

    ifstream inputFile( filename );

    if( !inputFile )
      {
        cout << "Error : Unable to open File" 
             << endl;
        return false;
      }

    else
      {

        char In_String[128];

          //
          // count data
          //
        while( !inputFile.eof() )
          {
            inputFile >> In_String;
            
            if( strcmp( In_String , "v"  ) == 0 ) vertexCount++;
            if( strcmp( In_String , "vn" ) == 0 ) normalCount++;
            if( strcmp( In_String , "vt" ) == 0 ) texCoordCount++;
            if( strcmp( In_String , "f"  ) == 0 ) triangleCount++;

          }


          // allocate memory and set vertexCount
        vertexArray   = new Vertex  [ vertexCount   ];
        normalArray   = new Normal  [ normalCount   ];
        texCoordArray = new TexCoord[ texCoordCount ];
        triangleArray = new Triangle[ triangleCount ];

        CheckMemory( vertexArray );
        CheckMemory( normalArray );
        CheckMemory( texCoordArray );
        CheckMemory( triangleArray );

          // close & re-open file stream
        inputFile.close();
        inputFile.open( filename );


          // create some temorary count values
        int vCount = 0 ,
            nCount = 0 ,
            tCount = 0 ,
            fCount = 0 ;


          // get data and place in array of verticies
        while( !inputFile.eof() )
          {
             inputFile >> In_String;
             if( strcmp( In_String , "v"  ) == 0 )
               {
                 inputFile >> vertexArray[ vCount ].x
                           >> vertexArray[ vCount ].y
                           >> vertexArray[ vCount ].z;
                 vertexArray[ vCount ].CurrentPos = vertexArray[ vCount ];
                 vCount++;
               } 
             else
             if( strcmp( In_String , "vn" ) == 0 )
               {
                 inputFile >> normalArray[ nCount ].x
                           >> normalArray[ nCount ].y
                           >> normalArray[ nCount ].z;
                 nCount++;
               }
             else
             if( strcmp( In_String , "vt" ) == 0 )
               {
                 inputFile >> texCoordArray[ tCount ].pos[0]
                           >> texCoordArray[ tCount ].pos[1];
                 tCount++;
               }
             else
             if( strcmp( In_String , "f"  ) == 0 )
               {
                 inputFile.getline( In_String , 127 , '\n' );
                 for( i=0; i<128; i++ )
                   {
                     if( In_String[i] == '/' ) In_String[i] = ' ';
                   }
                 sscanf( In_String , "%d%d%d%d%d%d%d%d%d" ,
                                     &triangleArray[ fCount ].vinds[0] ,
                                     &triangleArray[ fCount ].tinds[0] ,
                                     &triangleArray[ fCount ].ninds[0] ,
                                     &triangleArray[ fCount ].vinds[1] ,
                                     &triangleArray[ fCount ].tinds[1] ,
                                     &triangleArray[ fCount ].ninds[1] ,
                                     &triangleArray[ fCount ].vinds[2] ,
                                     &triangleArray[ fCount ].tinds[2] ,
                                     &triangleArray[ fCount ].ninds[2] );
                 for( i=0; i<3; i++ )
                   {
                      triangleArray[ fCount ].vinds[i] -= 1;
                      triangleArray[ fCount ].tinds[i] -= 1;
                      triangleArray[ fCount ].ninds[i] -= 1;
                   }
                 fCount++;
               }

          }
        inputFile.close();
      }

    setPlaneEqns();

    boundingBox[0] = vertexArray[0];
    boundingBox[1] = vertexArray[0];

    for( i=1;i<vertexCount;i++)  // brute force bounding box calculation
      {
        if( vertexArray[i].x < boundingBox[0].x ) boundingBox[0].x = vertexArray[i].x ;
        if( vertexArray[i].y < boundingBox[0].y ) boundingBox[0].y = vertexArray[i].y ;
        if( vertexArray[i].z < boundingBox[0].z ) boundingBox[0].z = vertexArray[i].z ;
        if( vertexArray[i].x > boundingBox[1].x ) boundingBox[0].x = vertexArray[i].x ;
        if( vertexArray[i].y > boundingBox[1].y ) boundingBox[0].y = vertexArray[i].y ;
        if( vertexArray[i].z > boundingBox[1].z ) boundingBox[0].z = vertexArray[i].z ;
      }
 
    // set up some planes that define the bounding box....


    boundingPlanes[0].set(  1, 0, 0, boundingBox[1].x );
    boundingPlanes[1].set( -1, 0, 0, boundingBox[0].x );
    boundingPlanes[2].set(  0, 1, 0, boundingBox[1].y );
    boundingPlanes[3].set(  0,-1, 0, boundingBox[0].y );
    boundingPlanes[4].set(  0, 0, 1, boundingBox[1].z );
    boundingPlanes[5].set(  0, 0,-1, boundingBox[0].z );


    return true;

  }

void objFile::setPlaneEqns( void )
  {
    for( uint i=0; i<triangleCount; i++ )
      {
        triangleArray[i].set( vertexArray[ triangleArray[i].vinds[0] ],
                              vertexArray[ triangleArray[i].vinds[1] ],
                              vertexArray[ triangleArray[i].vinds[2] ] );
      }
  }

void objFile::calc_lights( cLight *light , bool additive , float rotx , float roty )
  {
    float col[3];
    matrix m;
    m.rotateY( roty ).rotateX( rotx );
    for( uint i=0; i<triangleCount; i++ )
      {
        for( uint j=0; j<3; j++ )
          {
            //if( triangleArray[i].distance ((*light)) >= 0 )
              {
                light->setFragmentColour( vertexArray[   triangleArray[ i ].vinds[j] ].CurrentPos,
                                          m*normalArray[   triangleArray[ i ].ninds[j] ],
                                          col );
                if( additive )
                  {
                    triangleArray[ i ].colour[3*j  ] += col[0];
                    triangleArray[ i ].colour[3*j+1] += col[1];
                    triangleArray[ i ].colour[3*j+2] += col[2];
                  }
                else
                  {
                    triangleArray[ i ].colour[3*j  ] = col[0];
                    triangleArray[ i ].colour[3*j+1] = col[1];
                    triangleArray[ i ].colour[3*j+2] = col[2];
                  }
              }
          }
      }
  }


/*##############################################################################
###                         FUNCTION TO DRAW VERTICIES.
##############################################################################*/

/*
 * Use a for loop to go through all the triangles, and draw them.
 *
*/

bool objFile::draw( void )
  {
    if( triangleCount == 0 ) return false;
 
    glPushMatrix();
    glBegin(GL_TRIANGLES);
    uint i , j;

      for( i=0; i<triangleCount; i++ )
        {
          for( j=0; j<3; j++ )
             {
               glTexCoord2fv( &texCoordArray[ triangleArray[ i ].tinds[j] ].pos[0] );
               glColor4f    (  triangleArray[ i ].colour[3*j     ], 
                               triangleArray[ i ].colour[3*j + 1 ],
                               triangleArray[ i ].colour[3*j + 2 ], 0.4 );
               glVertex3v   (  vertexArray[   triangleArray[ i ].vinds[j] ].CurrentPos );
             }
        }
    glEnd();
    glPopMatrix();

    return true;
  }

bool objFile::drawSpecular( cPointLight *light , float shininess )
  {

    if( triangleCount == 0 ) return false;
 
    glPushMatrix();
    glBegin(GL_TRIANGLES);
    uint i , j;
    float specColour[3];

      for( i=0; i<triangleCount; i++ )
        {
          for( j=0; j<3; j++ )
             {
               light->setSpecularColourFragment( vertexArray[   triangleArray[ i ].vinds[j] ] ,
                                                 normalArray[   triangleArray[ i ].ninds[j] ] ,
                                                 specColour , shininess );
               glTexCoord2fv( &texCoordArray[ triangleArray[ i ].tinds[j] ].pos[0] );
               glColor4f    (  specColour[0] , specColour[1] , specColour[2] , 0.5 );
               glVertex3v   (  vertexArray[   triangleArray[ i ].vinds[j] ] );
             }
        }
    glEnd();
    glPopMatrix();


  }

/*##############################################################################
###                            OBJFILE DESTRUCTOR
##############################################################################*/

/*
** Delete all dynamic arrays
*/

objFile::~objFile(void)
  {
     if( vertexArray != NULL )
       {
          delete [] vertexArray;
          delete [] normalArray;
          delete [] texCoordArray;

          delete [] triangleArray;
       }
  };
