///////////////////////////////////////////////////////////////////////////////////
//
//
//      NURBS interface example v0.1
//
//      Robert Bateman : 28/02/01
//
//      Small demo program that allows you to 
//      edit all attributes of a nurbs surface
//
///////////////////////////////////////////////////////////////////////////////////


//_______________________________________________________________ Includes ________


#include <string.h>
#include "./fruity_nurbs.h"
#include "./fruity_defs.h"

#ifdef WIN32
  #include <GL/glut.h>
  #include <GL/glui.h>
#else
  #include "./glut.h"
  #include "./glui.h"
#endif

//____________________________________________________________ Global Data ________


uint  uGrid                ;        //   Display List For Grid
bool  bLMbPress    = false ;        //   Is LMB pressed?
bool  bMMbPress    = false ;        //   is RMB pressed?
int   iModifier           ,         //   Modifier Value for ALT key
      iMousePressX        , 
      iMousePressY         ;
float fSpinX       = 0    ,         //   Spin Rotation In X
      fSpinY       = 0    ,         //   Spin Rotation In y
      fDolly       = 0    ,         //   Camera Dolly Value
      fTranslateX  = 0    ,         //   Translation in X
      fTranslateY  = 0     ;        //   Translation in Y


/*
** ---====####  Nurbs Surface Declaration  ####====---
*/

cNurbsSurface   NurbSurface;


/*
** ---====####  Declaration of GLUI controls & live variables  ####====---
*/

GLUI *window_controls;

GLUI_Panel *DOMAIN_DISTANCE_CONTROLS ,
           *PATH_LENGTH_CONTROLS     ,
           *PARAMETRIC_CONTROLS       ;

GLUI_Translation *x_pos ,
                 *y_pos ,
                 *z_pos ;

int         iMode         = 0 ,
            bWireframe    = 0 ,
            bFlatShaded   = 0 ,
            bDisplayCVs   = 0 ,
            bDisplayHulls = 0 ,
            bDisplayNurbs = 1 ,
            iCvU          = 0 ,
            iCvV          = 0 ,
            iWinHandle         ;

float       fsmpTol       = 100.0 ,
            psmpTol       = 20.0 ,
            uStep         = 5.0,
            vStep         = 5.0,
            xP                 ,
            yP                 ,
            zP                 ;



//____________________________________________________ Function Prototypes ________


void  Init          ( void );
void  Screen_Display( void );
void  Screen_Reshape( int  ,
                      int  );
void  Mouse_Motion  ( int  ,
                      int  );
void  Mouse_Button  ( int  ,
                      int  ,
                      int  ,
                      int  );

//__________________________________________________ GLUI callback Function ________

void  GLUI_CallbackFunc( int ID )

  //
  //   This function responds to user imput and updates the 
  //   nurbs surface 
  //

  {


    //
    // ---====####  Set Nurbs Surface Mode  ####====---
    //

    switch( iMode )
      {
        case 1:
          NurbSurface.setSamplingMethod( GLU_PARAMETRIC_ERROR );
          NurbSurface.setParametricTolerance( psmpTol );
        break;
  
        case 2:
          NurbSurface.setSamplingMethod( GLU_DOMAIN_DISTANCE );
          NurbSurface.set_Ustep( uStep );
          NurbSurface.set_Vstep( vStep );
        break;

        default:
          NurbSurface.setSamplingMethod( GLU_PATH_LENGTH );
          NurbSurface.setSamplingTolerance( fsmpTol );
        break;
      }

    if( ID == 101 )
      {
        NurbSurface.setCV( iCvU , iCvV , xP , yP , zP ); 
      }

    if( ID == 99 )
      {
        float *pos = NurbSurface.getCV( iCvU , iCvV );

        if( pos )
          {
            xP = pos[0];
            yP = pos[1];
            zP = pos[2];
          }
      }


    //
    // ---====####  If we need to change the drawing style  ####====---
    //

    if( ID == 1 )
      {
         NurbSurface.drawStyle();
      }

    GLUI_Master.sync_live_all();

    //
    // ---====####  Set the window to the 3d view  ####====---
    //

    if ( glutGetWindow() != iWinHandle ) 
         glutSetWindow(iWinHandle);

    //
    // ---====####  Set the shading model if it has been changed  ####====---
    //

    if( ID == 2 )
      {
         if( bFlatShaded )
           {
             glShadeModel( GL_FLAT );
           }
         else
           {
             glShadeModel( GL_SMOOTH );
           }
      }


    //
    // ---====####  Redraw 3D view  ####====---
    //

    glutPostRedisplay();
  }




//__________________________________________________ GLUI init Function ________

void  Init_glui( void )

  //
  //  All glui calls required to create the ui window.
  //

  {

    GLUI_Spinner *spin;
    GLUI_Listbox *list;

    //
    // ---====####  Create GLUI window  ####====---
    //
    
    window_controls = GLUI_Master.create_glui( "nurbs controls" , 0 );

    //
    // ---====####  Add Rendering Options  ####====---
    //

    window_controls->add_checkbox( "Wire Frame"  , &bWireframe  , 1 , GLUI_CallbackFunc );
    window_controls->add_checkbox( "Flat Shaded" , &bFlatShaded , 2 , GLUI_CallbackFunc );

    window_controls->add_separator( );

    window_controls->add_checkbox( "show surfaces" , &bDisplayNurbs , 0 , GLUI_CallbackFunc );
    window_controls->add_checkbox( "show cv's"     , &bDisplayCVs   , 0 , GLUI_CallbackFunc );
    window_controls->add_checkbox( "show hulls"    , &bDisplayHulls , 0 , GLUI_CallbackFunc );

    window_controls->add_separator( );

    uint limit = iCvU - 1;
    iCvU = 0;
    spin = window_controls->add_spinner( "CV in U"          ,
                                          GLUI_SPINNER_INT  , 
                                         &iCvU              ,
                                          99                ,
                                          GLUI_CallbackFunc );
    spin->set_int_limits( 0 , limit );

    limit = iCvV - 1;
    iCvV  = 0;
    spin  = window_controls->add_spinner( "CV in V"          ,
                                           GLUI_SPINNER_INT  , 
                                          &iCvV              ,
                                           99               ,
                                           GLUI_CallbackFunc );
    spin->set_int_limits( 0 , limit );

    GLUI_Panel *pan = window_controls->add_panel( "CV co-ordinate" ); 

    window_controls->add_edittext_to_panel   (  pan , "x coord" , 
                                                GLUI_EDITTEXT_FLOAT ,
                                                &xP                 ,
                                                101 ,
                                                GLUI_CallbackFunc   );
    window_controls->add_edittext_to_panel   (  pan , "y coord" , 
                                                GLUI_EDITTEXT_FLOAT ,
                                                &yP                 ,
                                                101 ,
                                                GLUI_CallbackFunc   );
    window_controls->add_edittext_to_panel   (  pan , "z coord" , 
                                                GLUI_EDITTEXT_FLOAT ,
                                                &zP                 ,
                                                101 ,
                                                GLUI_CallbackFunc   );
    window_controls->add_column_to_panel( pan , true );
    window_controls->add_translation_to_panel(  pan                ,
                                               "x"                 ,
                                                GLUI_TRANSLATION_X ,
                                               &xP                 ,
                                                101                ,
                                                GLUI_CallbackFunc  );
    window_controls->add_column_to_panel( pan , false );
    window_controls->add_translation_to_panel(  pan                ,
                                               "y"                 ,
                                                GLUI_TRANSLATION_Y ,
                                               &yP                 ,
                                                101                ,
                                                GLUI_CallbackFunc  );
    window_controls->add_column_to_panel( pan , false );
    window_controls->add_translation_to_panel(  pan                ,
                                               "z"                 ,
                                                GLUI_TRANSLATION_Z ,
                                               &zP                 ,
                                                101                ,
                                                GLUI_CallbackFunc  );


    window_controls->add_separator( );

    //
    // ---====####  Add Tessalation method list  ####====---
    //

    list = window_controls->add_listbox( "Tessalation Method" , &iMode , 8 , GLUI_CallbackFunc );

    list->add_item( 0 , "GLU_PATH_LENGTH" );
    list->add_item( 1 , "GLU_PARAMETRIC_ERROR" );
    list->add_item( 2 , "GLU_DOMAIN_DISTANCE" );


    //
    // ---====####  Add Path Length Controls  ####====---
    //


    PATH_LENGTH_CONTROLS = window_controls->add_panel           ( "GLU_PATH_LENGTH"        );
    spin                 = window_controls->add_spinner_to_panel(  PATH_LENGTH_CONTROLS    ,
                                                                  "GLU_SAMPLING_TOLERANCE" ,
                                                                   GLUI_SPINNER_FLOAT      , 
                                                                  &fsmpTol                 ,
                                                                   5                       ,
                                                                   GLUI_CallbackFunc       );
    spin->set_float_limits( 10.0 , 200.0 );
    spin->set_speed( 0.6 );


    //
    // ---====####  Add Paramatric error Controls  ####====---
    //

    PARAMETRIC_CONTROLS = window_controls->add_panel           ( "GLU_PARAMETRIC_ERROR"     );
    spin = window_controls->add_spinner_to_panel(  PARAMETRIC_CONTROLS       ,
                                                  "GLU_PARAMETRIC_TOLERANCE" ,
                                                   GLUI_SPINNER_FLOAT        , 
                                                  &psmpTol                   ,
                                                   6                         ,
                                                   GLUI_CallbackFunc         );
    spin->set_float_limits( 0.1 , 50.0 );
    spin->set_speed( 0.6 );


    //
    // ---====####  Add Domain Distance Controls  ####====---
    //

    DOMAIN_DISTANCE_CONTROLS = window_controls->add_panel           ( "GLU_DOMAIN_DISTANCE"     );
    spin = window_controls->add_spinner_to_panel(  DOMAIN_DISTANCE_CONTROLS ,
                                                  "GLU_U_STEP"              , 
                                                   GLUI_SPINNER_FLOAT       ,
                                                  &uStep                    ,
                                                   3                        ,
                                                   GLUI_CallbackFunc        );
    spin->set_float_limits( 1.0 , 50.0 );
    spin->set_speed( 0.6 );

    spin = window_controls->add_spinner_to_panel(  DOMAIN_DISTANCE_CONTROLS ,
                                                  "GLU_V_STEP"              ,
                                                   GLUI_SPINNER_FLOAT       ,
                                                  &vStep                    ,
                                                   4                        ,
                                                   GLUI_CallbackFunc        );
    spin->set_float_limits( 1.0 , 50.0 );
    spin->set_speed( 0.6 );


  }


//__________________________________________________________ Init Function ________

void Init( void )


  //
  //  This function sets up openGL and loads required data
  //

  {

    /*
    ** ---====####  OpenGL Initialisation  ####====---
    */

    glClearColor( 0.5 , 0.5 , 0.6 , 0.0 );
    glShadeModel ( GL_SMOOTH );
    glEnable    ( GL_DEPTH_TEST  );


    /*
    ** ---====####  Lighting Set-Up  ####====---
    */

    {
      GLfloat ambient[]  = { 0.3f , 0.3f ,  0.3f , 1.0f };
      GLfloat diffuse[]  = { 0.4f , 0.4f ,  0.4f , 1.0f };
      GLfloat specular[] = { 0.8f , 0.8f ,  0.8f , 1.0f };
      GLfloat position[] = { 0.5f , 0.1f , 10.0f , 0.0f };

      glLightfv( GL_LIGHT0 , GL_AMBIENT  , ambient  );
      glLightfv( GL_LIGHT0 , GL_DIFFUSE  , diffuse  );
      glLightfv( GL_LIGHT0 , GL_SPECULAR , specular );
      glLightfv( GL_LIGHT0 , GL_POSITION , position );
    }

    float lmodel_ambient[] = {0.3f, 0.3f, 0.3f, 1.0};
    float local_view[] = {0.0f};

    glLightModelfv( GL_LIGHT_MODEL_AMBIENT      , lmodel_ambient );
    glLightModelfv( GL_LIGHT_MODEL_LOCAL_VIEWER , local_view     );
 
    glEnable( GL_LIGHTING );
    glEnable( GL_LIGHT0   );
    glEnable( GL_AUTO_NORMAL );
    glEnable( GL_NORMALIZE );



    /*
    ** ---====####  Compile 'Grid' Display List  ####====---
    */

    int i ,
        j ;

    uGrid = glGenLists(1);
    glNewList( uGrid , GL_COMPILE );


      glDisable( GL_LIGHTING );
      glDisable( GL_LIGHT0   );

      glBegin(GL_LINES);

        glColor3f( 0,0,0 );

        for(i=-100;i<=100;i+=10)
          {
            glVertex3f(    i , 0 , -100 );
            glVertex3f(    i , 0 ,  100 );
            glVertex3f(  100 , 0 ,    i );
            glVertex3f( -100 , 0 ,    i );
          }

      glEnd();

      glEnable( GL_LIGHTING );
      glEnable( GL_LIGHT0   );

    glEndList();


    /*
    ** ---====####  Load Nurbs Surface Data  ####====---
    */

    NurbSurface.loadNurbsData( "./data/plane.nurb" );

    iCvU = NurbSurface.getCvCountU();
    iCvV = NurbSurface.getCvCountV();

    {
       float *pos = NurbSurface.getCV( 0 , 0 );
       if( pos )
         {
           xP = pos[0];
           yP = pos[1];
           zP = pos[2];
         }
    }
    

  }




//_______________________________________________________ Display Function ________

void Screen_Display( void )

  {

    glClear( GL_COLOR_BUFFER_BIT |
             GL_DEPTH_BUFFER_BIT  );

    glLoadIdentity();
    gluLookAt ( 0 ,  1 , 25 ,    // eye Position
                0 ,  0 ,  0 ,    // aim Position
                0 ,  1 ,  0 );   // camera up vector

    glPushMatrix();


      glTranslatef( fTranslateX ,
                    fTranslateY ,
                    fDolly      );
      glRotatef   ( fSpinX  , 
                    1.0     ,
                    0.0     ,
                    0.0     );
      glRotatef   ( fSpinY  ,
                    0.0     ,
                    1.0     ,
                    0.0     );

        glPushMatrix();
          glCallList( uGrid );
          glPopMatrix();
            if( bDisplayCVs   )
              NurbSurface.drawCVs();
            if( bDisplayHulls )
              NurbSurface.drawHulls();
            if( bDisplayNurbs )
              NurbSurface.drawSurface();
          glPopMatrix();
        glPopMatrix();


      glPopMatrix();


    glutSwapBuffers();
  }


//__________________________________________________ Mouse Button Function ________

void Mouse_Button( int iButton , int bState , int iXpos , int iYpos ) 


  //
  // Function to deal with clicks of the mouse
  //


  {
    iModifier = glutGetModifiers();

    if( bState == GLUT_DOWN )
      {
        iMousePressX = iXpos; 
        iMousePressY = iYpos;

        switch( iButton ) 
          {
            case GLUT_LEFT_BUTTON:
                bLMbPress = true;
            break;

            case GLUT_MIDDLE_BUTTON:
                bMMbPress = true;
            break;

            case GLUT_RIGHT_BUTTON:
                bMMbPress = true;
            break;

            default: 
            break;
          }
      }
    if( bState == GLUT_UP )
      { 
        switch( iButton )
          {
            case GLUT_LEFT_BUTTON:
              glutSetCursor( GLUT_CURSOR_INHERIT );
              bLMbPress = false;
            break;

            case GLUT_MIDDLE_BUTTON:
              glutSetCursor( GLUT_CURSOR_INHERIT );
              bMMbPress = false;
            break;

            case GLUT_RIGHT_BUTTON:
                bMMbPress = false;
            break;

            default: 
            break;
          }
      }
    glutPostRedisplay();
  }




//__________________________________________________ Mouse Motion Function ________


void Mouse_Motion( int iXpos , int iYpos )


  //
  //     This Function Checks to see if the 'Alt' key is pressed
  //  and performs mouse movement.
  //


  {

    /*
    ** ---====####  Record Movement When Mouse is Pressed  ####====---
    */

    int iStepX = ( iXpos - iMousePressX ) ,
        iStepY = ( iYpos - iMousePressY ) ;

    iMousePressX = iXpos ;
    iMousePressY = iYpos ;


    /*
    ** ---====####  If Alt Key is Pressed  ####====---
    */

    if( iModifier == GLUT_ACTIVE_ALT )

      {

        /*
        ** ---====#### Tumble Transformation  ####====---
        */

        if( bLMbPress && 
           !bMMbPress )  
          {
            #ifdef WIN32
               glutSetCursor(GLUT_CURSOR_CROSSHAIR);
            #else
               glutSetCursor(GLUT_CURSOR_CYCLE);
            #endif

            fSpinX += ( 0.5 * iStepY );
            fSpinY += ( 0.5 * iStepX );
          }

        else


        /*
        ** ---====#### Track Transformation  ####====---
        */

        if( bMMbPress &&
           !bLMbPress )
          {
            #ifdef WIN32
               glutSetCursor(GLUT_CURSOR_SPRAY);
            #else
               glutSetCursor(GLUT_CURSOR_CROSSHAIR);
            #endif

            fTranslateX  += ( 0.2 * iStepX );
            fTranslateY  -= ( 0.2 * iStepY );
          }
        else

        /*
        ** ---====#### Dolly Transformation  ####====---
        */

        if( bLMbPress &&
            bMMbPress  )
          {
            glutSetCursor( GLUT_CURSOR_LEFT_RIGHT );
            fDolly += ( 0.2 * iStepX );
          }

      }
    glutPostRedisplay();
  }


//________________________________________________ Screen Reshape Function ________

void Screen_Reshape(int w, int h)


  //
  //  This Function Re-sizes the screen
  //


  {

    if( h == 0 )
      {
        h = 1;
      }

    glViewport     ( 0 , (GLsizei) 0 , (GLsizei) w , (GLsizei) h );
    glMatrixMode   (GL_PROJECTION);
    glLoadIdentity ();
    gluPerspective ( 45.0 ,(GLfloat) w/(GLfloat) h ,  0.01 , 500 );
    glMatrixMode   (GL_MODELVIEW);
    glLoadIdentity ();

  }




//__________________________________________________________ Main Function ________

int main( int argc , char** argv )


  //
  //  This function initialises the window and registers the glut callbacks
  //


  {


    /*
    **  ---====####  INIT MAIN WINDOW  ####====---
    */

    glutInit              ( &argc , argv );
    glutInitDisplayMode   ( GLUT_DOUBLE |
                            GLUT_RGBA   |
                            GLUT_DEPTH  );
    glutInitWindowSize    (  640  ,  480  );
    glutInitWindowPosition(  100  ,  100  );
    iWinHandle = glutCreateWindow      ( "Simple Nurbs Example" );

    cout << "alt + LMB : rotate"     << endl
         << "alt + RMB : track"      << endl
         << "alt + LMB + RMB : zoom" << endl;

    /*
    **  ---====####  DO OPENGL INITIALISATION  ####====---
    */

    Init();

    /*
    **  ---====####  DO GLUI INITIALISATION  ####====---
    */

    Init_glui();


    /*
    **  ---====####  REGISTER GLUT CALLBACKS  ####====---
    */

    glutDisplayFunc ( Screen_Display    );
    glutReshapeFunc ( Screen_Reshape    );
    GLUI_Master.set_glutMouseFunc   ( Mouse_Button      );
    glutMotionFunc  ( Mouse_Motion      );


    glutMainLoop();
    return 0;

  }




