/*######################################################################
###
###   OBJECT LOADER HEADER FILE
###
######################################################################*/

#ifndef OBJLOADER_H
#define OBJLOADER_H

#include "./opengl_includes.h"
#include "./fruity_exception.h"
#include "./fruity_math.h"
#include "./scene_lighting.h"

#include <string.h>
#include <iostream.h>
#include <fstream.h>


// use these functions to pass in cVector3f's into gl geometry calls

inline void glVertex3v( cVector3f &vert ) {
    glVertex3f( vert.x , vert.y , vert.z );
  }

inline void glNormal3v( cVector3f &norm ) {
    glNormal3f( norm.x , norm.y , norm.z );
  }

inline void glTranslatev( cVector3f &norm ) {
    glTranslatef( norm.x , norm.y , norm.z );
  }


class Vertex : public cVector3f
  {
     public:

       Vertex( void ) : cVector3f() 
         {
           inShadow = false;
           usedForShadowVolume = false;
         }

       inline Vertex& operator=( const cVector3f &v )
         {
           x = v.x; y = v.y; z = v.z;
           return *this;
         };

       friend bool operator==( const Vertex &V1 , const Vertex &V2  )
         {
           if( V1.x == V2.x &&
               V1.y == V2.y &&
               V1.z == V2.z )
           return true;
           return false;
         };

       cVector3f CurrentPos; // position after transformations...
       cVector3f ShadowVec;  // position of vector after transformed to shadow space
       bool      inShadow;   // if vertex is in shadow....
       float     zDepth;     // distace from light source to current deformed position
       bool      usedForShadowVolume;
  };

class Normal : public cVector3f
  {
     public:

       Normal( void ) : cVector3f()
         {
         }
  };

class TexCoord
  {
     public:

       float pos[2];
  };

class Triangle : public plane4f
  {
     public:

       Triangle( void ) : plane4f()
         {
           shadowEdgeMarkers[0] = false;
           shadowEdgeMarkers[1] = false;
           shadowEdgeMarkers[2] = false;
           for( uint i=0; i<9; i++ )
              colour[i] = 0.5;
         }

       uint      tinds  [3] ,
                 ninds  [3] ,
                 vinds  [3] ;
       float     colour [9] ;
       bool      shadowEdgeMarkers[3];

  };


class objFile
  {
     public:

       //
       // redeclared pure virtual functions inherited from 'geometry' class
       //

       virtual bool load  ( char *filename );
       virtual bool draw  ( void );
       bool         drawSpecular( cPointLight *light , float fShininess = 15 );

       void calc_lights( cLight *light , bool additive = false, float rotx=0 , float roty=0 );

       void setPlaneEqns( void );
       


       objFile( void ) 
         {
           vertexCount   = 0;
           normalCount   = 0;
           texCoordCount = 0;
           triangleCount = 0;
           vertexArray   = NULL;
           normalArray   = NULL;
           texCoordArray = NULL;
           triangleArray = NULL;
		   bRecievesShadows = false;
           bCastsShadows    = false;
           boundingBox[0].set(0,0,0);
           boundingBox[1].set(0,0,0);
         }
       ~objFile(void);



     //private:

       unsigned int    vertexCount;       //  Total Number Of Verticies
       Vertex         *vertexArray;       //  Array Of Verticies

       unsigned int    normalCount;       //  Total Number Of Normals
       Normal         *normalArray;       //  Array Of Normals

       unsigned int    texCoordCount;     //  Total Number Of TexCoords
       TexCoord       *texCoordArray;     //  Array Of TexCoords

       unsigned int    triangleCount;     //  Total Number Of Triangles
       Triangle       *triangleArray;     //  Array Of Triangles

       bool bRecievesShadows,
            bCastsShadows;

       cVector3f boundingBox[2];
       plane4f   boundingPlanes[6];

       //static objFile *LinkedLightList;

  };

#endif
