/******************************************************************************/
/*                                                                            */
/*        Project: Boona Taxi                                                 */
/*          Title: BSP tree functions                                         */
/*        Version: 0.1                                                        */
/*                                                                            */
/*        Started: 01/02/2001                                                 */
/*   Last Updated: 14/02/2001                                                 */
/*                                                                            */
/*        Authors: Michael Bonnington, Rob Bateman                            */
/*                                                                            */
/******************************************************************************/

/* bsp.h: Contains functions for the compilation and traversal of a BSP (Binary
 *        Space Partition) tree containing the level geometry.
 */

#ifndef BSP_H
#define BSP_H

#include <iostream.h>
#include "./win32.h"
#include "./callbacks.h"
#include "./fruity_vector3f.h"

#define CP_BACK 0
#define CP_FRONT 1
#define CP_COINCIDENT 2



/* Test if a point is behind or in front of a plane ==========================*/
/*int classifyPoint(cVector3f *cam_pos, polygon *plane)
{
   float result;
   cVector3f *vector1 = (cVector3f *)&plane->vertices[0];
   cVector3f direction = (*vector1)-(*cam_pos);
   result = direction*plane->face_normal;   //dot product
   if(result < -0.001) return CP_FRONT;
   else if(result > 0.001) return CP_BACK;
   else return CP_COINCIDENT;
}*/

/* Walk through the tree and render polygons =================================*/
/*int walkBSPTree(node *Node, cVector3f *cam_pos)
{
   if(Node->isLeaf) return;

   int result = classifyPoint(cam_pos, Node->splitter);
   if(result == CP_FRONT)   //change to CP_BACK to reverse rendering order
   {
      if(Node->back != NULL walkBSPTree(Node->back, cam_pos);
      drawPoly(&Node->splitter, m_pVertexArray, m_pNormalArray, m_pTexCoordArray);
      if(Node->front != NULL walkBSPTree(Node->front, cam_pos);
   }
   else
   {
      if(Node->front != NULL walkBSPTree(Node->front, cam_pos);
      //drawPoly(&Node->splitter, m_pVertexArray, m_pNormalArray, m_pTexCoordArray);
      if(Node->back != NULL walkBSPTree(Node->back, cam_pos);
   }
   return;
}*/

/* Node structure ============================================================*/
class node
{
   public:

      /* Constructor... */
      node(void)
      {
         sib = NULL;
      }

      void add_sibling(node *Ptr)
      {
         if(sib == NULL)
         {
            sib = Ptr;
         }
         else
         {
            sib->add_sibling(Ptr);
         }
      }

//      polygon *splitter;   //current polygon being used to partition the space

      node *sib,       //pointer to next polygon in linked list
           *infront,
           *behind;

      bool isLeaf,    //is this a leaf node?
           isSolid;   //is this space solid? (useful for collisions, etc.)
};

/* Polygon structure =========================================================*/
class polygon : public node
{
   public:

      bool triangle;
      friend void drawPoly(polygon *, float *, float *, float *);

      /* Constuctor... */
      polygon(void) : node()
      {
         vertices = NULL;
         normals  = NULL;
         uvCoords = NULL;
      };

      /* A bodge to allocate the correct amount of memory depending on the
       * polygon being a triangle or a quad...
       */
      bool alloc(uint isTriangle)
      {
         if(vertices) delete [] vertices;
         if(normals)  delete [] normals;
         if(uvCoords) delete [] uvCoords;

         if(isTriangle == 1)
         {
            vertices = new uint[3];
            normals  = new uint[3];
            uvCoords = new uint[3];

            triangle = true;
         }
         else
         {
            vertices = new uint[4];
            normals  = new uint[4];
            uvCoords = new uint[4];

            triangle = false;
         }
         return true;
      }

      uint *vertices ,
           *normals  ,
           *uvCoords ;

      cVector3f face_normal;   //needs to be calculated and set somewhere.
                               //perhaps use fruity_plane4f.h ?

};


/*class BSPTRREEEEE
  {
    public :

      bool Load( char *filename )
        {
          HeadNode = LoadObjFile( filename , &m_pVertexArray , &m_pNormalArray , &m_pTexCoordArray ) ;
          if( HeadNode == NULL ) 
            {
              return false;
            }
          return true;
        }


    protected :


      float *m_pVertexArray;
      float *m_pNormalArray;
      float *m_pTexCoordArray;

      polygon *HeadNode;

  };*/







#endif
