/*#####################################################################################
 *
 *   Title        : fruity_exception.cpp
 *   Author       : Robert Bateman
 *   Date         : 30-Nov-2000
 *   Last Updated : 30-Nov-2000
 *
#####################################################################################*/

/*
 *   Revision History :
 *
 *
 *
*/


#include "./fruity_exception.h"
#include "./opengl_includes.h"
#include <iostream.h>

float divide_f( float _x , float _y )
  {
    if( _y == 0 ) return 0.0001;

    return (float)_x/_y;
  }

double divide_d( double _x , double _y )
  {
    if( _y == 0 ) return 0.0001;

    return (double)_x/_y;
  }

float divide_i( int _x , int _y )
  {
    if( _y == 0 ) return 0.0001;

    return (float)_x/_y;
  }

/*##############################################################################
###                           OUTPUT LOG CREATION
##############################################################################*/

#ifdef HANDLE_MY_EXCEPTIONS
  #ifdef GENERATE_EXCEPTION_LOG

    #include <fstream.h>
    #include <iostream.h>

   ofstream ofs_ex_log("./logs/runtime_error_log.txt",ios::out);;

   #ifdef __DEBUG__
   ofstream _ofs_;

   void DebugLog( char *string )
    {
    //  _ofs_.open( "./logs/debug.txt" , ios::ate | ios::app );
      _ofs_ << string << endl;
    //  _ofs_.close(); 
    }


   #endif


    void InitErrorLog( void )
      {
        #ifdef __DEBUG__
        _ofs_.open( "./logs/debug.txt" , ios::out );
        _ofs_ << "Debug Started #####\n" << endl;
       // _ofs_.close();
        #endif

         ofs_ex_log << "***** Error Log Started ******" << endl;
      }

    void ErrorLog( char *string , char *stringy )
      {
         ofs_ex_log << "Error Message : " << string;
         if(stringy) ofs_ex_log << " " << stringy << endl;
         else ofs_ex_log << endl;
      };

  #endif
#endif


#ifdef HANDLE_MY_EXCEPTIONS

/*##############################################################################
###                       TEST TO SEE IF TGA LOADED OK
##############################################################################*/

void loadTexture( char *filename , GLuint *uiTexObj)
  {
     
     glGenTextures( 1 , uiTexObj );

     try 
       {
         int Case;
         if( (Case=loadTGA( filename , *uiTexObj ))!=1 ) throw Case;
         return;
       }
     catch(int a)
       {
         switch( a )
           {

              case TGA_FILE_NOT_FOUND:

                 ErrorLog("Texture File Not Found : " , filename ); 
                 #ifdef EXIT_ON_ALL_EXCEPTIONS
                    exit(0);
                 #endif
                 break;

              case TGA_BAD_DIMENSION:
   
                 ErrorLog("Texture File Not Power Of 2 : ", filename );  
                 #ifdef EXIT_ON_ALL_EXCEPTIONS
                    exit(0);
                 #endif
                 break;

              case TGA_BAD_IMAGE_TYPE:

                 ErrorLog("Texture File Type Not Supported : ", filename ); 
                 #ifdef EXIT_ON_ALL_EXCEPTIONS
                    exit(0);
                 #endif
                 break;

              case TGA_BAD_BITS:

                 ErrorLog("Texture File Type Not Supported : " , filename ); 
                 #ifdef EXIT_ON_ALL_EXCEPTIONS
                    exit(0);
                 #endif
                 break;
 
              case TGA_BAD_DATA:

                 ErrorLog("Texture File Corrupt : " , filename );  
                 #ifdef EXIT_ON_ALL_EXCEPTIONS
                    exit(0);
                 #endif
                 break;
 
              default:
                 break;
            }
       }
     return;
  }

/*##############################################################################
###                           MEMORY CHECKING
##############################################################################*/


bool CheckMemory( void *Ptr, int type )
  {

    try
      {
        switch(type)
          {
            case MEM_CHECK_MALLOC:
              if( Ptr==NULL ) throw (int) ERROR_MEM_ALLOC_FAILED;
             break;

            case MEM_CHECK_NULL_ACCESS:
              if( Ptr==NULL ) throw (int) ERROR_MEM_NULL_DATA_ACCESS;
             break;

            default :
              break;
           }
         return true;
      }
        
    catch(int a)
      {
        switch(a)
          {
            case ERROR_MEM_ALLOC_FAILED:
          
              #ifdef GENERATE_EXCEPTION_LOG
                ErrorLog("**** Fatal Memory Allocation Failure");
              #else
                cout << "Memory Allocation failed\n  *** exiting ***\n" << endl;
              #endif
              exit(0);
              break;

            case ERROR_MEM_NULL_DATA_ACCESS :

              #ifdef GENERATE_EXCEPTION_LOG
                ErrorLog("**** Error : Tried to access NULL");
              #endif

              #ifdef EXIT_ON_ALL_EXCEPTIONS
                exit(0);
              #endif
              break;
 
            default:
              break;
          }
      }
    return false;
  }

#endif
