//
//   TITLE : 	scene_lighting.h
//   AUTHOR :	Robert Bateman
//   NOTES :	Sorts out scenery lighting etc
//


#ifndef LIGHTING_H
#define LIGHTING_H

#include "./fruity_math.h"
#include "./fruity_exception.h"
#include "./opengl_includes.h"

#define CONSTANT  0
#define LINEAR    1
#define QUADRATIC 2

class cLight

  //
  //  class definition for openGL light
  //

  {

    public:

      float  pAmbient  [4];
      float  pDiffuse  [4];
      float  pSpecular [4];
      float  pPosition [4];
      float  pDirection[4];

      float  fConstantAttenuation;
      float  fLinearAttenuation;
      float  fQuadraticAttenuation;
      float  fExponent;

      float  fCutoff;

  };



class cScene

  //
  //  class to hold lighting and fog data
  //

  {

    public  :

      bool      loadScene           ( char * );
      bool      saveScene           ( char * );

      void      setLightPos         ( uint , cVector3f );
      void      setLightAimPos      ( uint , cVector3f );
      void      setLightDirection   ( uint , cVector3f );

      cVector3f getLightPos         ( uint );
      cVector3f getLightAimPos      ( uint );
      cVector3f getLightDirection   ( uint );

      void      setLightColour      ( uint , GLenum , float * );
      float    *getLightColour      ( uint , GLenum );

      void      setLightAttenuation ( uint , char , float );
      float     getLightAttenuation ( uint , char );

      void      setLightExponent    ( uint , float );
      float     getLightExponent    ( uint );

      void      setLightCutoff      ( uint , float );
      float     getLightCutoff      ( uint );

      void      setLightModelAmbient( float , float , float , float );
      float    *getLightModelAmbient( void );


      void      setFogColour    ( float , float , float , float );
      void      setFogDensity   ( float );
      void      setFogFunc      ( GLenum );
      void      setFogStart     ( float );
      void      setFogEnd       ( float );

      float    *getFogColour    ( void );
      float     getFogDensity   ( void );
      GLenum    getFogFunc      ( void );
      float     getFogStart     ( void );
      float     getFogEnd       ( void );

    protected :

      uint      uLightCount;
      cLight   *pLightArray;
      float     pLightModelAmbient[4];
 
      bool      bFogEnabled;
      float     pFogColor[4];
      float     fFogDensity;
      float     fFogStart;
      float     fFogEnd;
      GLenum    eFogFunc;


  };



#endif
