//
//  TITLE  : Soft Edged Shadow Demo 
//
//  AUTHOR : Robert Bateman
//
//  DATE   : 8 March 2001
//
//  NOTES  : Repeatedly draws the shadow on the plane until 
//           a nice soft shadow is formed
//


//______________________________________________________________ Includes

#include <iostream.h>
#include <fstream.h>
#include <malloc.h>
#include <string.h>

#include "./opengl_includes.h"

#ifdef WIN32
  #include <GL/glui.h>
#else
  #include "./glui.h"
#endif


// simple state definitions to be passed into GLUI callback func
#define OBJECT_MOVED 1
#define PLANE_MOVED  2
#define LIGHT_MOVED  3

int iModifier;

#include "./fruity_exception.h"
#include "./fruity_timer.h"
#include "./fruity_font.h" 
#include "./fruity_mayamouse.h"

#include "./objloader.h"
#include "./object_plane.h"
#include "./tga.h"
#include "./scene_lighting.h"

uint winHandle;

//______________________________________________________________ Globals

// Variables to deal with performance counters

float         fProcessingTime   = 0 ,
              fDisplayTime      = 0 ,
              fProcessingLast   = 0 ,
              fDisplayLast      = 0 ,
              fFrametime        = 0 ;
long          frames            = 0 ;
char          pzStatusLine[128]     ;


// 2D & 3D Views held in display lists

GLuint        Persp   ,
              Ortho   ,
              WoodTex ,
              EnvMap  ;

// self defined types to handle geometry & lighting


// The 3D cube

objFile       cube ;

float cube_pos_x = 0,
      cube_pos_y = 3,
      cube_pos_z = 0,
      cube_rot_x = 0,
      cube_rot_y = 0;

// the 3D reflection plane.

cObjectPlane plane ;

float floor_pos_x = 0,
      floor_pos_y = 0,
      floor_pos_z = 0,
      floor_rot_x = 0,
      floor_rot_y = 0;

// The light structure used

cPointLight  pntL ,pntL2 ;

float pntL_pos_x = -7,
      pntL_pos_y = 10,
      pntL_pos_z = 1,
      pntL_area  = 0.5;

//______________________________________________________________ Function Declarations
 
void init( void );
void idle( void );
void display( void );
void reshape( int , int );
void Mouse_Motion( int , int );
void Mouse_Button( int , int , int , int );


//______________________________________________________________ Init Function


void init( void )

  { 
 
    InitFrameCounter();
    makeRasterFont();

    glClearColor( 0.0 , 0.0 , 0.0 , 0.0 );
    glClearStencil( 0x0 );
    glShadeModel( GL_SMOOTH );


      // set up depth testing
    glDepthFunc( GL_LEQUAL );

    glBlendFunc( GL_SRC_ALPHA  ,GL_ONE_MINUS_SRC_ALPHA );

      // load up geometry
    cube.load  ( "./data/cube.obj"   );
    plane.load ( "./data/floor.obj" );


      // load textures
    loadTexture( "./data/ceiling.tga" , &WoodTex );
    loadTexture( "./data/reflect.tga" , &EnvMap );

      // set up and pre-calculate lighting
    pntL.set( pntL_pos_x , pntL_pos_y , pntL_pos_z );

    pntL.pDiffuse[0] = 0.8;
    pntL.pDiffuse[1] = 0.7;
    pntL.pDiffuse[2] = 1.0;
    pntL.pAmbient[0] = 0.6;
    pntL.pAmbient[1] = 0.6;
    pntL.pAmbient[2] = 0.7;
    pntL.fLinearAttenuation    = 0.06 ;
    pntL.fQuadraticAttenuation = 0.00;


    // a copy of the first light is used to generate soft shadows.

    pntL2.pDiffuse[0] = 0.8;
    pntL2.pDiffuse[1] = 0.7;
    pntL2.pDiffuse[2] = 1.0;
    pntL2.pAmbient[0] = 0.6;
    pntL2.pAmbient[1] = 0.6;
    pntL2.pAmbient[2] = 0.7;
    pntL2.fLinearAttenuation    = 0.06 ;
    pntL2.fQuadraticAttenuation = 0.00;

    matrix m;
    m.translate( cube_pos_x , cube_pos_y , cube_pos_z );
    for( uint i=0; i<cube.vertexCount;i++ )
      {
        cube.vertexArray[i].CurrentPos = m*cube.vertexArray[i];
      }
    cube.calc_lights( &pntL , false );
    plane.calc_lights( &pntL , false );

  }



//______________________________________________________________ Idle Function

void idle( void )
  {
    frames++;


    fProcessingLast = GetFrameTime();
    fProcessingTime += fProcessingLast;
    fFrametime = divide_d( (fProcessingTime + fDisplayTime) , frames );

    glutSetWindow( winHandle );
    glutPostRedisplay();
  }




//______________________________________________________________ Display Function

void display( void )

  {
 
    glClear( GL_COLOR_BUFFER_BIT |
             GL_DEPTH_BUFFER_BIT |
             GL_STENCIL_BUFFER_BIT );

    glCallList( Persp );

      // set up mouse transforms

    glPushMatrix();
      Mousey.doTransform();
      glPushMatrix();

          // in this situation, we dont strictly need the 
          // stencil buffer, but it allows us to project
          // cleanly off any given plane

        glStencilOp( GL_REPLACE ,
                     GL_REPLACE ,
                     GL_REPLACE );
        glStencilFunc( GL_ALWAYS , 0x1 , 0x1 );
        glEnable( GL_STENCIL_TEST );
        glEnable( GL_TEXTURE_2D );
        glEnable( GL_DEPTH_TEST );

          // draw plane and mark out stencil buffer region in one go
        glPushMatrix();
          plane.draw();
        glPopMatrix();
 

          // only draw the shadow if the camera is above the plane
        if( plane.distance( camPos ) > 0 )
          {

               // Limit drawing to the stencilled area only.
               // This will force the shadow to be drawn 
               // within the limits of the plane

               // - rethink - this is actually doing nothing. (why?)
               // wont change it just yet though... 28/03/01
             glStencilOp( GL_KEEP ,
                          GL_KEEP ,
                          GL_KEEP );
             glStencilFunc( GL_EQUAL , 0x1 , 0x1 );
             glEnable( GL_BLEND );
             glDisable( GL_DEPTH_TEST );

               // draw the shadow cast by the cube,
               // - this time scale all points until they hit the plane 
             glPushMatrix();
               glColor4f( 0,0,0,0.03 );
               plane.drawShadowOnPlane( cube , &pntL );
               pntL2.set( pntL_pos_x + pntL_area , pntL_pos_y + pntL_area , pntL_pos_z + pntL_area );
               plane.drawShadowOnPlane( cube , &pntL2 );
               pntL2.set( pntL_pos_x - pntL_area , pntL_pos_y - pntL_area , pntL_pos_z - pntL_area );
               plane.drawShadowOnPlane( cube , &pntL2 );
               pntL2.set( pntL_pos_x - pntL_area , pntL_pos_y + pntL_area , pntL_pos_z + pntL_area );
               plane.drawShadowOnPlane( cube , &pntL2 );
               pntL2.set( pntL_pos_x + pntL_area , pntL_pos_y - pntL_area , pntL_pos_z - pntL_area );
               plane.drawShadowOnPlane( cube , &pntL2 );
               pntL2.set( pntL_pos_x - pntL_area , pntL_pos_y + pntL_area , pntL_pos_z - pntL_area );
               plane.drawShadowOnPlane( cube , &pntL2 );
               pntL2.set( pntL_pos_x + pntL_area , pntL_pos_y - pntL_area , pntL_pos_z + pntL_area );
               plane.drawShadowOnPlane( cube , &pntL2 );
               pntL2.set( pntL_pos_x + pntL_area , pntL_pos_y + pntL_area , pntL_pos_z - pntL_area );
               plane.drawShadowOnPlane( cube , &pntL2 );
               pntL2.set( pntL_pos_x - pntL_area , pntL_pos_y - pntL_area , pntL_pos_z + pntL_area );
               plane.drawShadowOnPlane( cube , &pntL2 );
             glPopMatrix(); 

               // disble stencilling, wont need it for next part
             glDisable( GL_STENCIL_TEST );
             glDisable( GL_BLEND );
             glEnable( GL_DEPTH_TEST );

          }

          // finally draw the object
        glPushMatrix();
          glBindTexture( GL_TEXTURE_2D , WoodTex );
          glTranslatef( 0 , 2 , 0 );
          cube.draw();
        glPopMatrix();


        glPushMatrix();
          glColor3f( 1,1,0);
          glDisable( GL_TEXTURE_2D );
          glTranslatev( pntL );
          glutSolidCube( 0.4 );
        glPopMatrix();


      glPopMatrix();
    glPopMatrix();

 
    // Draw Performance Counters on a 2D overlay

    glCallList( Ortho );

    sprintf( pzStatusLine , 
            "Average Processing : %f secs : Last Frame : %f" ,
             divide_d( fProcessingTime , frames ) ,
             fProcessingLast                    );
    font(    pzStatusLine  , 5 , 12 , 1, 1, 0.4 );


    sprintf( pzStatusLine ,
             "Average Display Time : %f secs : Last Frame : %f" ,
             divide_d( fDisplayTime , frames   ) ,
             fDisplayLast                      ) ;
    font(    pzStatusLine , 5 , 22 , 1, 1, 0.4 ) ;


    sprintf( pzStatusLine ,
             "Average fps: %f : Last Frame : %f" ,
             divide_d( 1 , fFrametime) ,
             divide_f( 1 , (fDisplayLast+fProcessingLast) ) );
    font(    pzStatusLine , 5 , 32 , 1, 1, 1 );



    glutSwapBuffers();


    fDisplayLast  = GetFrameTime();;
    fDisplayTime += fDisplayLast;

  }



//______________________________________________________________ Reshape Function

void reshape( int w , int h )
  
  //
  //  The reshape function resets performance counters and
  //  resizes the window.
  //


  {
   
    glDeleteLists(Persp,1);
    glDeleteLists(Ortho,1);
    Persp = glGenLists(1);
    Ortho = glGenLists(1);

    fProcessingTime = 0;
    fDisplayTime    = 0;
    fProcessingLast = 0; 
    fDisplayLast    = 0;
    fFrametime      = 0;
    frames          = 0;

    glNewList(Persp,GL_COMPILE);
      glMatrixMode( GL_TEXTURE );
      glLoadIdentity();
      glViewport(0, (GLsizei) 0, (GLsizei) w, (GLsizei) h);
      glMatrixMode(GL_PROJECTION);
      glLoadIdentity();
      gluPerspective(55.0, divide_f( w , h ) , 0.01, 500 );
      glMatrixMode(GL_MODELVIEW);
      glLoadIdentity();
    glEndList();

    glNewList(Ortho,GL_COMPILE);
      glViewport(0, (GLsizei) 0, (GLsizei) w, (GLsizei) h);
      glMatrixMode(GL_PROJECTION);
      glLoadIdentity();
      glOrtho(0, w, h, 0, -1, 1);
      glMatrixMode(GL_MODELVIEW);
      glLoadIdentity();
      glDisable( GL_TEXTURE_2D );
      glDisable( GL_DEPTH_TEST );
    glEndList();

    glCallList(Persp);
    GetFrameTime();
  }

 
//______________________________________________________________ Mouse Motion Function

void Mouse_Motion(int x, int y)

  //
  //  This function deals with mouse movement events & passes values to
  //  the class defined in "./mayamouse.h" which sorts out maya style input...
  //

  {
    Mousey.motion( x , y );
uint i;

    if( iModifier != GLUT_ACTIVE_ALT )
      {
        if( Mousey.bLMB )
          {
            cube_rot_x += 0.5*Mousey.iStepY;
            cube_rot_y += 0.5*Mousey.iStepX;
            matrix m;
            m.translate( cube_pos_x, cube_pos_y , cube_pos_z);
            m.rotateY( cube_rot_y ).rotateX( cube_rot_x );
            for( i=0; i<cube.vertexCount;i++ )
              {
                cube.vertexArray[i].CurrentPos = m*cube.vertexArray[i];
              }
            cube.calc_lights( &pntL , false ,cube_rot_x,cube_rot_y );
          }
        else
        if( Mousey.bMMB )
          {
            floor_rot_x += 0.5*Mousey.iStepY;
            floor_rot_y += 0.5*Mousey.iStepX;
            matrix m;
            m.translate( floor_pos_x,
                         floor_pos_y ,
                         floor_pos_z );
            m.rotateY( floor_rot_y ).rotateX( floor_rot_x );
            for( i=0; i<cube.vertexCount;i++ )
              {
                plane.vertexArray[i].CurrentPos = m*plane.vertexArray[i];
              }
            plane.set( plane.vertexArray[ plane.triangleArray[0].vinds[0] ].CurrentPos ,
                       plane.vertexArray[ plane.triangleArray[0].vinds[1] ].CurrentPos ,
                       plane.vertexArray[ plane.triangleArray[0].vinds[2] ].CurrentPos );
          }
      }


    glutPostRedisplay();
  }


//______________________________________________________________ Mouse Button Function

void Mouse_Button( int button , int state , int x , int y )

  //
  //  This function deals with mouse button events & passes values to
  //  the class defined in "./mayamouse.h" which sorts out maya style input...
  //
 
  {
    Mousey.buttonPress( button , state , x , y );

    //if

    glutPostRedisplay();
  }


void  GLUI_CallbackFunc( int ID )

  //
  //   This function responds to user imput and updates the 
  //   nurbs surface 
  //

  {
     matrix m;
     uint i,j;
     switch( ID )
       {
          case OBJECT_MOVED:
            cube.setPlaneEqns();
            m.translate( cube_pos_x , cube_pos_y , cube_pos_z);
            m.rotateY( cube_rot_y ).rotateX( cube_rot_x );
            for( i=0; i<cube.vertexCount;i++ )
              {
                cube.vertexArray[i].CurrentPos = m*cube.vertexArray[i];
              }
            cube.calc_lights( &pntL , false ,cube_rot_x,cube_rot_y );
          break;

          case PLANE_MOVED:
            m.translate( floor_pos_x ,
                         floor_pos_y ,
                         floor_pos_z );
            m.rotateY( floor_rot_y ).rotateX( floor_rot_x );
            for( i=0; i<cube.vertexCount;i++ )
              {
                plane.vertexArray[i].CurrentPos = m*plane.vertexArray[i];
              }
            plane.set( plane.vertexArray[ plane.triangleArray[0].vinds[0] ].CurrentPos ,
                       plane.vertexArray[ plane.triangleArray[0].vinds[1] ].CurrentPos ,
                       plane.vertexArray[ plane.triangleArray[0].vinds[2] ].CurrentPos );
          break;

          case LIGHT_MOVED:
            pntL.set( pntL_pos_x , pntL_pos_y , pntL_pos_z );
            cube.calc_lights( &pntL , false ,cube_rot_x,cube_rot_y );
          break;

          default:
          break;
       }

    glutSetWindow( winHandle );
    glutPostRedisplay();
  }

void  initGlui( void )
  {
    GLUI *window_controls = GLUI_Master.create_glui( "controls" , 0 );
    GLUI_Panel *panel = window_controls->add_panel( "Cube" , GLUI_PANEL_EMBOSSED );
    GLUI_Translation *trn;

    window_controls->add_translation_to_panel ( panel              ,
                                               "x"                 ,
                                                GLUI_TRANSLATION_X ,
                                               &cube_pos_x         ,
                                                OBJECT_MOVED       ,
                                                GLUI_CallbackFunc  );
    window_controls->add_column_to_panel      ( panel , false );
    trn =window_controls->add_translation_to_panel ( panel              ,
                                               "y"                 ,
                                                GLUI_TRANSLATION_Y ,
                                               &cube_pos_y         ,
                                                OBJECT_MOVED       ,
                                                GLUI_CallbackFunc  );
    trn->set_y( 3 );
    window_controls->add_column_to_panel      ( panel , false );
    trn = window_controls->add_translation_to_panel ( panel              ,
                                               "z"                 ,
                                                GLUI_TRANSLATION_Z ,
                                               &cube_pos_z         ,
                                                OBJECT_MOVED       ,
                                                GLUI_CallbackFunc  );

    panel = window_controls->add_panel       ( "Ground"            ,
                                                GLUI_PANEL_EMBOSSED);
    window_controls->add_translation_to_panel(  panel              ,
                                               "x"                 ,
                                                GLUI_TRANSLATION_X ,
                                               &floor_pos_x       ,
                                                PLANE_MOVED        ,
                                                GLUI_CallbackFunc  );
    window_controls->add_column_to_panel      ( panel , false );
    window_controls->add_translation_to_panel ( panel              ,
                                               "y"                 ,
                                                GLUI_TRANSLATION_Y ,
                                               &floor_pos_y       ,
                                                PLANE_MOVED        ,
                                                GLUI_CallbackFunc  );
    window_controls->add_column_to_panel      ( panel , false );
    window_controls->add_translation_to_panel ( panel              ,
                                               "z"                 ,
                                                GLUI_TRANSLATION_Z ,
                                               &floor_pos_z       ,
                                                PLANE_MOVED        ,
                                                GLUI_CallbackFunc  );


    panel = window_controls->add_panel       ( "Light"             ,
                                                GLUI_PANEL_EMBOSSED);
    GLUI_Spinner *spin = window_controls->add_spinner_to_panel ( panel,
                                               "light source area" ,
                                                GLUI_SPINNER_FLOAT ,
                                               &pntL_area          );
    spin->set_float_limits( 0, 1 );
    panel =window_controls->add_panel_to_panel( panel              ,
                                               "translation controls",
                                                GLUI_PANEL_NONE    );
    window_controls->add_translation_to_panel ( panel              ,
                                               "x"                 ,
                                                GLUI_TRANSLATION_X ,
                                               &pntL_pos_x         ,
                                                LIGHT_MOVED        ,
                                                GLUI_CallbackFunc  );
    window_controls->add_column_to_panel      ( panel , false );
    window_controls->add_translation_to_panel ( panel              ,
                                               "y"                 ,
                                                GLUI_TRANSLATION_Y ,
                                               &pntL_pos_y         ,
                                                LIGHT_MOVED        ,
                                                GLUI_CallbackFunc  );
    window_controls->add_column_to_panel      ( panel , false );
    window_controls->add_translation_to_panel ( panel              ,
                                               "z"                 ,
                                                GLUI_TRANSLATION_Z ,
                                               &pntL_pos_z         ,
                                                LIGHT_MOVED        ,
                                                GLUI_CallbackFunc  );



  }

//______________________________________________________________ Main Function

int main( int argc , char **argv )

  {

     glutInit(&argc, argv);
     glutInitDisplayMode   ( GLUT_DOUBLE |
                             GLUT_RGBA   |
                             GLUT_DEPTH  |
                             GLUT_STENCIL );
     glutInitWindowSize    ( 800 , 600 );
     glutInitWindowPosition( 100 , 100 );
     winHandle = glutCreateWindow      ( "Realtime Soft Edged Shadow Demo - LMB controls cube, RMB/MMB controls floor, hold ALT to move around view" );

     glutDisplayFunc( display );
     glutReshapeFunc( reshape );

     init();
     initGlui();     


     glutMotionFunc ( Mouse_Motion );

     GLUI_Master.set_glutMouseFunc   ( Mouse_Button );
     GLUI_Master.set_glutIdleFunc    ( idle );

     glutMainLoop();
     return 0;

  }


