//
//  TITLE  : Object Plane Source 
//
//  AUTHOR : Robert Bateman
//
//  DATE   : 15 March 2001
//
//  NOTES  : specified class cObjectPlane that recivies shadows
//           or can act as mirrors. Nothing elegant in here whatsoever,
//           99% of calculation is done by brute force calculation,
//           
//


//______________________________________________________________ Includes


#include "./object_plane.h"


//______________________________________________________________ Calculate Geometric Planes When Needed

void cObjectPlane::setPlanes( void )

  //
  //  four geometric planes that make up the edges of the plane
  // assumes all corners of plane are right angle's - in development......
  //

  {
    midPoint = ( boundingBox[0] + boundingBox[1] )*0.5;
    uint lowest = 0,i,j;

    if( vertexArray )
      {
        float checkvals[] = { 0.0 , 0.0 , 0.0 , 0.0 };

        for( i=0; i<vertexCount; i++ )
          {
            float cur = ( vertexArray[i] - midPoint ).length();
            if( cur > checkvals[lowest] )
              {

                checkvals[lowest] = cur;
                vrefs[lowest]     =   i;

                for(j=0;j<4;j++)
                  {
                    if( checkvals[j] < checkvals[lowest] ) lowest = j;
                  }
              }
          }
      }

    // find edge pairs : start with all possiblilities, & remove 2 unusable ones

    uint edges[]        = { 0 , 1 ,
                            0 , 2 ,
                            0 , 3 ,
                            1 , 2 ,
                            1 , 3 ,
                            2 , 3 };
    float edgeLengths[6];
    uint  dontUseEdges[2];

    // work out edge lengths

    for( i=0 ; i<6; i++ )
      {
         edgeLengths[i] = (vertexArray[vrefs[edges[2*i+1]]] - vertexArray[vrefs[edges[2*i]]]).length();
      }

    // work out which two edges we should ignore

    for( i=0 , j=0; i<2; i++,j+=3 )
      {
        if( edgeLengths[j] > edgeLengths[j+1] )
          {
            if( edgeLengths[j] > edgeLengths[j+2] )
              {
                dontUseEdges[i] = j;
              }
            else dontUseEdges[i] = j+2;
          }
        else
        if( edgeLengths[j+1] > edgeLengths[j+2] )
          {
            dontUseEdges[i] = j+1;
          }
        else dontUseEdges[i] = j+2;
      }

    uint cnt=0;

    for( i=0; i<6; i++ )
      {
        if( dontUseEdges[0] != i && dontUseEdges[1] != i)
          {
            cVector3f middleOfEdge( ( vertexArray[vrefs[edges[2*i+1]]] - 
                                      vertexArray[vrefs[edges[2*i  ]]] ) * 0.5 );
            cVector3f norm( middleOfEdge - midPoint );
            cullEdges[cnt].setNormal( norm.normalized() );
            cullEdges[cnt].d = -(norm.normalized()*vertexArray[vrefs[edges[2*i+1]]]);
            cnt ++;
          }
      }

  }


//______________________________________________________________ redeclared load function

bool cObjectPlane::load  ( char *filename )

  //
  //  Loads Obj file and sets plane equation
  //

  {
    if( !objFile::load( filename ) ) return false;

    setPlanes();

    a=triangleArray[0].a;
    b=triangleArray[0].b;
    c=triangleArray[0].c;
    d=triangleArray[0].d;
  }


//______________________________________________________________ redeclared draw function

bool cObjectPlane::draw( void )

  //
  //  No specific need for this function, the onlt difference is that
  //  it places 4 white cubes at the corners of the plane
  //

  {
    objFile::draw();
    glDisable( GL_TEXTURE_2D );
    for( uint i=0 ; i< 4; i++ )
      {
        glPushMatrix();
          glColor3f( 1,1,1 );
          glTranslatev( vertexArray[ vrefs[i] ].CurrentPos );
          glutSolidCube(0.2);
        glPopMatrix();
      }
    glEnable( GL_TEXTURE_2D );
  }

//______________________________________________________________ draw reflection function

bool cObjectPlane::drawReflectedInPlane ( objFile &obj )

  //
  //  This function scales the speified object through the plane
  //  to mimic the effect of a reflection
  //

  {
    if( obj.triangleCount == 0 ) return false;
 
	if( distance( camPos ) > 0 )
      {
        glPushMatrix();
        glBegin(GL_TRIANGLES);
        for( uint i=0; i<obj.triangleCount; i++ )
          {
                for( uint j=0; j<3; j++ )
                  {
                    
                    glTexCoord2fv( &obj.texCoordArray[ obj.triangleArray[ i ].tinds[j] ].pos[0] );
                    glColor3f    (  obj.triangleArray[ i ].colour[3*j     ], 
                                    obj.triangleArray[ i ].colour[3*j + 1 ],
                                    obj.triangleArray[ i ].colour[3*j + 2 ]);
                    glVertex3v   ( reflectPoint( obj.vertexArray[ obj.triangleArray[ i ].vinds[j] ].CurrentPos ) );
                  }
          }
        glEnd();
        glPopMatrix();
      }
    return true;
  }

//______________________________________________________________ draw shadow function

bool cObjectPlane::drawShadowOnPlane( objFile &obj , cLight *light )

  //
  //  Translates all verticies of specified objFile along a vector
  //  specified by the reletive position of light, until the points
  //  are all lying on the plane.
  //

  {

uint i,j;
    if( obj.triangleCount == 0 ) return false;
 
	if( distance( *light ) > 0 )
      {
        float dist;
        cVector3f pos,dir;
        glPushMatrix();
        glBegin(GL_TRIANGLES);
        for( i=0; i<obj.triangleCount; i++ )
          {
            dist = distance( *light );

              // nasty if statement - if the light is nearer to the plane than the 
              // polygon, dont draw it. The light would not be cast onto the surface,
              // it causes funny problems in that large shadow regions appear for no reason
            if( dist > distance(obj.vertexArray[ obj.triangleArray[ i ].vinds[0] ].CurrentPos)&&
                dist > distance(obj.vertexArray[ obj.triangleArray[ i ].vinds[1] ].CurrentPos)&&
                dist > distance(obj.vertexArray[ obj.triangleArray[ i ].vinds[2] ].CurrentPos) )
              {
                for( j=0; j<3; j++ )
                  {
                    pos = obj.vertexArray[ obj.triangleArray[ i ].vinds[j] ].CurrentPos;
                    dir = obj.vertexArray[ obj.triangleArray[ i ].vinds[j] ].CurrentPos - *light;
                    glVertex3v( (obj.vertexArray[ obj.triangleArray[ i ].vinds[j] ].ShadowVec
                                = pos + dir*lineIntersection(pos,dir)) );
                  }
              }
          }
        glEnd();
        glPopMatrix();
      }
    return true;
  }




