//
//  TITLE  : simple *.obj file loader.
//  AUTHOR : Robert Bateman
//  DATE   : 12/Nov/2001
//  NOTES  : Only does triangles, only 1 mesh. Written explicitly for 
//           innovations project. 
//

//______________________________________________________________________ Includes

#include "./objloader.h"
#include <math.h>

//______________________________________________________________________ Load function

bool objFile::load( char *filename )
  {
    ifstream inputFile( filename );

    if( !inputFile )
      {
        cout << "Error : Unable to open File" 
             << endl;
        return false;
      }


    else
      {

        char In_String[128];
        uint i;

          //
          // count data
          //
        while( !inputFile.eof() )
          {
            inputFile >> In_String;
            
            if( strcmp( In_String , "v"  ) == 0 ) vertexCount++;
            if( strcmp( In_String , "vn" ) == 0 ) normalCount++;
            if( strcmp( In_String , "vt" ) == 0 ) texCoordCount++;
            if( strcmp( In_String , "f"  ) == 0 ) triangleCount++;

          }


          // allocate memory and set vertexCount
        vertexArray   = new Vertex  [ vertexCount   ];
        normalArray   = new Normal  [ normalCount   ];
        texCoordArray = new TexCoord[ texCoordCount ];
        triangleArray = new Triangle[ triangleCount ];

        CheckMemory( vertexArray );
        CheckMemory( normalArray );
        CheckMemory( texCoordArray );
        CheckMemory( triangleArray );

          // close & re-open file stream
        inputFile.close();
        inputFile.open( filename );


          // create some temorary count values
        int vCount = 0 ,
            nCount = 0 ,
            tCount = 0 ,
            fCount = 0 ;


          // get data and place in array of verticies
        while( !inputFile.eof() )
          {
             inputFile >> In_String;
             if( strcmp( In_String , "v"  ) == 0 )
               {
                 inputFile >> vertexArray[ vCount ].pos[0]
                           >> vertexArray[ vCount ].pos[1]
                           >> vertexArray[ vCount ].pos[2];
                 vertexArray[ vCount ].upos[0] = vertexArray[ vCount ].pos[0];
                 vertexArray[ vCount ].upos[1] = vertexArray[ vCount ].pos[1];
                 vertexArray[ vCount ].upos[2] = vertexArray[ vCount ].pos[2];
                 vCount++;
               } 
             else
             if( strcmp( In_String , "vn" ) == 0 )
               {
                 inputFile >> normalArray[ nCount ].pos[0]
                           >> normalArray[ nCount ].pos[1]
                           >> normalArray[ nCount ].pos[2];
                 nCount++;
               }
             else
             if( strcmp( In_String , "vt" ) == 0 )
               {
                 inputFile >> texCoordArray[ tCount ].pos[0]
                           >> texCoordArray[ tCount ].pos[1];
                 tCount++;
               }
             else
             if( strcmp( In_String , "f"  ) == 0 )
               {
                 inputFile.getline( In_String , 127 , '\n' );
                 for( i=0; i<128; i++ )
                   {
                     if( In_String[i] == '/' ) In_String[i] = ' ';
                   }
                 sscanf( In_String , "%d%d%d%d%d%d%d%d%d" ,
                                     &triangleArray[ fCount ].vinds[0] ,
                                     &triangleArray[ fCount ].tinds[0] ,
                                     &triangleArray[ fCount ].ninds[0] ,
                                     &triangleArray[ fCount ].vinds[1] ,
                                     &triangleArray[ fCount ].tinds[1] ,
                                     &triangleArray[ fCount ].ninds[1] ,
                                     &triangleArray[ fCount ].vinds[2] ,
                                     &triangleArray[ fCount ].tinds[2] ,
                                     &triangleArray[ fCount ].ninds[2] );
                 for( i=0; i<3; i++ )
                   {
                      triangleArray[ fCount ].vinds[i] -= 1;
                      triangleArray[ fCount ].tinds[i] -= 1;
                      triangleArray[ fCount ].ninds[i] -= 1;
                   }
                 fCount++;
               }

          }
        inputFile.close();
      }

    return true;

  }


//______________________________________________________________________ Deform function

bool objFile::deform( float timeInc  )

  //
  // simple deformation function that deforms the surface by a sine wave
  //

  {
    uint i;
    Time += 4*timeInc;
    if( vertexArray )
      {
        for( i=0; i< vertexCount ; i++ )
          {
             vertexArray[ i ].pos[1] 
               = vertexArray[ i ].upos[1] 
                 +  0.25*sin(   2*vertexArray[ i ].upos[0]
                            + 2*vertexArray[ i ].upos[2] + Time ) ;
          }
          return true;
      }
    return false;
  };

//______________________________________________________________________ Draw function

/*
 * Use a for loop to go through all the triangles, and draw them.
 *
*/

bool objFile::draw( void )
  {
    if( triangleCount == 0 ) return false;

    glPushMatrix();
    glBegin(GL_TRIANGLES);
    glColor3f( 1,1,1 );

    uint i , j;

      for( i=0; i<triangleCount; i++ )
        {
          for( j=0; j<3; j++ )
             {
               glTexCoord2fv( &texCoordArray[ triangleArray[ i ].tinds[j] ].pos[0] );
               glNormal3fv  ( &normalArray[   triangleArray[ i ].ninds[j] ].pos[0] );
               glVertex3fv  ( &vertexArray[   triangleArray[ i ].vinds[j] ].pos[0] );

             }
        }
    glEnd();
    glPopMatrix();

    return true;
  }

//______________________________________________________________________ Destructor function

/*
** Delete all dynamic arrays
*/

objFile::~objFile(void)
  {
     if( vertexArray != NULL )
       {
          delete [] vertexArray;
          delete [] normalArray;
          delete [] texCoordArray;

          delete [] triangleArray;
       }
  };
