//
//  TITLE : nurbs water pool v0.1 
//
//  AUTHOR : Robert Bateman
//
//  DATE : 1 Jan 2001
//
//  NOTES : Deformation of nurbs & polygon planes to get funky water pool
//          effect
//


//______________________________________________________________ Includes

#include <iostream.h>
#include <fstream.h>
#include <malloc.h>
#include <string.h>

#include "./opengl_includes.h"

#include "./fruity_exception.h"
#include "./fruity_timer.h"
#include "./fruity_font.h" 

int iModifier;

#include "./fruity_mayamouse.h"

#include "./fruity_nurbs.h"
#include "./objloader.h"
#include "./tga.h"
#include "./scene_lighting.h"


//______________________________________________________________ Globals

double fProcessingTime = 0 , fDisplayTime = 0;
float fFrametime = 0;
long  frames = 0;
char pzStatusLine[128];
GLuint Persp , Ortho;

cScene Lights;

GLuint env   ,
       stone ,
       wood  ;

objFile polyPlane , scenery;
cNurbsSurface nurbsPlane ;


//______________________________________________________________ Function Declarations

void init( void )
  {
    InitFrameCounter();
    makeRasterFont();

    glClearColor( 0.0 , 0.0 , 0.0 , 0.0 );
    glShadeModel( GL_SMOOTH );

    float lmodel_ambient[] = {0.3f, 0.3f, 0.3f, 1.0};
    float local_view[] = {1.0f};

    /* Set up the background ambient lighting in the scene... */

    glLightModelfv(GL_LIGHT_MODEL_AMBIENT, lmodel_ambient);
    glLightModelfv(GL_LIGHT_MODEL_LOCAL_VIEWER, local_view);


    glTexEnvf (GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE , GL_MODULATE);
    glEnable(GL_TEXTURE_2D);

    glEnable( GL_DEPTH_TEST );
    glDepthFunc( GL_LEQUAL );

    Lights.loadScene( "./data/water_info.txt" );

    loadTexture( "./data/scene_env.tga" , &env   );
    loadTexture( "./data/stone.tga"     , &stone );
    loadTexture( "./data/ceiling.tga"   , &wood  );

    nurbsPlane.load( "./data/water_plane.nurb" );
    scenery.load   ( "./data/water_floor.obj"  );
    polyPlane.load ( "./data/water_plane.obj"  );

  }

void idle( void )
  {
    frames++;
    nurbsPlane.deform( fFrametime );
    polyPlane.deform ( fFrametime );
    fProcessingTime += GetFrameTime();
    fFrametime = divide_d( (fProcessingTime + fDisplayTime) , frames );
    glutPostRedisplay();
  }

void display( void )
  {
    glClear( GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT );

    glCallList( Persp );


        float diffuse[] = { 0.5 , 0.5 , 0.5 , 0.5 } ,
              ambient[] = { 0.2 , 0.2 , 0.2 , 0.2 } ,
              specular[] = { 0.8 , 0.8 , 0.8 , 1.0 },
              zero[]     = { 0.0 , 0.0 , 0.0, 0.0 } ;


    glPushMatrix();
      Mousey.doTransform();
      glPushMatrix();


        glEnable( GL_AUTO_NORMAL );
        glEnable( GL_NORMALIZE );

        glBindTexture( GL_TEXTURE_2D , wood );

        glMaterialfv( GL_FRONT_AND_BACK , GL_DIFFUSE , diffuse );
        glMaterialfv( GL_FRONT_AND_BACK , GL_AMBIENT , ambient );
        glMaterialfv( GL_FRONT_AND_BACK , GL_SPECULAR , specular );
        glMaterialf( GL_FRONT_AND_BACK , GL_SHININESS , 10 );

        scenery.draw();


        glPushMatrix();
          glMaterialfv( GL_FRONT_AND_BACK , GL_SPECULAR , zero );
          glBindTexture( GL_TEXTURE_2D , stone );
          glTranslatef( 0 , -0.5 , 0 );
          polyPlane.draw();
        glPopMatrix();

        glPushMatrix();
        glTexGeni(GL_S, GL_TEXTURE_GEN_MODE, GL_SPHERE_MAP );
        glTexGeni(GL_T, GL_TEXTURE_GEN_MODE, GL_SPHERE_MAP );
        glEnable(GL_TEXTURE_GEN_S);
        glEnable(GL_TEXTURE_GEN_T);
        glBindTexture( GL_TEXTURE_2D , env );
        glBlendFunc( GL_ONE , GL_ONE );
        glEnable( GL_BLEND );

        glPushMatrix();
          glTranslatef( 0 , -0.5 , 0 );
          polyPlane.draw();
        glPopMatrix();

        nurbsPlane.draw();

        glDisable( GL_BLEND );
        glDisable( GL_TEXTURE_GEN_S );
        glDisable( GL_TEXTURE_GEN_T );
        glDisable( GL_AUTO_NORMAL );
        glDisable( GL_NORMALIZE );
        glPopMatrix();

      glPopMatrix();
    glPopMatrix();

    glCallList( Ortho );


    // baseline display time - 0.034 secs display, 29fps - glutSolidTeapot(10);

    sprintf( pzStatusLine , "Processing : %f secs" , divide_d(fProcessingTime,frames) );
    font( pzStatusLine , 5 , 12 , 1, 1, 0.4 );
    sprintf( pzStatusLine , "Display Time : %f secs" , divide_d(fDisplayTime,frames) );
    font( pzStatusLine , 5 , 22 , 1, 1, 0.4 );
    sprintf( pzStatusLine , "Frames Per Sec: %f" , divide_d(1,fFrametime) );
    font( pzStatusLine , 5 , 32 , 1, 1, 1 );

    glutSwapBuffers();

    fDisplayTime += GetFrameTime();
  }

void reshape( int w , int h )
  {
   
    glDeleteLists(Persp,1);
    glDeleteLists(Ortho,1);
    Persp = glGenLists(1);
    Ortho = glGenLists(1);

    glNewList(Persp,GL_COMPILE);
      glMatrixMode( GL_TEXTURE );
      glLoadIdentity();
      glViewport(0, (GLsizei) 0, (GLsizei) w, (GLsizei) h);
      glMatrixMode(GL_PROJECTION);
      glLoadIdentity();
      gluPerspective(35.0, divide_f( w , h ) , 0.01, 500 );
      glMatrixMode(GL_MODELVIEW);
      glLoadIdentity();
      glEnable( GL_TEXTURE_2D );
      glEnable( GL_LIGHTING );
    glEndList();

    glNewList(Ortho,GL_COMPILE);
      glViewport(0, (GLsizei) 0, (GLsizei) w, (GLsizei) h);
      glMatrixMode(GL_PROJECTION);
      glLoadIdentity();
      glOrtho(0, w, h, 0, -1, 1);
      glMatrixMode(GL_MODELVIEW);
      glLoadIdentity();
      glDisable( GL_TEXTURE_2D );
      glDisable( GL_LIGHTING );
    glEndList();

    glCallList(Persp);
  }

void Mouse_Motion(int x, int y)
  {
    Mousey.motion( x , y );
    glutPostRedisplay();
  }

void Mouse_Button( int button , int state , int x , int y )
  {
    Mousey.buttonPress( button , state , x , y );
    glutPostRedisplay();
  }

int main( int argc , char **argv )
  {
     glutInit(&argc, argv);
     glutInitDisplayMode   ( GLUT_DOUBLE |
                             GLUT_RGBA   |
                             GLUT_DEPTH   );
     glutInitWindowSize    ( 800 , 600 );
     glutInitWindowPosition( 100 , 100 );
     glutCreateWindow      ( "Nurbs Water Pool" );
     

     glutDisplayFunc( display );
     glutReshapeFunc( reshape );

     glutMouseFunc  ( Mouse_Button );
     glutMotionFunc ( Mouse_Motion );
     glutIdleFunc   ( idle );

     init();

     glutMainLoop();
     return 0;
  }


