#include "./fruity_nurbs.h"
//#include <math.h>

void CALLBACK nurbsErrorCallback( void )

  //
  // callback function to deal with nurbs error's
  //

  {
    cout << "nurbs error "  << endl;
    exit( 0 );
  }


//______________________________________________________________________ Deform function

bool cNurbsSurface::deform( float timeInc  )

  //
  // simple deformation function that deforms the surface by a sine wave
  //

  {
    uint i,j;
    Time += 4*timeInc;
    if( pCVs )
      {
        for( i=0; i< uiNumOfCvsInU ; i++ )
          {
            for( j=0; j< uiNumOfCvsInV ; j++ )
              {
                pCVs[ (i*uiNumOfCvsInV*3) + (j*3) + 1 ] 
                  = pCVs_undeformed[ (i*uiNumOfCvsInV*3) + (j*3) + 1] 
                    +  0.25*sin( pCVs_undeformed[ (i*uiNumOfCvsInV*3) + (j*3) ] 
                               + pCVs_undeformed[ (i*uiNumOfCvsInV*3) + (j*3) +2 ] + Time ) ;
              }
          }
        return true;
      }
    return false;
  }

uint cNurbsSurface::getCvCountU ( void )
  {
    return uiNumOfCvsInU;
  }

uint cNurbsSurface::getCvCountV ( void )
  {
    return uiNumOfCvsInV;
  }

uint cNurbsSurface::getKnotCountU ( void )
  {
    return uiNumOfKnotsInU;
  }

uint cNurbsSurface::getKnotCountV ( void )
  {
    return uiNumOfKnotsInV;
  }


float cNurbsSurface::getKnotU ( uint num )
  {
    if( num < uiNumOfKnotsInU )
      {
        return pKnotsU[ num ];
      }
    return 0;
  }

float cNurbsSurface::getKnotV ( uint num )
  {
    if( num < uiNumOfKnotsInV )
      {
        return pKnotsV[ num ];
      }
    return 0;
  }

bool cNurbsSurface::setKnotU ( uint num , float val )
  {
    if( num < uiNumOfKnotsInU )
      {
        pKnotsU[ num ] = val;
        return true;
      }
    return false;
  }

bool cNurbsSurface::setKnotV ( uint num , float val )
  {
    if( num < uiNumOfKnotsInV )
      {
        pKnotsV[ num ] = val;
        return true;
      }
    return false;
  }

float *cNurbsSurface::getCV( uint uiU ,uint uiV )
  {
    if( uiU < uiNumOfCvsInU &&
        uiV < uiNumOfCvsInV  )
      {
        float val[3];
        val[0] = pCVs[ (uiU*uiNumOfCvsInV*3) + (uiV*3)     ];
        val[1] = pCVs[ (uiU*uiNumOfCvsInV*3) + (uiV*3) + 1 ];
        val[2] = pCVs[ (uiU*uiNumOfCvsInV*3) + (uiV*3) + 2 ];

        return &val[0];
      }

    return NULL;
  }

bool cNurbsSurface::setCV ( uint uiU , uint uiV , float x , float y , float z )
  {
    if( uiU < uiNumOfCvsInU &&
        uiV < uiNumOfCvsInV  )
      {

        pCVs[ (uiU*uiNumOfCvsInV*3) + (uiV*3)     ] = x;
        pCVs[ (uiU*uiNumOfCvsInV*3) + (uiV*3) + 1 ] = y;
        pCVs[ (uiU*uiNumOfCvsInV*3) + (uiV*3) + 2 ] = z;

        return true;
      }

    return false;
  }

bool cNurbsSurface::moveCV ( uint uiU , uint uiV , float x , float y , float z )
  {
    if( uiU < uiNumOfCvsInU &&
        uiV < uiNumOfCvsInV  )
      {

        pCVs[ (uiU*uiNumOfCvsInV*3) + (uiV*3)     ] += x;
        pCVs[ (uiU*uiNumOfCvsInV*3) + (uiV*3) + 1 ] += y;
        pCVs[ (uiU*uiNumOfCvsInV*3) + (uiV*3) + 2 ] += z;

        return true;
      }

    return false;
  }

bool cNurbsSurface::setSamplingMethod ( GLenum method )

  //
  //  Changes nurbs sampling methods to only the ones that are valid
  //


  {
    switch ( method )
      {
        case GLU_PATH_LENGTH :
         
          gluNurbsProperty( oglSurface , GLU_SAMPLING_METHOD      , GLU_PATH_LENGTH      );
          gluNurbsProperty( oglSurface , GLU_SAMPLING_TOLERANCE   , fSamplingTolerance   );

        break;

        case GLU_PARAMETRIC_ERROR :
         
          gluNurbsProperty( oglSurface , GLU_SAMPLING_METHOD      , GLU_PARAMETRIC_ERROR );
          gluNurbsProperty( oglSurface , GLU_PARAMETRIC_TOLERANCE , fPrmetricTolerance   );

        break;

        case GLU_DOMAIN_DISTANCE :
         
          gluNurbsProperty( oglSurface , GLU_SAMPLING_METHOD      , GLU_DOMAIN_DISTANCE  );
          gluNurbsProperty( oglSurface , GLU_U_STEP 	          , fU_Step              );
          gluNurbsProperty( oglSurface , GLU_V_STEP 	          , fV_Step              );

        break;

        default :
        break;

      }

    SamplingMethod = method;

    return true;

  }

bool cNurbsSurface::set_Ustep             ( float val )

  // sets U step parameter for DOMAIN based tessalation.

  {
    if( val > 0 )
      {
        fU_Step = val;

        if( SamplingMethod == GLU_DOMAIN_DISTANCE
            && oglSurface )
          {
            gluNurbsProperty( oglSurface , GLU_U_STEP , fU_Step );
          }

        return true;
      }
    return false;
  }

bool cNurbsSurface::set_Vstep             ( float val )

  // sets V step parameter for DOMAIN based tessalation.

  {
    if( val > 0 )
      {
        fV_Step = val;

        if( SamplingMethod == GLU_DOMAIN_DISTANCE
            && oglSurface )
          {
            gluNurbsProperty( oglSurface , GLU_V_STEP , fV_Step );
          }

        return true;
      }
    return false;
  }
 
bool cNurbsSurface::setSamplingTolerance  ( float val )
  {
    if( val > 0 )
      {
        fSamplingTolerance = val;
        if( SamplingMethod == GLU_PATH_LENGTH 
            && oglSurface )
          {
            gluNurbsProperty( oglSurface , GLU_SAMPLING_TOLERANCE , fSamplingTolerance );
          }

        return true;
      }
    return false;
  }

bool cNurbsSurface::setParametricTolerance( float val )
  {
    if( val > 0 )
      {
        fPrmetricTolerance = val;
        if( SamplingMethod == GLU_PATH_LENGTH 
            && oglSurface )
          {
            gluNurbsProperty( oglSurface , GLU_PARAMETRIC_TOLERANCE , fPrmetricTolerance );
          }

        return true;
      }
    return false;
  }

void cNurbsSurface::drawStyle ( void )

  {

    if( bWireframe )
      {
        gluNurbsProperty( oglSurface , GLU_DISPLAY_MODE , GLU_FILL );
      }
    else
      {
        gluNurbsProperty( oglSurface , GLU_DISPLAY_MODE , GLU_OUTLINE_POLYGON );
      }

    bWireframe = !bWireframe;

  }


//_______________________________________________ Surface Drawing Function ________

bool cNurbsSurface::draw( void )

  //
  //
  //    This runs through the array of quads and draws each one.
  //  Returns false if no data is loaded.
  //
  //

  {

    float diffuse[] = { 0.5 , 0.5 , 0.5 , 0.5 } ,
          ambient[] = { 0.1 , 0.1 , 0.1 , 0.2 } ,
          specular[] = { 1.0 , 1.0 , 1.0 , 1.0 } ;

    /*
    **  ---====####  Draw Each Quad In Array  ####====---
    */

    glPushMatrix();

      glMaterialfv( GL_FRONT_AND_BACK , GL_DIFFUSE , diffuse );
      glMaterialfv( GL_FRONT_AND_BACK , GL_AMBIENT , ambient );
      glMaterialfv( GL_FRONT_AND_BACK , GL_SPECULAR , specular );
      glMaterialf( GL_FRONT_AND_BACK , GL_SHININESS , 15 );

      gluBeginSurface( oglSurface );
        gluNurbsSurface( oglSurface , 
                         uiNumOfKnotsInU , pKnotsU ,
                         uiNumOfKnotsInV , pKnotsV ,
                         uiNumOfCvsInV * 3 , 3 , &pCVs[0] , 
                         uiDegreeInU+1 , uiDegreeInV+1 , GL_MAP2_VERTEX_3 );
      gluEndSurface( oglSurface );
  
    glPopMatrix();

    return true;

  }



//________________________________________________ CV's Drawing Function ________

bool cNurbsSurface::drawCVs( void )


  //
  // A small function to draw the CV's - used in debugging & testing
  //


  {

    if( !pCVs )
      {
        return false;
      }

    float colour[4] = { 0.6 , 0.4 , 0.4 , 0.0 };

    for( uint i=0; i<uiNumOfCvsInU; i++ )
      {
        for( uint j=0; j<uiNumOfCvsInV; j++ )
          {
            glPushMatrix();
              glMaterialfv( GL_FRONT_AND_BACK , GL_AMBIENT_AND_DIFFUSE , colour );
              glTranslatef(   pCVs[ (i*uiNumOfCvsInV*3) + (j*3)     ] , 
                              pCVs[ (i*uiNumOfCvsInV*3) + (j*3) + 1 ] ,
                              pCVs[ (i*uiNumOfCvsInV*3) + (j*3) + 2 ] );
              
              glutSolidCube( 0.2 );
            glPopMatrix();
          }


      }

    return true;

  }


//________________________________________________ CV's Drawing Function ________

bool cNurbsSurface::drawHulls( void )


  //
  // A small function to draw the CV's - used in debugging & testing
  //


  {
    uint i , j;

    if( !pCVs )
      {
        return false;
      }

    glDisable( GL_LIGHTING );


    for( i=0; i<uiNumOfCvsInU; i++ )
      {
        glPushMatrix();
  
          glColor3f( 0.2 , 0.3 , 1.0 );

          glBegin( GL_LINE_STRIP );

            for( j=0; j<uiNumOfCvsInV; j++ )
              {
                  glVertex3f(   pCVs[ (i*uiNumOfCvsInV*3) + (j*3)     ] , 
                                pCVs[ (i*uiNumOfCvsInV*3) + (j*3) + 1 ] ,
                                pCVs[ (i*uiNumOfCvsInV*3) + (j*3) + 2 ] );
              }

          glEnd();

        glPopMatrix();
      }

    for( i=0; i<uiNumOfCvsInV; i++ )
      {
        glPushMatrix();
  
          glColor3f( 0.2 , 0.3 , 1.0 );

          glBegin( GL_LINE_STRIP );

            for( j=0; j<uiNumOfCvsInU; j++ )
              {
                  glVertex3f(   pCVs[ (j*uiNumOfCvsInV*3) + (i*3)     ] , 
                                pCVs[ (j*uiNumOfCvsInV*3) + (i*3) + 1 ] ,
                                pCVs[ (j*uiNumOfCvsInV*3) + (i*3) + 2 ] );
              }

          glEnd();

        glPopMatrix();
      }


    glEnable( GL_LIGHTING );

    return true;

  }


//_______________________________________________ Delete Surface Function ________

bool cNurbsSurface::deleteData ( void )

  //
  //
  //     This function deletes all contained surface information
  //  So that an additional surface can be loaded.
  //
  //

  {

    /*
    **  ---====#### If No Cv's Are Stored return false  ####====---
    */

    if( !pCVs )
      {
        return false;
      }

    /*
    **  ---====#### Delete Dynamically Allocated Data ####====---
    */

    delete [] pKnotsU;
    delete [] pKnotsV;
    delete [] pCVs   ;


    return true;


  };

//___________________________________________________ Constructor Function ________


cNurbsSurface::cNurbsSurface( void ) : geometry()

  {
    /*
    **  ---====####  Set Pointers To NULL  ####====---
    */
 
    pKnotsU            = NULL;
    pKnotsV            = NULL;
    pCVs               = NULL;

    /*
    **  ---====####  Set Default Data Values  ####====---
    */

    uiTypeU            = 0;
    uiTypeV            = 0;

    bWireframe         = true ;

    SamplingMethod     = GLU_PATH_LENGTH;

    fU_Step            = 4.0;
    fV_Step            = 4.0;

    fSamplingTolerance = 25.0;
    fPrmetricTolerance = 100.0;
          
    oglSurface         = NULL;
    Time               = 0;  
 

  };



//____________________________________________________ Destructor Function ________


cNurbsSurface::~cNurbsSurface( void )

  {
    /*
    **  ---====####  Delete dynamically allocated data  ####====---
    */
 
    if( pKnotsU ) delete [] pKnotsU ;
    if( pKnotsV ) delete [] pKnotsV ;
    if( pCVs    ) delete [] pCVs    ;

  };



//_______________________________________________ Load Nurbs Data Function ________


bool cNurbsSurface::load( char *filename )

  //
  // 
  //     This function loads up data files exported 
  //  from Maya using nurbDataExport.mel
  //
  //

  {

    /*
    **  ---====####  Runs some check to see if parameters are valid  ####====---
    */


    if( !filename )
      {
        return false;
      }

    if( pCVs )
      {
        cout << "data loaded, request to load surface ignored" << endl;
        return false;
      }


    /*
    **  ---====####  Create an Input File Stream  ####====---
    */


    ifstream  Input( filename , ios::in );


    /*
    **  ---====####  Check to see if file opened Ok  ####====---
    */


    if( !Input )
      {
        cout << "failed to load file : " << filename << endl;
        return false;
      }
    else
      {
        uint i,j,k;
        char buffer[256];

        /*
        **  ---====####  Check File Header To See If File is Valid  ####====---
        */

        Input >> buffer;

        if( strcmp( buffer , "NDF1.0" ) == 0 )
          {

            /*
            **   ---====####  Get Surface Name ####====---
            */

            Input.getline( buffer , 254 , '\n' );
            Input >> buffer
                  >> buffer ;

            cout  << "loading surface " << buffer << endl;


            /*
            **  ---====####  Get Surface Description & Count Values  ####====---
            */

            Input >> buffer
                  >> uiTypeU
                  >> buffer
                  >> uiTypeV
                  >> buffer
                  >> uiDegreeInU 
                  >> buffer
                  >> uiDegreeInV
                  >> buffer
                  >> uiNumOfSpansInU
                  >> buffer
                  >> uiNumOfSpansInV;


            /*
            **   ---====####  Get Surface Knots For U  ####====---
            */

            Input >> buffer
                  >> uiNumOfKnotsInU;

            pKnotsU = new float[ uiNumOfKnotsInU ];

            CheckMemory( pKnotsU );


            for( i=0; i<uiNumOfKnotsInU; i++ )
              {
                Input >> pKnotsU[ i ];
              }

            /*
            **   ---====####  Get Surface Knots For V  ####====---
            */

            Input >> buffer
                  >> uiNumOfKnotsInV;

            pKnotsV = new float[ uiNumOfKnotsInV ];

            CheckMemory( pKnotsV );


            for( i=0; i<uiNumOfKnotsInV; i++ )
              {
                Input >> pKnotsV[ i ];
              }


            /*
            **   ---====####  Get Surface CV Counts & allocate Memory  ####====---
            */

            Input >> buffer
                  >> uiNumOfCvsInU
                  >> buffer
                  >> uiNumOfCvsInV
                  >> buffer
                  >> uiSurfaceDimension
                  >> buffer;

             pCVs            = new float[ (uiNumOfCvsInU * uiNumOfCvsInV * 3) ];
             pCVs_undeformed = new float[ (uiNumOfCvsInU * uiNumOfCvsInV * 3) ];
             CheckMemory( pCVs );

             for( i=0; i< uiNumOfCvsInU; i++ )
               {
                 for( j=0; j< uiNumOfCvsInV; j++ )
                   {
                     Input >> buffer 
                           >> buffer
                           >> buffer
                           >> pCVs[ (i*uiNumOfCvsInV*3) + (j*3)     ]  
                           >> pCVs[ (i*uiNumOfCvsInV*3) + (j*3) + 1 ] 
                           >> pCVs[ (i*uiNumOfCvsInV*3) + (j*3) + 2 ] ;
                     pCVs_undeformed[ (i*uiNumOfCvsInV*3) + (j*3)     ] = pCVs[ (i*uiNumOfCvsInV*3) + (j*3)     ];
                     pCVs_undeformed[ (i*uiNumOfCvsInV*3) + (j*3) + 1 ] = pCVs[ (i*uiNumOfCvsInV*3) + (j*3) + 1 ];
                     pCVs_undeformed[ (i*uiNumOfCvsInV*3) + (j*3) + 2 ] = pCVs[ (i*uiNumOfCvsInV*3) + (j*3) + 2 ];
                   }
               } 

          }

        else
          {

            cout << "File Not Valid For This Loader\n" << endl;
            return false;

          }

      } 

    oglSurface = gluNewNurbsRenderer();

    gluNurbsProperty( oglSurface , GLU_SAMPLING_TOLERANCE , 25.0 );
    gluNurbsProperty( oglSurface , GLU_DISPLAY_MODE       , GLU_FILL );
    gluNurbsCallback( oglSurface , GLU_ERROR              , nurbsErrorCallback );
    gluNurbsProperty( oglSurface , GLU_CULLING            , GL_TRUE );

    return true;
  }

