//_____________________________________________________________________________________________fruity_nurbs.h
//
//   Title  :    Fruity_Nurbs header
//   Author :    Rob Bateman
//   Date   :    28/Jan/01
//
//   Notes  :  added deformation function for water type effect.
//



#ifndef FRUITY_NURBS
#define FRUITY_NURBS


//_________ ---====####  includes  ####====---  _____________________________________________


#include "./geometry_base.h"
#include "./fruity_exception.h"


//_____ ---====####  nurbs curve types  ####====---  ________________________________________



#define OPEN 0
#define CLOSED 1
#define PERIODIC 2


//______ ---====####  cNurbsSurface class  ####====--- _______________________________________


class cNurbsSurface : public geometry

  //
  //
  //    This class holds all data required to hold and display a Nubrs
  //  Surface. It holds The final Tessalated Surface As well as all control
  //  point & knots etc.
  //
  //

  {

    public :

      /*
      **  ---====####  Constructor & Destructor Functions  ####====---
      */

      cNurbsSurface ( void );
      ~cNurbsSurface( void );


      /*
      **  ---====####  Public Member Functions  ####====---
      */

      bool load                  ( char *filename );  // Loads Up Nurbs Data
      bool draw                  ( void );            // Draws Data
      bool deform                ( float timeInc  );  // Deforms surface using sine curves


      bool drawCVs               ( void );            // Draws C.V. Positions only
      bool drawHulls             ( void );            // Draws Nurbs Hulls


      void drawStyle             ( void );

      bool setSamplingMethod     ( GLenum method );

      bool set_Ustep             ( float val );
      bool set_Vstep             ( float val );
 
      bool setSamplingTolerance  ( float val );
      bool setParametricTolerance( float val );

      bool deleteData            ( void );            // Deletes Current Surface


      


      float *getCV               ( uint   uiU ,
                                   uint   uiV );
      bool   setCV               ( uint   uiU ,
                                   uint   uiV ,
                                   float  x   ,
                                   float  y   ,
                                   float  z   );
      bool   moveCV              ( uint   uiU ,
                                   uint   uiV ,
                                   float  x   ,
                                   float  y   ,
                                   float  z   );

      uint   getCvCountU         ( void       );
      uint   getCvCountV         ( void       );

      uint   getKnotCountU       ( void       );
      uint   getKnotCountV       ( void       );

      float  getKnotU            ( uint   num );
      float  getKnotV            ( uint   num );

      bool   setKnotU            ( uint   num ,
                                   float  val );
      bool   setKnotV            ( uint   num ,
                                   float  val );

    private :

      /*
      **  ---====####  Private Data Members  ####====---
      */



      uint         uiTypeU            ,    // Type Of Nurbs Curve :
                   uiTypeV            ,    //      can be 'OPEN'(0), 'CLOSED'(1), 'PERIODIC'(2).


                                           // !!! MENTAL NOTE !!!   - NEED EXTRA PROCESSING FOR PERIODIC AND OPEN CURVES


                   uiNumOfCvsInU      ,    // Number of Cv's in U direction
                   uiNumOfCvsInV      ,    // Nurber of Cv's in V direction
  
                   uiDegreeInU        ,    // Degree of the curve in the U direction
                   uiDegreeInV        ,    // Degree of the curve in the V direction

                   uiNumOfSpansInU    ,    // Number Of Surface Spans In U Direction
                   uiNumOfSpansInV    ,    // Number Of Surface Spans In V Direction

                   uiNumOfKnotsInU    ,    // Number Of Surface Knots In U Direction
                   uiNumOfKnotsInV    ,    // Number Of Surface Knots In V Direction

                   uiSurfaceDimension  ;    

      /*
      **  ---====####  Nurb Surfae Data Arrays  ####====---
      */

      float       *pKnotsU            ,    // Array Of Knot Values for U direction
                  *pKnotsV            ,    // Array Of Knot Values for V direction
                  *pCVs               ,    // 3D Array Of Control Verticies
                  *pCVs_undeformed    ,    // 3D Array Of Undeformed CV's

      /*
      **  ---====####  Data Required for OpenGL nurbs implimentation  ####====---
      */
 
      //
      //  !!! MENTAL NOTE !!! : NONE OF THIS IS ACTUALLY NEEDED BUT ALLOWS FOR 
      //                        A LOT OF FLEXIBILITY IN ITS USAGE.
      //

                   fU_Step            ,    // used for GLU_DOMAIN_DISTANCE tessalation
                   fV_Step            ,    // used for GLU_DOMAIN_DISTANCE tessalation

                   fSamplingTolerance ,    // used for GLU_PATH_LENGTH sampling tolerance
                   fPrmetricTolerance  ;   // used for GLU_PARAMETRIC_ERROR sampling
          
      GLUnurbsObj *oglSurface          ;   // Pointer to openGL nurbs object


      bool         bWireframe          ;   // boolean variable for drawing style (filled/wireframe)

      GLenum       SamplingMethod      ;   // tells openGL what tessalation type to use
      float        Time;
  };


#endif   //______________________________________________________________________________________End Of fruity_nurbs.h


