/*                         Compile With:   make                               */

/*##############################################################################
###                              INCLUDES
##############################################################################*/

#define WIN32
#include <GL/glut.h>
#include <stdio.h>
#include <stdlib.h>
#include "tga.h"
#include "fruity_glext.h"

/*##############################################################################
###                        STRUCTURE DEFINITIONS
##############################################################################*/

struct cam
{
  float xrot;
  float yrot;
  GLboolean LMB;
  int LastPosition[2];
} CAM;

/*##############################################################################
###                              GLOBALS
##############################################################################*/

GLuint Reflection;

/*##############################################################################
###                         FUNCTION PROTOTYPES
##############################################################################*/

void init(void); 
void reshape(int,int); 
void display(void);
void Button(int,int,int,int);
void Motion(int,int);

/*##############################################################################
###                            INIT FUNCTION
##############################################################################*/

void init(void)
{
  GLfloat lmodel_ambient[] = { 0.3f, 0.3f, 0.3f, 1.0 };

  glClearColor(0.0, 0.0, 0.0, 0.0);
  glShadeModel(GL_SMOOTH);
  glEnable(GL_DEPTH_TEST);

  /* SET UP LIGHTING */
  { 
    GLfloat ambient[] = { 0.1f, 0.1f, 0.1f, 1.0f };
    GLfloat diffuse[] = { 0.4f, 0.4f, 0.4f, 1.0f };
    GLfloat specular[] ={ 1.0f, 1.0f, 1.0f, 1.0f };
    GLfloat position[] ={ 0.5f, 0.1f, 1.0f, 0.0f };
      
    glLightfv(GL_LIGHT0, GL_AMBIENT, ambient);
    glLightfv(GL_LIGHT0, GL_DIFFUSE, diffuse);
    glLightfv(GL_LIGHT0, GL_SPECULAR, specular);
    glLightfv(GL_LIGHT0, GL_POSITION, position);
  }

  glLightModelfv(GL_LIGHT_MODEL_AMBIENT, lmodel_ambient);
  glLightModelf(GL_LIGHT_MODEL_LOCAL_VIEWER, GL_FALSE);

  glEnable(GL_LIGHTING);
  glEnable(GL_LIGHT0);

  /* CHECK FOR OPENGL 1.2 */
  if(CheckGLExt("GL_VERSION_1_2")) 
    glLightModeli(GL_LIGHT_MODEL_COLOR_CONTROL_EXT,GL_SEPARATE_SPECULAR_COLOR_EXT);

  /* INIT CAM STRUCTURE */
  CAM.xrot = 0;
  CAM.yrot = 0;
  CAM.LMB = GL_FALSE;


  /* LOAD TEXTURE AND SET UP TEXTURING */
  glGenTextures(1,&Reflection);
  loadTGA( "Reflect.tga", Reflection );
  glTexEnvf (GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE , GL_MODULATE);
  glEnable(GL_TEXTURE_2D);


  /* SET UP TEXTURE COORD GENERATION */
  glTexGeni(GL_S, GL_TEXTURE_GEN_MODE, GL_SPHERE_MAP);
  glTexGeni(GL_T, GL_TEXTURE_GEN_MODE, GL_SPHERE_MAP);
  glEnable(GL_TEXTURE_GEN_S);
  glEnable(GL_TEXTURE_GEN_T);

  /* ENABLE BACK FACE CULLING */
  glCullFace(GL_FRONT);
  glEnable(GL_CULL_FACE);

}

/*------------------------------------------------------------------------------

if(CheckGLExt("GL_VERSION_1_2")) 
  glLightModeli(GL_LIGHT_MODEL_COLOR_CONTROL_EXT,GL_SEPARATE_SPECULAR_COLOR_EXT);

The Above Statement checks to see if openGL 1.2 is available at runtime. If it
is it forces openGL to calculate the specular highlight after texturing. This 
gives a much more realistic look to the reflections and highlights. Quite often
calculating the specular colour before texturing will lead to a much duller, more
muted highlight.

The chance of your grahics card actually supporting this is slim, I'd say you'd
have a chance if you have an NVIDIA card or an ATI RADEON, but otherwise its 
unlikely.

The jpeg 'ScreenShot' shows what it looks like with seperate specular on the 
left, and without on the right.

------------------------------------------------------------------------------*/

/*##############################################################################
###                           MOUSE BUTTON FUNCTION
##############################################################################*/

void Button(int button, int state, int x, int y) 
{
  if(state == GLUT_DOWN)
  {
    switch(button)
    {
      case GLUT_LEFT_BUTTON:
        CAM.LMB = GL_TRUE;
        CAM.LastPosition[0] = x;
        CAM.LastPosition[1] = y;
      break;

      default: 
      break;
    }
  }
  else
  {
    switch(button)
    {
      case GLUT_LEFT_BUTTON:
        CAM.LMB = GL_FALSE;
      break;

      default: 
      break;
    }
  }
  glutPostRedisplay();
}

/*##############################################################################
###                           MOUSE MOTION FUNCTION
##############################################################################*/

void Motion(int x, int y)
{
  int stepX = (x - CAM.LastPosition[0]);
  int stepY = (y - CAM.LastPosition[1]);

  CAM.LastPosition[0] = x; CAM.LastPosition[1] = y;

  if(CAM.LMB) {
    CAM.xrot += stepX;
    CAM.yrot += stepY;
  }

  glutPostRedisplay();
}

/*##############################################################################
###                            RESHAPE FUNCTION
##############################################################################*/

void reshape(int w, int h)
{
   if(h == 0) h = 1;
   glViewport(0, 0, (GLsizei) w, (GLsizei) h);
   glMatrixMode(GL_PROJECTION);
   glLoadIdentity();
   gluPerspective(45.0, (GLfloat) w/(GLfloat) h, 0.1, 100);
   glMatrixMode(GL_MODELVIEW);
   glLoadIdentity();
}


/*##############################################################################
###                            DISPLAY FUNCTION
##############################################################################*/


void display(void)
{
  GLfloat specular[4] = {1,1,1,1};
  GLfloat diffuse[4] = {0.6,0.6,0.6,1};

  glClear( GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT );
  glLoadIdentity();
  gluLookAt( 0 , 0 , 2.5 , 0 , 0 , 0 , 0 , 1, 0 );
  glRotatef(CAM.xrot,0,1,0);
  glRotatef(CAM.yrot,1,0,0);

  glBindTexture(GL_TEXTURE_2D,Reflection);

  glPushMatrix();
   glMaterialfv(GL_FRONT,GL_SPECULAR,specular);
   glMaterialfv(GL_FRONT,GL_DIFFUSE,diffuse);
   glMaterialf(GL_FRONT,GL_SHININESS,25);

   glutSolidTeapot(0.5); 
  glPopMatrix();

  glutSwapBuffers();
}

/*##############################################################################
###                              MAIN FUNCTION
##############################################################################*/

int main(int argc, char** argv)
{
   glutInit(&argc, argv);
   glutInitDisplayMode(GLUT_DOUBLE | GLUT_DEPTH | GLUT_RGB );
   glutInitWindowSize( 500 , 500 );
   glutInitWindowPosition(100, 100);
   glutCreateWindow("Environment Mapping         LMB - Rotate");
   init();
   glutReshapeFunc(reshape);
   glutDisplayFunc(display);
   glutMouseFunc(Button);
   glutMotionFunc(Motion);
   glutMainLoop();
   return 0;
}