/*#####################################################################################
 *
 *   Title        : fruity_glext.h
 *   Author       : Robert Bateman
 *   Date         : 15-Sep-2000
 *   Last Updated : 30-Nov-2000
 *
 *   Explanation  :
 *
 *     This header and source file check for the existence of a given openGL extension.
 *   This extension will then have all its associated functions set up for use.
 *
 *   Call :
 *
 *                        'CheckGLExt( char *extString )'
 *
 *   with the name of the openGL extension you wish to use. It will return true if the
 *   extension is supported by the hardware, and false if it is unavailable.
 *
 *
 *           *** DO NOT USE THE EXTENSION IF IT IS NOT SUPPORTED!! ***
 *
 *
#####################################################################################*/


#ifndef GLEXT_SUPPORT_H
#define GLEXT_SUPPORT_H

#define WIN32
#include <GL/glut.h>
#include "./glext.h"

extern GLboolean CheckGLExt( char * );

//#####################################################################################
//                                   OPENGL 1.2
//#####################################################################################

#ifdef GL_VERSION_1_2
  extern PFNGLBLENDCOLORPROC                glBlendColor                ;
  extern PFNGLBLENDEQUATIONPROC             glBlendEquation             ;
  extern PFNGLDRAWRANGEELEMENTSPROC         glDrawRangeElements         ;
  extern PFNGLCOLORTABLEPROC                glColorTable                ;
  extern PFNGLCOLORTABLEPARAMETERFVPROC     glColorTableParameterfv     ;
  extern PFNGLCOLORTABLEPARAMETERIVPROC     glColorTableParameteriv     ;
  extern PFNGLCOPYCOLORTABLEPROC            glCopyColorTable            ;
  extern PFNGLGETCOLORTABLEPROC             glGetColorTable             ;
  extern PFNGLGETCOLORTABLEPARAMETERFVPROC  glGetColorTableParameterfv  ;
  extern PFNGLGETCOLORTABLEPARAMETERIVPROC  glGetColorTableParameteriv  ;
  extern PFNGLCOLORSUBTABLEPROC             glColorSubTable             ;
  extern PFNGLCOPYCOLORSUBTABLEPROC         glCopyColorSubTable         ;
  extern PFNGLCONVOLUTIONFILTER1DPROC       glConvolutionFilter1D       ;
  extern PFNGLCONVOLUTIONFILTER2DPROC       glConvolutionFilter2D       ;
  extern PFNGLCONVOLUTIONPARAMETERFPROC     glConvolutionParameterf     ;
  extern PFNGLCONVOLUTIONPARAMETERFVPROC    glConvolutionParameterfv    ;
  extern PFNGLCONVOLUTIONPARAMETERIPROC     glConvolutionParameteri     ;
  extern PFNGLCONVOLUTIONPARAMETERIVPROC    glConvolutionParameteriv    ;
  extern PFNGLCOPYCONVOLUTIONFILTER1DPROC   glCopyConvolutionFilter1D   ;
  extern PFNGLCOPYCONVOLUTIONFILTER2DPROC   glCopyConvolutionFilter2D   ;
  extern PFNGLGETCONVOLUTIONFILTERPROC      glGetConvolutionFilter      ;
  extern PFNGLGETCONVOLUTIONPARAMETERFVPROC glGetConvolutionParameterfv ;
  extern PFNGLGETCONVOLUTIONPARAMETERIVPROC glGetConvolutionParameteriv ;
  extern PFNGLGETSEPARABLEFILTERPROC        glGetSeparableFilter        ;
  extern PFNGLSEPARABLEFILTER2DPROC         glSeparableFilter2D         ;
  extern PFNGLGETHISTOGRAMPROC              glGetHistogram              ;
  extern PFNGLGETHISTOGRAMPARAMETERFVPROC   glGetHistogramParameterfv   ;
  extern PFNGLGETHISTOGRAMPARAMETERIVPROC   glGetHistogramParameteriv   ;
  extern PFNGLGETMINMAXPROC                 glGetMinmax                 ;
  extern PFNGLGETMINMAXPARAMETERFVPROC      glGetMinmaxParameterfv      ; 
  extern PFNGLGETMINMAXPARAMETERIVPROC      glGetMinmaxParameteriv      ;
  extern PFNGLHISTOGRAMPROC                 glHistogram                 ;
  extern PFNGLMINMAXPROC                    glMinmax                    ;
  extern PFNGLRESETHISTOGRAMPROC            glResetHistogram            ; 
  extern PFNGLRESETMINMAXPROC               glResetMinmax               ;
  extern PFNGLTEXIMAGE3DPROC                glTexImage3D                ;
  extern PFNGLTEXSUBIMAGE3DPROC             glTexSubImage3D             ;
  extern PFNGLCOPYTEXSUBIMAGE3DPROC         glCopyTexSubImage3D         ;
#endif


//#####################################################################################
//                               ARB MULTITEXTURE
//#####################################################################################

#ifdef GL_ARB_multitexture
  extern PFNGLACTIVETEXTUREARBPROC       glActiveTextureARB        ;
  extern PFNGLCLIENTACTIVETEXTUREARBPROC glClientActiveTextureARB  ;
  extern PFNGLMULTITEXCOORD1DARBPROC     glMultiTexCoord1dARB      ;
  extern PFNGLMULTITEXCOORD1DVARBPROC    glMultiTexCoord1dvARB     ;
  extern PFNGLMULTITEXCOORD1FARBPROC     glMultiTexCoord1fARB      ;
  extern PFNGLMULTITEXCOORD1FVARBPROC    glMultiTexCoord1fvARB     ;
  extern PFNGLMULTITEXCOORD1IARBPROC     glMultiTexCoord1iARB      ;
  extern PFNGLMULTITEXCOORD1IVARBPROC    glMultiTexCoord1ivARB     ;
  extern PFNGLMULTITEXCOORD1SARBPROC     glMultiTexCoord1sARB      ;
  extern PFNGLMULTITEXCOORD1SVARBPROC    glMultiTexCoord1svARB     ;
  extern PFNGLMULTITEXCOORD2DARBPROC     glMultiTexCoord2dARB      ;
  extern PFNGLMULTITEXCOORD2DVARBPROC    glMultiTexCoord2dvARB     ;
  extern PFNGLMULTITEXCOORD2FARBPROC     glMultiTexCoord2fARB      ;
  extern PFNGLMULTITEXCOORD2FVARBPROC    glMultiTexCoord2fvARB     ;
  extern PFNGLMULTITEXCOORD2IARBPROC     glMultiTexCoord2iARB      ;
  extern PFNGLMULTITEXCOORD2IVARBPROC    glMultiTexCoord2ivARB     ;
  extern PFNGLMULTITEXCOORD2SARBPROC     glMultiTexCoord2sARB      ;
  extern PFNGLMULTITEXCOORD2SVARBPROC    glMultiTexCoord2svARB     ;
  extern PFNGLMULTITEXCOORD3DARBPROC     glMultiTexCoord3dARB      ;
  extern PFNGLMULTITEXCOORD3DVARBPROC    glMultiTexCoord3dvARB     ;
  extern PFNGLMULTITEXCOORD3FARBPROC     glMultiTexCoord3fARB      ;
  extern PFNGLMULTITEXCOORD3FVARBPROC    glMultiTexCoord3fvARB     ;
  extern PFNGLMULTITEXCOORD3IARBPROC     glMultiTexCoord3iARB      ;
  extern PFNGLMULTITEXCOORD3IVARBPROC    glMultiTexCoord3ivARB     ;
  extern PFNGLMULTITEXCOORD3SARBPROC     glMultiTexCoord3sARB      ;
  extern PFNGLMULTITEXCOORD3SVARBPROC    glMultiTexCoord3svARB     ;
  extern PFNGLMULTITEXCOORD4DARBPROC     glMultiTexCoord4dARB      ;
  extern PFNGLMULTITEXCOORD4DVARBPROC    glMultiTexCoord4dvARB     ;
  extern PFNGLMULTITEXCOORD4FARBPROC     glMultiTexCoord4fARB      ;
  extern PFNGLMULTITEXCOORD4FVARBPROC    glMultiTexCoord4fvARB     ;
  extern PFNGLMULTITEXCOORD4IARBPROC     glMultiTexCoord4iARB      ;
  extern PFNGLMULTITEXCOORD4IVARBPROC    glMultiTexCoord4ivARB     ;
  extern PFNGLMULTITEXCOORD4SARBPROC     glMultiTexCoord4sARB      ;
  extern PFNGLMULTITEXCOORD4SVARBPROC    glMultiTexCoord4svARB     ;
#endif

//#####################################################################################
//                              ARB TRANSPOSE MATRIX
//#####################################################################################

#ifdef GL_ARB_transpose_matrix
  extern PFNGLLOADTRANSPOSEMATRIXFARBPROC glLoadTransposeMatrixfARB ;
  extern PFNGLLOADTRANSPOSEMATRIXDARBPROC glLoadTransposeMatrixdARB ;
  extern PFNGLMULTTRANSPOSEMATRIXFARBPROC glMultTransposeMatrixfARB ;
  extern PFNGLMULTTRANSPOSEMATRIXDARBPROC glMultTransposeMatrixdARB ;
#endif

//#####################################################################################
//                                ARB MULTISAMPLE
//#####################################################################################

#ifdef GL_ARB_multisample
  extern PFNGLSAMPLECOVERAGEARBPROC  glSampleCoverageARB ;
  extern PFNGLSAMPLEPASSARBPROC      glSamplePassARB     ;
#endif

//#####################################################################################
//                             ARB TEXTURE COMPRESSION
//#####################################################################################

#ifdef GL_ARB_texture_compression
  extern PFNGLCOMPRESSEDTEXIMAGE3DARBPROC      glCompressedTexImage3DARB     ;
  extern PFNGLCOMPRESSEDTEXIMAGE2DARBPROC      glCompressedTexImage2DARB     ;
  extern PFNGLCOMPRESSEDTEXIMAGE1DARBPROC      glCompressedTexImage1DARB     ;
  extern PFNGLCOMPRESSEDTEXSUBIMAGE3DARBPROC   glCompressedTexSubImage3DARB  ;
  extern PFNGLCOMPRESSEDTEXSUBIMAGE2DARBPROC   glCompressedTexSubImage2DARB  ;
  extern PFNGLCOMPRESSEDTEXSUBIMAGE1DARBPROC   glCompressedTexSubImage1DARB  ;
  extern PFNGLGETCOMPRESSEDTEXIMAGEARBPROC     glGetCompressedTexImageARB    ;
#endif

//#####################################################################################
//                      EXT BLEND COLOUR - Part Of OpenGL1.2
//#####################################################################################

#ifdef GL_EXT_blend_color
  extern PFNGLBLENDCOLOREXTPROC  glBlendColorEXT  ;
#endif

//#####################################################################################
//                      EXT POLYGON OFFSET - Part Of OpenGL1.1
//#####################################################################################

#ifdef GL_EXT_polygon_offset
  extern PFNGLPOLYGONOFFSETEXTPROC glPolygonOffsetEXT ;
#endif

//#####################################################################################
//                        EXT TEXTURE 3D - Part Of OpenGL1.2
//#####################################################################################

#ifdef GL_EXT_texture3D
  extern PFNGLTEXIMAGE3DEXTPROC     glTexImage3DEXT    ;
  extern PFNGLTEXSUBIMAGE3DEXTPROC  glTexSubImage3DEXT ;
#endif

//#####################################################################################
//                         EXT HISTOGRAM - Part Of OpenGL1.2
//#####################################################################################

#ifdef GL_EXT_histogram
  extern PFNGLGETHISTOGRAMEXTPROC             glGetHistogramEXT             ;
  extern PFNGLGETHISTOGRAMPARAMETERFVEXTPROC  glGetHistogramParameterfvEXT  ;
  extern PFNGLGETHISTOGRAMPARAMETERIVEXTPROC  glGetHistogramParameterivEXT  ;
  extern PFNGLGETMINMAXEXTPROC                glGetMinmaxEXT                ;
  extern PFNGLGETMINMAXPARAMETERFVEXTPROC     glGetMinmaxParameterfvEXT     ;
  extern PFNGLGETMINMAXPARAMETERIVEXTPROC     glGetMinmaxParameterivEXT     ;
  extern PFNGLHISTOGRAMEXTPROC                glHistogramEXT                ;
  extern PFNGLMINMAXEXTPROC                   glMinmaxEXT                   ;
  extern PFNGLRESETHISTOGRAMEXTPROC           glResetHistogramEXT           ;
  extern PFNGLRESETMINMAXEXTPROC              glResetMinmaxEXT              ;
#endif

//#####################################################################################
//                     EXT CONVOLUTION FILTERS - Part Of OpenGL1.2
//#####################################################################################

#ifdef GL_EXT_convolution
  extern PFNGLCONVOLUTIONFILTER1DEXTPROC       glConvolutionFilter1DEXT       ;
  extern PFNGLCONVOLUTIONFILTER2DEXTPROC       glConvolutionFilter2DEXT       ;
  extern PFNGLCONVOLUTIONPARAMETERFEXTPROC     glConvolutionParameterfEXT     ;
  extern PFNGLCONVOLUTIONPARAMETERFVEXTPROC    glConvolutionParameterfvEXT    ;
  extern PFNGLCONVOLUTIONPARAMETERIEXTPROC     glConvolutionParameteriEXT     ;
  extern PFNGLCONVOLUTIONPARAMETERIVEXTPROC    glConvolutionParameterivEXT    ;
  extern PFNGLCOPYCONVOLUTIONFILTER1DEXTPROC   glCopyConvolutionFilter1DEXT   ;
  extern PFNGLCOPYCONVOLUTIONFILTER2DEXTPROC   glCopyConvolutionFilter2DEXT   ;
  extern PFNGLGETCONVOLUTIONFILTEREXTPROC      glGetConvolutionFilterEXT      ;
  extern PFNGLGETCONVOLUTIONPARAMETERFVEXTPROC glGetConvolutionParameterfvEXT ;
  extern PFNGLGETCONVOLUTIONPARAMETERIVEXTPROC glGetConvolutionParameterivEXT ;
  extern PFNGLGETSEPARABLEFILTEREXTPROC        glGetSeparableFilterEXT        ;
  extern PFNGLSEPARABLEFILTER2DEXTPROC         glSeparableFilter2DEXT         ;
#endif

//#####################################################################################
//                      EXT TEXTURE OBJECT - Part Of OpenGL1.1
//#####################################################################################

#ifdef GL_EXT_texture_object
  extern PFNGLARETEXTURESRESIDENTEXTPROC glAreTexturesResidentEXT ;
  extern PFNGLBINDTEXTUREEXTPROC         glBindTextureEXT         ;
  extern PFNGLDELETETEXTURESEXTPROC      glDeleteTexturesEXT      ;
  extern PFNGLGENTEXTURESEXTPROC         glGenTexturesEXT         ;
  extern PFNGLISTEXTUREEXTPROC           glIsTextureEXT           ;
  extern PFNGLPRIORITIZETEXTURESEXTPROC  glPrioritizeTexturesEXT  ;
#endif

//#####################################################################################
//                      EXT VERTEX ARRAY - Part Of OpenGL1.1
//#####################################################################################

#ifdef GL_EXT_vertex_array
  extern PFNGLARRAYELEMENTEXTPROC     glArrayElementEXT     ;
  extern PFNGLCOLORPOINTEREXTPROC     glColorPointerEXT     ;
  extern PFNGLDRAWARRAYSEXTPROC       glDrawArraysEXT       ;
  extern PFNGLEDGEFLAGPOINTEREXTPROC  glEdgeFlagPointerEXT  ;
  extern PFNGLGETPOINTERVEXTPROC      glGetPointervEXT      ;
  extern PFNGLINDEXPOINTEREXTPROC     glIndexPointerEXT     ;
  extern PFNGLNORMALPOINTEREXTPROC    glNormalPointerEXT    ;
  extern PFNGLTEXCOORDPOINTEREXTPROC  glTexCoordPointerEXT  ;
  extern PFNGLVERTEXPOINTEREXTPROC    glVertexPointerEXT    ;
#endif

//#####################################################################################
//                                 EXT BLEND MINMAX
//#####################################################################################

#ifdef GL_EXT_blend_minmax
  extern PFNGLBLENDEQUATIONEXTPROC   glBlendEquationEXT  ;
#endif

//#####################################################################################
//                               EXT POINT PARAMETERS
//#####################################################################################

#ifdef GL_EXT_point_parameters
  extern PFNGLPOINTPARAMETERFEXTPROC   glPointParameterfEXT   ;
  extern PFNGLPOINTPARAMETERFVEXTPROC  glPointParameterfvEXT  ;
  extern PFNGLPOINTPARAMETERFSGISPROC  glPointParameterfSGIS  ;
  extern PFNGLPOINTPARAMETERFVSGISPROC glPointParameterfvSGIS ;
#endif

//#####################################################################################
//                               EXT LIGHT TEXTURE
//#####################################################################################

#ifdef GL_EXT_light_texture
  extern PFNGLAPPLYTEXTUREEXTPROC    glApplyTextureEXT    ;
  extern PFNGLTEXTURELIGHTEXTPROC    glTextureLightEXT    ;
  extern PFNGLTEXTUREMATERIALEXTPROC glTextureMaterialEXT ;
#endif

//#####################################################################################
//                                EXT INDEX MATERIAL
//#####################################################################################

#ifdef GL_EXT_index_material
  extern PFNGLINDEXMATERIALEXTPROC glIndexMaterialEXT ;
#endif

//#####################################################################################
//                                  EXT INDEX FUNC
//#####################################################################################

#ifdef GL_EXT_index_func
  extern PFNGLINDEXFUNCEXTPROC glIndexFuncEXT ;
#endif

//#####################################################################################
//                             EXT COMPILED VERTEX ARRAY
//#####################################################################################

#ifdef GL_EXT_compiled_vertex_array
  extern PFNGLLOCKARRAYSEXTPROC   glLockArraysEXT   ;
  extern PFNGLUNLOCKARRAYSEXTPROC glUnlockArraysEXT ;
#endif

//#####################################################################################
//                                EXT CULL VERTEX
//#####################################################################################

#ifdef GL_EXT_cull_vertex
  extern PFNGLCULLPARAMETERDVEXTPROC glCullParameterdvEXT ;
  extern PFNGLCULLPARAMETERFVEXTPROC glCullParameterfvEXT ;
#endif

//#####################################################################################
//                               EXT DRAW ARRAY RANGE
//#####################################################################################

#ifdef GL_EXT_draw_range_elements
  extern PFNGLDRAWRANGEELEMENTSEXTPROC glDrawRangeElementsEXT ;
#endif

//#####################################################################################
//                                EXT PIXEL TRANSFORM
//#####################################################################################

#ifdef GL_EXT_pixel_transform
  extern PFNGLPIXELTRANSFORMPARAMETERIEXTPROC   glPixelTransformParameteriEXT  ;
  extern PFNGLPIXELTRANSFORMPARAMETERFEXTPROC   glPixelTransformParameterfEXT  ;
  extern PFNGLPIXELTRANSFORMPARAMETERIVEXTPROC  glPixelTransformParameterivEXT ;
  extern PFNGLPIXELTRANSFORMPARAMETERFVEXTPROC  glPixelTransformParameterfvEXT ;
#endif

//#####################################################################################
//                              EXT SECONDARY COLOUR
//#####################################################################################

#ifdef GL_EXT_secondary_color
  extern PFNGLSECONDARYCOLOR3BEXTPROC       glSecondaryColor3bEXT      ;
  extern PFNGLSECONDARYCOLOR3BVEXTPROC      glSecondaryColor3bvEXT     ;
  extern PFNGLSECONDARYCOLOR3DEXTPROC       glSecondaryColor3dEXT      ;
  extern PFNGLSECONDARYCOLOR3DVEXTPROC      glSecondaryColor3dvEXT     ;
  extern PFNGLSECONDARYCOLOR3FEXTPROC       glSecondaryColor3fEXT      ;
  extern PFNGLSECONDARYCOLOR3FVEXTPROC      glSecondaryColor3fvEXT     ;
  extern PFNGLSECONDARYCOLOR3IEXTPROC       glSecondaryColor3iEXT      ;
  extern PFNGLSECONDARYCOLOR3IVEXTPROC      glSecondaryColor3ivEXT     ;
  extern PFNGLSECONDARYCOLOR3SEXTPROC       glSecondaryColor3sEXT      ;
  extern PFNGLSECONDARYCOLOR3SVEXTPROC      glSecondaryColor3svEXT     ;
  extern PFNGLSECONDARYCOLOR3UBEXTPROC      glSecondaryColor3ubEXT     ;
  extern PFNGLSECONDARYCOLOR3UBVEXTPROC     glSecondaryColor3ubvEXT    ;
  extern PFNGLSECONDARYCOLOR3UIEXTPROC      glSecondaryColor3uiEXT     ;
  extern PFNGLSECONDARYCOLOR3UIVEXTPROC     glSecondaryColor3uivEXT    ;
  extern PFNGLSECONDARYCOLOR3USEXTPROC      glSecondaryColor3usEXT     ;
  extern PFNGLSECONDARYCOLOR3USVEXTPROC     glSecondaryColor3usvEXT    ;
  extern PFNGLSECONDARYCOLORPOINTEREXTPROC  glSecondaryColorPointerEXT ;
#endif

//#####################################################################################
//                                EXT PERTURB NORMAL
//#####################################################################################

#ifdef GL_EXT_texture_perturb_normal
  extern PFNGLTEXTURENORMALEXTPROC  glTextureNormalEXT  ;
#endif

//#####################################################################################
//                                  EXT FOG COORD
//#####################################################################################

#ifdef GL_EXT_fog_coord
  extern PFNGLFOGCOORDFEXTPROC       glFogCoordfEXT       ;
  extern PFNGLFOGCOORDFVEXTPROC      glFogCoordfvEXT      ;
  extern PFNGLFOGCOORDDEXTPROC       glFogCoorddEXT       ;
  extern PFNGLFOGCOORDDVEXTPROC      glFogCoorddvEXT      ;
  extern PFNGLFOGCOORDPOINTEREXTPROC glFogCoordPointerEXT ;
#endif

//#####################################################################################
//                              EXT BLEND FUNC SEPERATE
//#####################################################################################

#ifdef GL_EXT_blend_func_separate
  extern PFNGLBLENDFUNCSEPARATEEXTPROC glBlendFuncSeparateEXT ;
#endif

//#####################################################################################
//                                EXT VERTEX WEIGHTING
//#####################################################################################

#ifdef GL_EXT_vertex_weighting
  extern PFNGLVERTEXWEIGHTFEXTPROC       glVertexWeightfEXT       ;
  extern PFNGLVERTEXWEIGHTFVEXTPROC      glVertexWeightfvEXT      ;
  extern PFNGLVERTEXWEIGHTPOINTEREXTPROC glVertexWeightPointerEXT ;
#endif

//#####################################################################################
//                               NV VERTEX ARRAY RANGE
//#####################################################################################

#ifdef GL_NV_vertex_array_range
  extern PFNGLFLUSHVERTEXARRAYRANGENVPROC glFlushVertexArrayRangeNV ;
  extern PFNGLVERTEXARRAYRANGENVPROC      glVertexArrayRangeNV      ;
#endif


//#####################################################################################
//                               NV REGISTER COMBINERS
//#####################################################################################

#ifdef GL_NV_register_combiners
  extern PFNGLCOMBINERPARAMETERFVNVPROC               glCombinerParameterfvNV              ;
  extern PFNGLCOMBINERPARAMETERFNVPROC                glCombinerParameterfNV               ;
  extern PFNGLCOMBINERPARAMETERIVNVPROC               glCombinerParameterivNV              ;
  extern PFNGLCOMBINERPARAMETERINVPROC                glCombinerParameteriNV               ;
  extern PFNGLCOMBINERINPUTNVPROC                     glCombinerInputNV                    ;
  extern PFNGLCOMBINEROUTPUTNVPROC                    glCombinerOutputNV                   ;
  extern PFNGLFINALCOMBINERINPUTNVPROC                glFinalCombinerInputNV               ;
  extern PFNGLGETCOMBINERINPUTPARAMETERFVNVPROC       glGetCombinerInputParameterfvNV      ;
  extern PFNGLGETCOMBINERINPUTPARAMETERIVNVPROC       glGetCombinerInputParameterivNV      ;
  extern PFNGLGETCOMBINEROUTPUTPARAMETERFVNVPROC      glGetCombinerOutputParameterfvNV     ;
  extern PFNGLGETCOMBINEROUTPUTPARAMETERIVNVPROC      glGetCombinerOutputParameterivNV     ;
  extern PFNGLGETFINALCOMBINERINPUTPARAMETERFVNVPROC  glGetFinalCombinerInputParameterfvNV ;
  extern PFNGLGETFINALCOMBINERINPUTPARAMETERIVNVPROC  glGetFinalCombinerInputParameterivNV ;
#endif

//#####################################################################################
//                                EXT MULTISAMPLE
//#####################################################################################

#ifdef GL_EXT_multisample
  extern PFNGLSAMPLEMASKEXTPROC    glSampleMaskEXT    ;
  extern PFNGLSAMPLEPATTERNEXTPROC glSamplePatternEXT ;
#endif

#endif