/*                         Compile With:   make                               */

/*##############################################################################
###                              INCLUDES
##############################################################################*/

#define WIN32
#include <GL/glut.h>
#include <stdio.h>
#include <stdlib.h>
#include "tga.h"
#include "fruity_glext.h"

/*##############################################################################
###                        STRUCTURE DEFINITIONS
##############################################################################*/

struct cam
{
  float xrot;
  float yrot;
  GLboolean LMB;
  int LastPosition[2];
} CAM;

/*##############################################################################
###                              GLOBALS
##############################################################################*/

GLuint Reflection,surface;

/*------------------------------------------------------------------------------

Two GLuint's to store textures for the Reflection Map and A Rusty Surface texture.

------------------------------------------------------------------------------*/

/*##############################################################################
###                         FUNCTION PROTOTYPES
##############################################################################*/

void init(void); 
void reshape(int,int); 
void display(void);
void Button(int,int,int,int);
void Motion(int,int);

/*##############################################################################
###                            INIT FUNCTION
##############################################################################*/

void init(void)
{
  GLfloat lmodel_ambient[] = { 0.3f, 0.3f, 0.3f, 1.0 };

  glClearColor(0.0, 0.0, 0.0, 0.0);
  glShadeModel(GL_SMOOTH);
  glEnable(GL_DEPTH_TEST);

  /* SET UP LIGHTING */
  { 
    GLfloat ambient[] = { 0.3f, 0.3f, 0.3f, 1.0f };
    GLfloat diffuse[] = { 0.7f, 0.7f, 0.7f, 1.0f };
    GLfloat specular[] ={ 1.0f, 1.0f, 1.0f, 1.0f };
    GLfloat position[] ={ 0.5f, 0.1f, 1.0f, 0.0f };
      
    glLightfv(GL_LIGHT0, GL_AMBIENT, ambient);
    glLightfv(GL_LIGHT0, GL_DIFFUSE, diffuse);
    glLightfv(GL_LIGHT0, GL_SPECULAR, specular);
    glLightfv(GL_LIGHT0, GL_POSITION, position);
  }

  glLightModelfv(GL_LIGHT_MODEL_AMBIENT, lmodel_ambient);
  glLightModelf(GL_LIGHT_MODEL_LOCAL_VIEWER, GL_FALSE);

  glEnable(GL_LIGHTING);
  glEnable(GL_LIGHT0);

  /* CHECK FOR OPENGL 1.2 */
  if(CheckGLExt("GL_VERSION_1_2"))
    glLightModeli(GL_LIGHT_MODEL_COLOR_CONTROL_EXT,GL_SEPARATE_SPECULAR_COLOR_EXT);

  /* CHECK FOR Multitextring */
  if(!CheckGLExt("GL_ARB_multitexture")) 
  {
    printf("sorry, your graphics card is chud\n");
    exit(0);
  }

  /* INIT CAM STRUCTURE */
  CAM.xrot = 0;
  CAM.yrot = 0;
  CAM.LMB = GL_FALSE;


  /* LOAD TEXTURE AND SET UP TEXTURING */
  glGenTextures(1,&Reflection);
  loadTGA( "Reflect.tga", Reflection );
  glGenTextures(1,&surface);
  loadTGA( "Rusty.tga", surface );
  glEnable(GL_TEXTURE_2D);

  /* ENABLE BACK FACE CULLING */
  glCullFace(GL_FRONT);
  glEnable(GL_CULL_FACE);

}

/*------------------------------------------------------------------------------

TO sum up whats going on this Time:

*Initialise The Lighting

*Check for The existance of openGL 1.2

*Check For The Existence of Multitexturing

*Initialise The CAM structure

*Load the two required Textures

*Enable Back Face Culling 

------------------------------------------------------------------------------*/

/*##############################################################################
###                           MOUSE BUTTON FUNCTION
##############################################################################*/

void Button(int button, int state, int x, int y) 
{
  if(state == GLUT_DOWN)
  {
    switch(button)
    {
      case GLUT_LEFT_BUTTON:
        CAM.LMB = GL_TRUE;
        CAM.LastPosition[0] = x;
        CAM.LastPosition[1] = y;
      break;

      default: 
      break;
    }
  }
  else
  {
    switch(button)
    {
      case GLUT_LEFT_BUTTON:
        CAM.LMB = GL_FALSE;
      break;

      default: 
      break;
    }
  }
  glutPostRedisplay();
}

/*##############################################################################
###                           MOUSE MOTION FUNCTION
##############################################################################*/

void Motion(int x, int y)
{
  int stepX = (x - CAM.LastPosition[0]);
  int stepY = (y - CAM.LastPosition[1]);

  CAM.LastPosition[0] = x; CAM.LastPosition[1] = y;

  if(CAM.LMB) {
    CAM.xrot += stepX;
    CAM.yrot += stepY;
  }

  glutPostRedisplay();
}

/*##############################################################################
###                            RESHAPE FUNCTION
##############################################################################*/

void reshape(int w, int h)
{
   if(h == 0) h = 1;
   glViewport(0, 0, (GLsizei) w, (GLsizei) h);
   glMatrixMode(GL_PROJECTION);
   glLoadIdentity();
   gluPerspective(45.0, (GLfloat) w/(GLfloat) h, 0.1, 100);
   glMatrixMode(GL_MODELVIEW);
   glLoadIdentity();
}


/*##############################################################################
###                            DISPLAY FUNCTION
##############################################################################*/


void display(void)
{
  // MATERIAL AND TEXTURE BLENDING PROPERTIES:
  GLfloat specular[4] = {1,1,1,1};
  GLfloat diffuse[4] = {0.6,0.6,0.6,1};
  GLfloat ambient[4] = {0.2,0.2,0.2,1.0};
  GLfloat blend_colour[4] = {0.45,0.35,0.3,1};

  glClear( GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT );
  glLoadIdentity();
  gluLookAt( 0 , 0 , 2.5 , 0 , 0 , 0 , 0 , 1, 0 );
  glRotatef(CAM.xrot,0,1,0);
  glRotatef(CAM.yrot,1,0,0);


  // TEXTURE-UNIT #0:
  glActiveTextureARB(GL_TEXTURE0_ARB);
  glEnable(GL_TEXTURE_2D);
  glDisable(GL_TEXTURE_GEN_S);
  glDisable(GL_TEXTURE_GEN_T);
  glBindTexture(GL_TEXTURE_2D, surface );
  glTexEnvf (GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE , GL_MODULATE);

  // TEXTURE-UNIT #1:
  glActiveTextureARB(GL_TEXTURE1_ARB);
  glEnable(GL_TEXTURE_2D); 
  glTexGeni(GL_S, GL_TEXTURE_GEN_MODE, GL_SPHERE_MAP);
  glTexGeni(GL_T, GL_TEXTURE_GEN_MODE, GL_SPHERE_MAP);
  glEnable(GL_TEXTURE_GEN_S);
  glEnable(GL_TEXTURE_GEN_T);
  glBindTexture(GL_TEXTURE_2D, Reflection );
  glTexEnvfv(GL_TEXTURE_ENV,GL_TEXTURE_ENV_COLOR , blend_colour);
  glTexEnvf (GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE , GL_BLEND);


  // DRAW OBJECT:
  glPushMatrix();
   glMaterialfv(GL_FRONT,GL_SPECULAR,specular);
   glMaterialfv(GL_FRONT,GL_DIFFUSE,diffuse);
   glMaterialfv(GL_FRONT,GL_AMBIENT,ambient);
   glMaterialf(GL_FRONT,GL_SHININESS,65);
   glColor3f(1,1,1);
   glutSolidTeapot(0.5); 
  glPopMatrix();

  glutSwapBuffers();
}

/*------------------------------------------------------------------------------

First A Quick Summary:

*Create Some Arrays to use for material proerties and texture blending

*Sort Out The Camera Position

*Set Up Texture Unit 0: No Auto Tex Coord Generation, Bind Surface Texture.

*Set Up Texture Unit 1:Enable sphere mapping, Bind Reflection Map.

*Draw Object


Now A Bit Of Extra Info:

To start, glutSolidTeapot() is the only one of the glut primatives to have tex
co-ords. All of those are defined with glTexCoord*().

When we do multitexturing we should be using 'glMultiTexCoord*ARB()' to define
pairs of texturing co-ords. So How Can This Work?

When 'glTexCoord*()' is called during multitexturing, it actually only affects
the first texturing unit, GL_TEXTURE0_ARB. So when we are drawing the teapot
we can bind a texture to the first texturing unit, and use that for our surface
texture. We must ensure that we turn off The automatic texture coordinate 
generation for this unit, because it is the only one with pre-defined texture
co-ords.

When We get to the second texture unit we need to generate sphere map texture
co-ords as we have done before. We have two new lines though:

     " glTexEnvfv(GL_TEXTURE_ENV,GL_TEXTURE_ENV_COLOR , blend_value); "
       " glTexEnvf (GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE , GL_BLEND); "

This reflection map has GL_BLEND enabled (Rather than MODULATE etc). What 
GL_BLEND does with regard to textures, is to blend the textures RGBA with a 
specified colour. The first line actually tells it what colour to use, and the
second enables GL_BLEND.

The only good thing about GL_BLEND I can think of is that it tends to change the
hue to your textures. For example, I've tried to get the blend colour to look 
a wee bit like copper. 

This, definately looks loads better if you've got separate specular, if not, try
reducing the shininess and increasing the blend_colour; alternatively, buy a
new graphics card........

------------------------------------------------------------------------------*/   

/*##############################################################################
###                              MAIN FUNCTION
##############################################################################*/

int main(int argc, char** argv)
{
   glutInit(&argc, argv);
   glutInitDisplayMode(GLUT_DOUBLE | GLUT_DEPTH | GLUT_RGB );
   glutInitWindowSize( 500 , 500 );
   glutInitWindowPosition(100, 100);
   glutCreateWindow("Multitextured Sphere Mapped Teapot, with Seperate Specular Colour.  LMB - Rotate");
   init();
   glutReshapeFunc(reshape);
   glutDisplayFunc(display);
   glutMouseFunc(Button);
   glutMotionFunc(Motion);
   glutMainLoop();
   return 0;
}