//####################################################################################################################
//
//                               GLEXT SUPPORT HEADER FILE - MAKING EXTENSIONS EASIER!
//                               *****************************************************
//
//                                           R BATEMAN: 15 SEPT 2000
//                                           ***********************
//
//  THE PURPOSE OF THIS HEADER FILE IS TO ALLOW QUICK AND EASY WAYS TO CHECK AND VERIFY IF VARIOUS OPENGL EXTENSIONS
//  ARE SUPPORTED. IT DEFINES THE FUNCTION AND SETS UP THEIR ADDRESS AND VERIFICATION AT RUNTIME.
//
//  USAGE: FIRST INCLUDE                "#include <GL/glut>"           (or other gl includes if you're not using glut)
//
//         THEN include             "#include "glext_support.h"
//
//         THEN QUERY AN OPENGL EXTENSION LISTED BELOW
//
//                                Check_GL_Ext( char *extString );
//
//         THIS FUNCTION WILL RETURN TRUE IF ITS AVAILABLE, FALSE IF IT ISNT. IF IT IS, IT WILL AUOTMATICALLY SET
//         UP THE FUNCTIONS IN THE EXTENSIONS FOR YOU. ALWAYS CHECK THE RETURNED VALUE, YOU MUST NOT USE A FUNCTION 
//         AT RUNTIME IF IT IS NOT AVAILABLE!!!!!!!!!!!!!
//
//
//  CURRENTLY SUPPORTED EXTENSIONS (Hardware Dependent)
//  ******************************
//
//  GL_VERSION_1_2
//  GL_ARB_multitexture
//  GL_NV_register_combiners
//  GL_NV_vertex_array_range
//  GL_EXT_point_parameters
//  GL_EXT_convolution
//
//  GL_ARB_texture_env_add                        GL_SGI_texture_color_table
//  GL_ARB_texture_cube_map                       GL_SGIS_texture_lod
//  GL_EXT_abgr                                   GL_SGIS_generate_mipmap
//  GL_EXT_texture                                GL_SGIX_clipmap
//  GL_EXT_color_matrix                           GL_SGIX_shadow
//  GL_EXT_cmyka                                  GL_SGIS_texture_edge_clamp
//  GL_EXT_packed_pixels                          GL_SGIS_texture_border_clamp
//  GL_EXT_rescale_normal                         GL_SGIX_interlace
//  GL_EXT_misc_attribute                         GL_SGIX_pixel_tiles
//  GL_EXT_blend_subtract                         GL_SGIX_texture_select
//  GL_EXT_blend_logic_op                         GL_SGIX_texture_scale_bias
//  GL_EXT_clip_volume_hint                       GL_SGIX_depth_texture
//  GL_EXT_index_texture                          GL_SGIX_fog_offset
//  GL_EXT_index_array_formats                    GL_SGIX_texture_add_env
//  GL_WIN_phong_shading                          GL_SGIX_ir_instrument1
//  GL_WIN_specular_fog                           GL_SGIX_calligraphic_fragment
//  GL_EXT_bgra                                   GL_SGIX_texture_lod_bias
//  GL_EXT_pixel_transform_color_table            GL_SGIX_shadow_ambient
//  GL_EXT_shared_texture_palette                 GL_SGIX_ycrcb
//  GL_EXT_separate_specular_color                GL_SGIX_blend_alpha_minmax
//  GL_EXT_texture_env_combine                    GL_SGIX_fog_scale
//  GL_EXT_stencil_wrap                           GL_SGIX_subsample
//  GL_EXT_422_pixels                             GL_SGIX_ycrcba
//  GL_EXT_texture_env_add                        GL_SGIX_ycrcb_subsample
//  GL_EXT_texture_lod_bias                       GL_SGIX_depth_pass_instrument
//  GL_EXT_texture_filter_anisotropic             GL_SGI_vertex_preclip
//  GL_NV_texgen_reflection                       GL_SGIX_convolution_accuracy
//  GL_NV_light_max_exponent                      GL_SGIX_resample
//  GL_NV_fog_distance                            GL_SGIS_point_line_texgen
//  GL_NV_texgen_emboss
//  GL_NV_blend_square
//  GL_NV_texture_env_combine4
//  
//
//####################################################################################################################

#ifndef GLEXTSUPPORT_H
#define GLEXTSUPPORT_H

#define MAX_EXTENSION_SPACE 10240   // Characters for Extension-Strings
#define MAX_EXTENSION_LENGTH 256    // Maximum Of Characters In One Extension-String
#include "glext.h"
#include <string.h>

GLboolean isInString(char *, const char *);

//####################################################################################################################
//
//                                               OPENGL 1.2
//
//####################################################################################################################

  PFNGLBLENDCOLORPROC                glBlendColor                = NULL;
  PFNGLBLENDEQUATIONPROC             glBlendEquation             = NULL;
  PFNGLDRAWRANGEELEMENTSPROC         glDrawRangeElements         = NULL;
  PFNGLCOLORTABLEPROC                glColorTable                = NULL;
  PFNGLCOLORTABLEPARAMETERFVPROC     glColorTableParameterfv     = NULL;
  PFNGLCOLORTABLEPARAMETERIVPROC     glColorTableParameteriv     = NULL;
  PFNGLCOPYCOLORTABLEPROC            glCopyColorTable            = NULL;
  PFNGLGETCOLORTABLEPROC             glGetColorTable             = NULL;
  PFNGLGETCOLORTABLEPARAMETERFVPROC  glGetColorTableParameterfv  = NULL;
  PFNGLGETCOLORTABLEPARAMETERIVPROC  glGetColorTableParameteriv  = NULL;
  PFNGLCOLORSUBTABLEPROC             glColorSubTable             = NULL;
  PFNGLCOPYCOLORSUBTABLEPROC         glCopyColorSubTable         = NULL;
  PFNGLCONVOLUTIONFILTER1DPROC       glConvolutionFilter1D       = NULL;
  PFNGLCONVOLUTIONFILTER2DPROC       glConvolutionFilter2D       = NULL;
  PFNGLCONVOLUTIONPARAMETERFPROC     glConvolutionParameterf     = NULL;
  PFNGLCONVOLUTIONPARAMETERFVPROC    glConvolutionParameterfv    = NULL;
  PFNGLCONVOLUTIONPARAMETERIPROC     glConvolutionParameteri     = NULL;
  PFNGLCONVOLUTIONPARAMETERIVPROC    glConvolutionParameteriv    = NULL;
  PFNGLCOPYCONVOLUTIONFILTER1DPROC   glCopyConvolutionFilter1D   = NULL;
  PFNGLCOPYCONVOLUTIONFILTER2DPROC   glCopyConvolutionFilter2D   = NULL;
  PFNGLGETCONVOLUTIONFILTERPROC      glGetConvolutionFilter      = NULL;
  PFNGLGETCONVOLUTIONPARAMETERFVPROC glGetConvolutionParameterfv = NULL;
  PFNGLGETCONVOLUTIONPARAMETERIVPROC glGetConvolutionParameteriv = NULL;
  PFNGLGETSEPARABLEFILTERPROC        glGetSeparableFilter        = NULL;
  PFNGLSEPARABLEFILTER2DPROC         glSeparableFilter2D         = NULL;
  PFNGLGETHISTOGRAMPROC              glGetHistogram              = NULL;
  PFNGLGETHISTOGRAMPARAMETERFVPROC   glGetHistogramParameterfv   = NULL;
  PFNGLGETHISTOGRAMPARAMETERIVPROC   glGetHistogramParameteriv   = NULL;
  PFNGLGETMINMAXPROC                 glGetMinmax                 = NULL;
  PFNGLGETMINMAXPARAMETERFVPROC      glGetMinmaxParameterfv      = NULL; 
  PFNGLGETMINMAXPARAMETERIVPROC      glGetMinmaxParameteriv      = NULL;
  PFNGLHISTOGRAMPROC                 glHistogram                 = NULL;
  PFNGLMINMAXPROC                    glMinmax                    = NULL;
  PFNGLRESETHISTOGRAMPROC            glResetHistogram            = NULL; 
  PFNGLRESETMINMAXPROC               glResetMinmax               = NULL;
  PFNGLTEXIMAGE3DPROC                glTexImage3D                = NULL;
  PFNGLTEXSUBIMAGE3DPROC             glTexSubImage3D             = NULL;
  PFNGLCOPYTEXSUBIMAGE3DPROC         glCopyTexSubImage3D         = NULL;

GLboolean initGL_1_2(void) 
{
  char *Version = (char *)glGetString(GL_VERSION);

  if(strncmp(Version,"1.2",3) == 0)
  {

    glBlendColor                = (PFNGLBLENDCOLORPROC)                wglGetProcAddress("glBlendColor");
    glBlendEquation             = (PFNGLBLENDEQUATIONPROC)             wglGetProcAddress("glBlendEquation"); 
    glDrawRangeElements         = (PFNGLDRAWRANGEELEMENTSPROC)         wglGetProcAddress("glDrawRangeElements"); 
    glColorTable                = (PFNGLCOLORTABLEPROC)                wglGetProcAddress("glColorTable");
    glColorTableParameterfv     = (PFNGLCOLORTABLEPARAMETERFVPROC)     wglGetProcAddress("glColorTableParameterfv"); 
    glColorTableParameteriv     = (PFNGLCOLORTABLEPARAMETERIVPROC)     wglGetProcAddress("glColorTableParameteriv");
    glCopyColorTable            = (PFNGLCOPYCOLORTABLEPROC)            wglGetProcAddress("glCopyColorTable");
    glGetColorTable             = (PFNGLGETCOLORTABLEPROC)             wglGetProcAddress("glGetColorTable");
    glGetColorTableParameterfv  = (PFNGLGETCOLORTABLEPARAMETERFVPROC)  wglGetProcAddress("glGetColorTableParameterfv");
    glGetColorTableParameteriv  = (PFNGLGETCOLORTABLEPARAMETERIVPROC)  wglGetProcAddress("glGetColorTableParameteriv");
    glColorSubTable             = (PFNGLCOLORSUBTABLEPROC)             wglGetProcAddress("glColorSubTable");
    glCopyColorSubTable         = (PFNGLCOPYCOLORSUBTABLEPROC)         wglGetProcAddress("glCopyColorSubTable"); 
    glConvolutionFilter1D       = (PFNGLCONVOLUTIONFILTER1DPROC)       wglGetProcAddress("glConvolutionFilter1D");
    glConvolutionFilter2D       = (PFNGLCONVOLUTIONFILTER2DPROC)       wglGetProcAddress("glConvolutionFilter2D");
    glConvolutionParameterf     = (PFNGLCONVOLUTIONPARAMETERFPROC)     wglGetProcAddress("glConvolutionParameterf");
    glConvolutionParameterfv    = (PFNGLCONVOLUTIONPARAMETERFVPROC)    wglGetProcAddress("glConvolutionParameterfv");
    glConvolutionParameteri     = (PFNGLCONVOLUTIONPARAMETERIPROC)     wglGetProcAddress("glConvolutionParameteri");
    glConvolutionParameteriv    = (PFNGLCONVOLUTIONPARAMETERIVPROC)    wglGetProcAddress("glConvolutionParameteriv");
    glCopyConvolutionFilter1D   = (PFNGLCOPYCONVOLUTIONFILTER1DPROC)   wglGetProcAddress("glCopyConvolutionFilter1D");
    glCopyConvolutionFilter2D   = (PFNGLCOPYCONVOLUTIONFILTER2DPROC)   wglGetProcAddress("glCopyConvolutionFilter2D");
    glGetConvolutionFilter      = (PFNGLGETCONVOLUTIONFILTERPROC)      wglGetProcAddress("glGetConvolutionFilter");
    glGetConvolutionParameterfv = (PFNGLGETCONVOLUTIONPARAMETERFVPROC) wglGetProcAddress("glGetConvolutionParameterfv"); 
    glGetConvolutionParameteriv = (PFNGLGETCONVOLUTIONPARAMETERIVPROC) wglGetProcAddress("glGetConvolutionParameteriv");
    glGetSeparableFilter        = (PFNGLGETSEPARABLEFILTERPROC)        wglGetProcAddress("glGetSeparableFilter");
    glSeparableFilter2D         = (PFNGLSEPARABLEFILTER2DPROC)         wglGetProcAddress("glSeparableFilter2D");
    glGetHistogram              = (PFNGLGETHISTOGRAMPROC)              wglGetProcAddress("glGetHistogram");
    glGetHistogramParameterfv   = (PFNGLGETHISTOGRAMPARAMETERFVPROC)   wglGetProcAddress("glGetHistogramParameterfv"); 
    glGetHistogramParameteriv   = (PFNGLGETHISTOGRAMPARAMETERIVPROC)   wglGetProcAddress("glGetHistogramParameteriv");
    glGetMinmax                 = (PFNGLGETMINMAXPROC)                 wglGetProcAddress("glGetMinmax");
    glGetMinmaxParameterfv      = (PFNGLGETMINMAXPARAMETERFVPROC)      wglGetProcAddress("glGetMinmaxParameterfv"); 
    glGetMinmaxParameteriv      = (PFNGLGETMINMAXPARAMETERIVPROC)      wglGetProcAddress("glGetMinmaxParameteriv");
    glHistogram                 = (PFNGLHISTOGRAMPROC)                 wglGetProcAddress("glHistogram");
    glMinmax                    = (PFNGLMINMAXPROC)                    wglGetProcAddress("glMinmax");
    glResetHistogram            = (PFNGLRESETHISTOGRAMPROC)            wglGetProcAddress("glResetHistogram"); 
    glResetMinmax               = (PFNGLRESETMINMAXPROC)               wglGetProcAddress("glResetMinmax");
    glTexImage3D                = (PFNGLTEXIMAGE3DPROC)                wglGetProcAddress("glTexImage3D");
    glTexSubImage3D             = (PFNGLTEXSUBIMAGE3DPROC)             wglGetProcAddress("glTexSubImage3D"); 
    glCopyTexSubImage3D         = (PFNGLCOPYTEXSUBIMAGE3DPROC)         wglGetProcAddress("glCopyTexSubImage3D"); 

    return GL_TRUE;
  }

  return GL_FALSE;
}


//####################################################################################################################
//
//                                           ARB MULTITEXTURING
//
//####################################################################################################################


  PFNGLACTIVETEXTUREARBPROC       glActiveTextureARB        = NULL;
  PFNGLCLIENTACTIVETEXTUREARBPROC glClientActiveTextureARB  = NULL;
  PFNGLMULTITEXCOORD1DARBPROC     glMultiTexCoord1dARB      = NULL;
  PFNGLMULTITEXCOORD1DVARBPROC    glMultiTexCoord1dvARB     = NULL;
  PFNGLMULTITEXCOORD1FARBPROC     glMultiTexCoord1fARB      = NULL;
  PFNGLMULTITEXCOORD1FVARBPROC    glMultiTexCoord1fvARB     = NULL;
  PFNGLMULTITEXCOORD1IARBPROC     glMultiTexCoord1iARB      = NULL;
  PFNGLMULTITEXCOORD1IVARBPROC    glMultiTexCoord1ivARB     = NULL;
  PFNGLMULTITEXCOORD1SARBPROC     glMultiTexCoord1sARB      = NULL;
  PFNGLMULTITEXCOORD1SVARBPROC    glMultiTexCoord1svARB     = NULL;
  PFNGLMULTITEXCOORD2DARBPROC     glMultiTexCoord2dARB      = NULL;
  PFNGLMULTITEXCOORD2DVARBPROC    glMultiTexCoord2dvARB     = NULL;
  PFNGLMULTITEXCOORD2FARBPROC     glMultiTexCoord2fARB      = NULL;
  PFNGLMULTITEXCOORD2FVARBPROC    glMultiTexCoord2fvARB     = NULL;
  PFNGLMULTITEXCOORD2IARBPROC     glMultiTexCoord2iARB      = NULL;
  PFNGLMULTITEXCOORD2IVARBPROC    glMultiTexCoord2ivARB     = NULL;
  PFNGLMULTITEXCOORD2SARBPROC     glMultiTexCoord2sARB      = NULL;
  PFNGLMULTITEXCOORD2SVARBPROC    glMultiTexCoord2svARB     = NULL;
  PFNGLMULTITEXCOORD3DARBPROC     glMultiTexCoord3dARB      = NULL;
  PFNGLMULTITEXCOORD3DVARBPROC    glMultiTexCoord3dvARB     = NULL;
  PFNGLMULTITEXCOORD3FARBPROC     glMultiTexCoord3fARB      = NULL;
  PFNGLMULTITEXCOORD3FVARBPROC    glMultiTexCoord3fvARB     = NULL;
  PFNGLMULTITEXCOORD3IARBPROC     glMultiTexCoord3iARB      = NULL;
  PFNGLMULTITEXCOORD3IVARBPROC    glMultiTexCoord3ivARB     = NULL;
  PFNGLMULTITEXCOORD3SARBPROC     glMultiTexCoord3sARB      = NULL;
  PFNGLMULTITEXCOORD3SVARBPROC    glMultiTexCoord3svARB     = NULL;
  PFNGLMULTITEXCOORD4DARBPROC     glMultiTexCoord4dARB      = NULL;
  PFNGLMULTITEXCOORD4DVARBPROC    glMultiTexCoord4dvARB     = NULL;
  PFNGLMULTITEXCOORD4FARBPROC     glMultiTexCoord4fARB      = NULL;
  PFNGLMULTITEXCOORD4FVARBPROC    glMultiTexCoord4fvARB     = NULL;
  PFNGLMULTITEXCOORD4IARBPROC     glMultiTexCoord4iARB      = NULL;
  PFNGLMULTITEXCOORD4IVARBPROC    glMultiTexCoord4ivARB     = NULL;
  PFNGLMULTITEXCOORD4SARBPROC     glMultiTexCoord4sARB      = NULL;
  PFNGLMULTITEXCOORD4SVARBPROC    glMultiTexCoord4svARB     = NULL;


GLboolean initMultitexture(void) 
{
  char *extensions;
  int len,i;
  extensions=(char *) glGetString(GL_EXTENSIONS);
  len=strlen(extensions);
  for (i=0; i<len; i++)
    if (extensions[i]==' ') extensions[i]='\n';

 #ifdef WIN32

  if ( isInString(extensions,"GL_ARB_multitexture") )
  {
    /*
    **** We Use wglGetProcAddress() To Get Address of GL Extensions At RunTime ****
    */

      glActiveTextureARB       = (PFNGLACTIVETEXTUREARBPROC)       wglGetProcAddress("glActiveTextureARB");
      glClientActiveTextureARB = (PFNGLCLIENTACTIVETEXTUREARBPROC) wglGetProcAddress("glClientActiveTextureARB");	
      glMultiTexCoord1dARB     = (PFNGLMULTITEXCOORD1DARBPROC)     wglGetProcAddress("glMultiTexCoord1dARB");
      glMultiTexCoord1dvARB    = (PFNGLMULTITEXCOORD1DVARBPROC)    wglGetProcAddress("glMultiTexCoord1dvARB");
      glMultiTexCoord1fARB     = (PFNGLMULTITEXCOORD1FARBPROC)     wglGetProcAddress("glMultiTexCoord1fARB");
      glMultiTexCoord1fvARB    = (PFNGLMULTITEXCOORD1FVARBPROC)    wglGetProcAddress("glMultiTexCoord1fvARB");
      glMultiTexCoord1iARB     = (PFNGLMULTITEXCOORD1IARBPROC)     wglGetProcAddress("glMultiTexCoord1iARB");
      glMultiTexCoord1ivARB    = (PFNGLMULTITEXCOORD1IVARBPROC)    wglGetProcAddress("glMultiTexCoord1ivARB");
      glMultiTexCoord1sARB     = (PFNGLMULTITEXCOORD1SARBPROC)     wglGetProcAddress("glMultiTexCoord1sARB");
      glMultiTexCoord1svARB    = (PFNGLMULTITEXCOORD1SVARBPROC)    wglGetProcAddress("glMultiTexCoord1svARB");
      glMultiTexCoord2dARB     = (PFNGLMULTITEXCOORD2DARBPROC)     wglGetProcAddress("glMultiTexCoord2dARB");
      glMultiTexCoord2dvARB    = (PFNGLMULTITEXCOORD2DVARBPROC)    wglGetProcAddress("glMultiTexCoord2dvARB");
      glMultiTexCoord2fARB     = (PFNGLMULTITEXCOORD2FARBPROC)     wglGetProcAddress("glMultiTexCoord2fARB");
      glMultiTexCoord2fvARB    = (PFNGLMULTITEXCOORD2FVARBPROC)    wglGetProcAddress("glMultiTexCoord2fvARB");
      glMultiTexCoord2iARB     = (PFNGLMULTITEXCOORD2IARBPROC)     wglGetProcAddress("glMultiTexCoord2iARB");
      glMultiTexCoord2ivARB    = (PFNGLMULTITEXCOORD2IVARBPROC)    wglGetProcAddress("glMultiTexCoord2ivARB");
      glMultiTexCoord2sARB     = (PFNGLMULTITEXCOORD2SARBPROC)     wglGetProcAddress("glMultiTexCoord2sARB");
      glMultiTexCoord2svARB    = (PFNGLMULTITEXCOORD2SVARBPROC)    wglGetProcAddress("glMultiTexCoord2svARB");
      glMultiTexCoord3dARB     = (PFNGLMULTITEXCOORD3DARBPROC)     wglGetProcAddress("glMultiTexCoord3dARB");
      glMultiTexCoord3dvARB    = (PFNGLMULTITEXCOORD3DVARBPROC)    wglGetProcAddress("glMultiTexCoord3dvARB");
      glMultiTexCoord3fARB     = (PFNGLMULTITEXCOORD3FARBPROC)     wglGetProcAddress("glMultiTexCoord3fARB");
      glMultiTexCoord3fvARB    = (PFNGLMULTITEXCOORD3FVARBPROC)    wglGetProcAddress("glMultiTexCoord3fvARB");
      glMultiTexCoord3iARB     = (PFNGLMULTITEXCOORD3IARBPROC)     wglGetProcAddress("glMultiTexCoord3iARB");
      glMultiTexCoord3ivARB    = (PFNGLMULTITEXCOORD3IVARBPROC)    wglGetProcAddress("glMultiTexCoord3ivARB");
      glMultiTexCoord3sARB     = (PFNGLMULTITEXCOORD3SARBPROC)     wglGetProcAddress("glMultiTexCoord3sARB");
      glMultiTexCoord3svARB    = (PFNGLMULTITEXCOORD3SVARBPROC)    wglGetProcAddress("glMultiTexCoord3svARB");
      glMultiTexCoord4dARB     = (PFNGLMULTITEXCOORD4DARBPROC)     wglGetProcAddress("glMultiTexCoord4dARB");
      glMultiTexCoord4dvARB    = (PFNGLMULTITEXCOORD4DVARBPROC)    wglGetProcAddress("glMultiTexCoord4dvARB");
      glMultiTexCoord4fARB     = (PFNGLMULTITEXCOORD4FARBPROC)     wglGetProcAddress("glMultiTexCoord4fARB");
      glMultiTexCoord4fvARB    = (PFNGLMULTITEXCOORD4FVARBPROC)    wglGetProcAddress("glMultiTexCoord4fvARB");
      glMultiTexCoord4iARB     = (PFNGLMULTITEXCOORD4IARBPROC)     wglGetProcAddress("glMultiTexCoord4iARB");
      glMultiTexCoord4ivARB    = (PFNGLMULTITEXCOORD4IVARBPROC)    wglGetProcAddress("glMultiTexCoord4ivARB");
      glMultiTexCoord4sARB     = (PFNGLMULTITEXCOORD4SARBPROC)     wglGetProcAddress("glMultiTexCoord4sARB");
      glMultiTexCoord4svARB    = (PFNGLMULTITEXCOORD4SVARBPROC)    wglGetProcAddress("glMultiTexCoord4svARB");

    return GL_TRUE;
  }

  #endif

  return GL_FALSE;
}

//####################################################################################################################
//
//                                           EXT CONVOLUTION FILTERS
//
//####################################################################################################################


PFNGLCONVOLUTIONFILTER1DEXTPROC       glConvolutionFilter1DEXT       = NULL;
PFNGLCONVOLUTIONFILTER2DEXTPROC       glConvolutionFilter2DEXT       = NULL;
PFNGLCONVOLUTIONPARAMETERFEXTPROC     glConvolutionParameterfEXT     = NULL;
PFNGLCONVOLUTIONPARAMETERFVEXTPROC    glConvolutionParameterfvEXT    = NULL;
PFNGLCONVOLUTIONPARAMETERIEXTPROC     glConvolutionParameteriEXT     = NULL;
PFNGLCONVOLUTIONPARAMETERIVEXTPROC    glConvolutionParameterivEXT    = NULL;
PFNGLCOPYCONVOLUTIONFILTER1DEXTPROC   glCopyConvolutionFilter1DEXT   = NULL;
PFNGLCOPYCONVOLUTIONFILTER2DEXTPROC   glCopyConvolutionFilter2DEXT   = NULL;
PFNGLGETCONVOLUTIONFILTEREXTPROC      glGetConvolutionFilterEXT      = NULL;
PFNGLGETCONVOLUTIONPARAMETERFVEXTPROC glGetConvolutionParameterfvEXT = NULL;
PFNGLGETCONVOLUTIONPARAMETERIVEXTPROC glGetConvolutionParameterivEXT = NULL;
PFNGLGETSEPARABLEFILTEREXTPROC        glGetSeparableFilterEXT        = NULL;
PFNGLSEPARABLEFILTER2DEXTPROC         glSeparableFilter2DEXT         = NULL;

GLboolean initConvolution(void) 
{
  char *extensions;
  int len,i;
  extensions=(char *) glGetString(GL_EXTENSIONS);
  len=strlen(extensions);
  for (i=0; i<len; i++)
    if (extensions[i]==' ') extensions[i]='\n';

 #ifdef WIN32

  if ( isInString(extensions,"GL_EXT_convolution") )
  {
    /*
    **** We Use wglGetProcAddress() To Get Address of GL Extensions At RunTime ****
    */

    glConvolutionFilter1DEXT              = (PFNGLCONVOLUTIONFILTER1DEXTPROC)       wglGetProcAddress("glConvolutionFilter1DEXT"); 
    glConvolutionFilter2DEXT              = (PFNGLCONVOLUTIONFILTER2DEXTPROC)       wglGetProcAddress("glConvolutionFilter2DEXT"); 
    glConvolutionParameterfEXT            = (PFNGLCONVOLUTIONPARAMETERFEXTPROC)     wglGetProcAddress("glConvolutionParameterfEXT"); 
    glConvolutionParameterfvEXT           = (PFNGLCONVOLUTIONPARAMETERFVEXTPROC)    wglGetProcAddress("glConvolutionParameterfvEXT"); 
    glConvolutionParameteriEXT            = (PFNGLCONVOLUTIONPARAMETERIEXTPROC)     wglGetProcAddress("glConvolutionParameteriEXT"); 
    glConvolutionParameterivEXT           = (PFNGLCONVOLUTIONPARAMETERIVEXTPROC)    wglGetProcAddress("glConvolutionParameterivEXT"); 
    glCopyConvolutionFilter1DEXT          = (PFNGLCOPYCONVOLUTIONFILTER1DEXTPROC)   wglGetProcAddress("glCopyConvolutionFilter1DEXT"); 
    glCopyConvolutionFilter2DEXT          = (PFNGLCOPYCONVOLUTIONFILTER2DEXTPROC)   wglGetProcAddress("glCopyConvolutionFilter2DEXT"); 
    glGetConvolutionFilterEXT             = (PFNGLGETCONVOLUTIONFILTEREXTPROC)      wglGetProcAddress("glGetConvolutionFilterEXT"); 
    glGetConvolutionParameterfvEXT        = (PFNGLGETCONVOLUTIONPARAMETERFVEXTPROC) wglGetProcAddress("glGetConvolutionParameterfvEXT"); 
    glGetConvolutionParameterivEXT        = (PFNGLGETCONVOLUTIONPARAMETERIVEXTPROC) wglGetProcAddress("glGetConvolutionParameterivEXT"); 
    glGetSeparableFilterEXT               = (PFNGLGETSEPARABLEFILTEREXTPROC)        wglGetProcAddress("glGetSeparableFilterEXT"); 
    glSeparableFilter2DEXT                = (PFNGLSEPARABLEFILTER2DEXTPROC)         wglGetProcAddress("glSeparableFilter2DEXT"); 

    return GL_TRUE;
  }

  #endif

  return GL_FALSE;
}


//####################################################################################################################
//
//                                        NVIDIA REGISTER COMBINERS
//
//####################################################################################################################

  PFNGLCOMBINERPARAMETERFVNVPROC               glCombinerParameterfvNV              = NULL;
  PFNGLCOMBINERPARAMETERFNVPROC                glCombinerParameterfNV               = NULL;
  PFNGLCOMBINERPARAMETERIVNVPROC               glCombinerParameterivNV              = NULL;
  PFNGLCOMBINERPARAMETERINVPROC                glCombinerParameteriNV               = NULL;
  PFNGLCOMBINERINPUTNVPROC                     glCombinerInputNV                    = NULL;
  PFNGLCOMBINEROUTPUTNVPROC                    glCombinerOutputNV                   = NULL;
  PFNGLFINALCOMBINERINPUTNVPROC                glFinalCombinerInputNV               = NULL;
  PFNGLGETCOMBINERINPUTPARAMETERFVNVPROC       glGetCombinerInputParameterfvNV      = NULL;
  PFNGLGETCOMBINERINPUTPARAMETERIVNVPROC       glGetCombinerInputParameterivNV      = NULL;
  PFNGLGETCOMBINEROUTPUTPARAMETERFVNVPROC      glGetCombinerOutputParameterfvNV     = NULL;
  PFNGLGETCOMBINEROUTPUTPARAMETERIVNVPROC      glGetCombinerOutputParameterivNV     = NULL;
  PFNGLGETFINALCOMBINERINPUTPARAMETERFVNVPROC  glGetFinalCombinerInputParameterfvNV = NULL;
  PFNGLGETFINALCOMBINERINPUTPARAMETERIVNVPROC  glGetFinalCombinerInputParameterivNV = NULL;


GLboolean initNV_Combiners(void) 
{
  char *extensions;
  int len,i;
  extensions=(char *) glGetString(GL_EXTENSIONS);
  len=strlen(extensions);
  for (i=0; i<len; i++)
    if (extensions[i]==' ') extensions[i]='\n';

  #ifdef WIN32

    if (isInString(extensions,"GL_NV_register_combiners"))
    {
      glCombinerParameterfvNV              = (PFNGLCOMBINERPARAMETERFVNVPROC)               wglGetProcAddress("glCombinerParameterfvNV"); 
      glCombinerParameterfNV               = (PFNGLCOMBINERPARAMETERFNVPROC)                wglGetProcAddress("glCombinerParameterfNV");
      glCombinerParameterivNV              = (PFNGLCOMBINERPARAMETERIVNVPROC)               wglGetProcAddress("glCombinerParameterivNV");
      glCombinerParameteriNV               = (PFNGLCOMBINERPARAMETERINVPROC)                wglGetProcAddress("glCombinerParameteriNV");
      glCombinerInputNV                    = (PFNGLCOMBINERINPUTNVPROC)                     wglGetProcAddress("glCombinerInputNV");
      glCombinerOutputNV                   = (PFNGLCOMBINEROUTPUTNVPROC)                    wglGetProcAddress("glCombinerOutputNV");
      glFinalCombinerInputNV               = (PFNGLFINALCOMBINERINPUTNVPROC)                wglGetProcAddress("glFinalCombinerInputNV");
      glGetCombinerInputParameterfvNV      = (PFNGLGETCOMBINERINPUTPARAMETERFVNVPROC)       wglGetProcAddress("glGetCombinerInputParameterfvNV");
      glGetCombinerInputParameterivNV      = (PFNGLGETCOMBINERINPUTPARAMETERIVNVPROC)       wglGetProcAddress("glGetCombinerInputParameterivNV");
      glGetCombinerOutputParameterfvNV     = (PFNGLGETCOMBINEROUTPUTPARAMETERFVNVPROC)      wglGetProcAddress("glGetCombinerOutputParameterfvNV");
      glGetCombinerOutputParameterivNV     = (PFNGLGETCOMBINEROUTPUTPARAMETERIVNVPROC)      wglGetProcAddress("glGetCombinerOutputParameterivNV");
      glGetFinalCombinerInputParameterfvNV = (PFNGLGETFINALCOMBINERINPUTPARAMETERFVNVPROC)  wglGetProcAddress("glGetFinalCombinerInputParameterfvNV");
      glGetFinalCombinerInputParameterivNV = (PFNGLGETFINALCOMBINERINPUTPARAMETERIVNVPROC)  wglGetProcAddress("glGetFinalCombinerInputParameterivNV");

    return GL_TRUE;
  }

  #endif

  return GL_FALSE;
}

//####################################################################################################################
//
//                                        NVIDIA VERTEX ARRAY RANGE
//
//####################################################################################################################

  PFNGLFLUSHVERTEXARRAYRANGENVPROC glFlushVertexArrayRangeNV = NULL;
  PFNGLVERTEXARRAYRANGENVPROC      glVertexArrayRangeNV      = NULL;

GLboolean init_NV_VA_Range(void) 
{
  char *extensions;
  int len,i;
  extensions=(char *) glGetString(GL_EXTENSIONS);
  len=strlen(extensions);
  for (i=0; i<len; i++)
    if (extensions[i]==' ') extensions[i]='\n';

  #ifdef WIN32

  if (isInString(extensions,"GL_NV_vertex_array_range"))
  {
    glFlushVertexArrayRangeNV = (PFNGLFLUSHVERTEXARRAYRANGENVPROC) wglGetProcAddress("glFlushVertexArrayRangeNV");
    glVertexArrayRangeNV      = (PFNGLVERTEXARRAYRANGENVPROC)      wglGetProcAddress("glVertexArrayRangeNV");
    return GL_TRUE;
  }

  #endif
  return GL_FALSE;
}

//####################################################################################################################
//
//                                        POINT PARAMETERS EXT
//
//####################################################################################################################

  PFNGLPOINTPARAMETERFEXTPROC   glPointParameterfEXT   = NULL;
  PFNGLPOINTPARAMETERFVEXTPROC  glPointParameterfvEXT  = NULL;
  PFNGLPOINTPARAMETERFSGISPROC  glPointParameterfSGIS  = NULL;
  PFNGLPOINTPARAMETERFVSGISPROC glPointParameterfvSGIS = NULL;

GLboolean init_Points(void) 
{
  char *extensions;
  int len,i;
  extensions=(char *) glGetString(GL_EXTENSIONS);
  len=strlen(extensions);
  for (i=0; i<len; i++)
    if (extensions[i]==' ') extensions[i]='\n';

  #ifdef WIN32

  if (isInString(extensions,"GL_EXT_point_parameters"))
  {
    glPointParameterfEXT   = (PFNGLPOINTPARAMETERFEXTPROC)   wglGetProcAddress("glPointParameterfEXT");
    glPointParameterfvEXT  = (PFNGLPOINTPARAMETERFVEXTPROC)  wglGetProcAddress("glPointParameterfvEXT");
    glPointParameterfSGIS  = (PFNGLPOINTPARAMETERFSGISPROC)  wglGetProcAddress("glPointParameterfSGIS");
    glPointParameterfvSGIS = (PFNGLPOINTPARAMETERFVSGISPROC) wglGetProcAddress("glPointParameterfvSGIS");
    return GL_TRUE;
  }
  #endif
  return GL_FALSE;
}


//####################################################################################################################
//
//             CHECKS TO SEE IF REQUIRED EXTENSIONS ARE IN STRING RETURNED FROM "glGetString(GL_EXTENSIONS);"
//
//####################################################################################################################

GLboolean isInString(char *string, const char *search) {
	int pos=0,i;
	int maxpos=strlen(search)-1;
	int len=strlen(string);	
	char *other;
	for (i=0; i<len; i++) {
		if ((i==0) || ((i>1) && string[i-1]=='\n')) {                   // New Extension Begins Here!
			other=&string[i];			
			pos=0;                                                    // Begin New Search
			while (string[i]!='\n') {                                 // Search Whole Extension-String
				if (string[i]==search[pos]) pos++;                  // Next Position
				if ((pos>maxpos) && string[i+1]=='\n') return GL_TRUE; // We Have A Winner!
				i++;
			}			
		}
	}	
	return GL_FALSE;                 // Sorry, Not Found!
}

//####################################################################################################################
//
//                                   FUNCTION TO CHECK OPENGL EXTENSIONS
//
//####################################################################################################################

GLboolean Check_GL_Ext( char *extString )
{
  int len,i;
  GLboolean Supported = GL_FALSE;
  char *extensions;

  if(strcmp(extString,"GL_ARB_multitexture") == 0)           Supported = initMultitexture();
  else if(strcmp(extString,"GL_NV_register_combiners") == 0) Supported = initNV_Combiners();
  else if(strcmp(extString,"GL_NV_vertex_array_range") == 0) Supported = init_NV_VA_Range();
  else if(strcmp(extString,"GL_VERSION_1_2") == 0)           Supported = initGL_1_2();
  else if(strcmp(extString,"GL_EXT_point_parameters") == 0)  Supported = init_Points();
  else if(strcmp(extString,"GL_EXT_convolution") == 0)       Supported = initConvolution();
  else {	
    extensions=(char *) glGetString(GL_EXTENSIONS);
    len=strlen(extensions);
    for (i=0; i<len; i++)
      if (extensions[i]==' ') extensions[i]='\n';
    if(isInString(extensions,extString)) Supported = GL_TRUE;
  }
  return Supported;
}

#endif